@@ -26,11 +26,6 @@
 #undef DBGC_CLASS
 #define DBGC_CLASS DBGC_AUTH
 
-extern DOM_SID global_sid_World;
-extern DOM_SID global_sid_Network;
-extern DOM_SID global_sid_Builtin_Guests;
-extern DOM_SID global_sid_Authenticated_Users;
-
 
 /****************************************************************************
  Create a UNIX user on demand.
@@ -50,40 +45,11 @@ static int smb_create_user(const char *domain, const char *unix_username, const
 	if (homedir)
 		all_string_sub(add_script, "%H", homedir, sizeof(pstring));
 	ret = smbrun(add_script,NULL);
-	DEBUG(3,("smb_create_user: Running the command `%s' gave %d\n",add_script,ret));
+	flush_pwnam_cache();
+	DEBUG(ret ? 0 : 3,("smb_create_user: Running the command `%s' gave %d\n",add_script,ret));
 	return ret;
 }
 
-/****************************************************************************
- Add and Delete UNIX users on demand, based on NTSTATUS codes.
- We don't care about RID's here so ignore.
-****************************************************************************/
-
-void auth_add_user_script(const char *domain, const char *username)
-{
-	uint32 rid;
-	/*
-	 * User validated ok against Domain controller.
-	 * If the admin wants us to try and create a UNIX
-	 * user on the fly, do so.
-	 */
-	
-	if ( *lp_adduser_script() )
-		smb_create_user(domain, username, NULL);
-	else {
-		DEBUG(10,("auth_add_user_script: no 'add user script'.  Asking winbindd\n"));
-		
-		/* should never get here is we a re a domain member running winbindd
-		   However, a host set for 'security = server' might run winbindd for 
-		   account allocation */
-		   
-		if ( !winbind_create_user(username, NULL) ) {
-			DEBUG(5,("auth_add_user_script: winbindd_create_user() failed\n"));
-			rid = 0;
-		}
-	}
-}
-
 /****************************************************************************
  Create a SAM_ACCOUNT - either by looking in the pdb, or by faking it up from
  unix info.
@@ -493,6 +459,8 @@ void debug_nt_user_token(int dbg_class, int dbg_lev, NT_USER_TOKEN *token)
 	for (i = 0; i < token->num_sids; i++)
 		DEBUGADDC(dbg_class, dbg_lev, ("SID[%3lu]: %s\n", (unsigned long)i, 
 					       sid_to_string(sid_str, &token->user_sids[i])));
+
+	dump_se_priv( dbg_class, dbg_lev, &token->privileges );
 }
 
 /****************************************************************************
@@ -583,9 +551,46 @@ static NTSTATUS create_nt_user_token(const DOM_SID *user_sid, const DOM_SID *gro
 			ptoken->num_sids--;
 		}
 	}
+
+	/* add privileges assigned to this user */
+
+	get_privileges_for_sids( &ptoken->privileges, ptoken->user_sids, ptoken->num_sids );
 	
 	debug_nt_user_token(DBGC_AUTH, 10, ptoken);
 	
+	if ((lp_log_nt_token_command() != NULL) &&
+	    (strlen(lp_log_nt_token_command()) > 0)) {
+		TALLOC_CTX *mem_ctx;
+		char *command;
+		fstring sidstr;
+		char *user_sidstr, *group_sidstr;
+
+		mem_ctx = talloc_init("setnttoken");
+		if (mem_ctx == NULL)
+			return NT_STATUS_NO_MEMORY;
+
+		sid_to_string(sidstr, &ptoken->user_sids[0]);
+		user_sidstr = talloc_strdup(mem_ctx, sidstr);
+
+		group_sidstr = talloc_strdup(mem_ctx, "");
+		for (i=1; i<ptoken->num_sids; i++) {
+			sid_to_string(sidstr, &ptoken->user_sids[i]);
+			group_sidstr = talloc_asprintf(mem_ctx, "%s %s",
+						       group_sidstr, sidstr);
+		}
+
+		command = SMB_STRDUP(lp_log_nt_token_command());
+		command = realloc_string_sub(command, "%s", user_sidstr);
+		command = realloc_string_sub(command, "%t", group_sidstr);
+		DEBUG(8, ("running command: [%s]\n", command));
+		if (smbrun(command, NULL) != 0) {
+			DEBUG(0, ("Could not log NT token\n"));
+			nt_status = NT_STATUS_ACCESS_DENIED;
+		}
+		talloc_destroy(mem_ctx);
+		SAFE_FREE(command);
+	}
+
 	*token = ptoken;
 
 	return nt_status;
@@ -657,47 +662,27 @@ static NTSTATUS get_user_groups(const char *username, uid_t uid, gid_t gid,
 
 	*n_groups = 0;
 	*groups   = NULL;
+
+	if (strchr(username, *lp_winbind_separator()) == NULL) {
+		NTSTATUS result;
+
+		become_root();
+		result = pdb_enum_group_memberships(username, gid, groups,
+						    unix_groups, n_groups);
+		unbecome_root();
+		return result;
+	}
+
+	/* We have the separator, this must be winbind */
 	
-	/* Try winbind first */
+	n_unix_groups = winbind_getgroups( username, unix_groups );
 
-	if ( strchr(username, *lp_winbind_separator()) ) {
-		n_unix_groups = winbind_getgroups( username, unix_groups );
-
-		DEBUG(10,("get_user_groups: winbind_getgroups(%s): result = %s\n", username, 
-			  n_unix_groups == -1 ? "FAIL" : "SUCCESS"));
+	DEBUG(10,("get_user_groups: winbind_getgroups(%s): result = %s\n",
+		  username,  n_unix_groups == -1 ? "FAIL" : "SUCCESS"));
 			  
-		if ( n_unix_groups == -1 )
-			return NT_STATUS_NO_SUCH_USER; /* what should this return value be? */	
-	}
-	else {
-		/* fallback to getgrouplist() */
-		
-		n_unix_groups = groups_max();
-		
-		if ((*unix_groups = SMB_MALLOC_ARRAY( gid_t, n_unix_groups ) ) == NULL) {
-			DEBUG(0, ("get_user_groups: Out of memory allocating unix group list\n"));
-			return NT_STATUS_NO_MEMORY;
-		}
-	
-		if (sys_getgrouplist(username, gid, *unix_groups, &n_unix_groups) == -1) {
-		
-			gid_t *groups_tmp;
-			
-			groups_tmp = SMB_REALLOC_ARRAY(*unix_groups, gid_t, n_unix_groups);
-			
-			if (!groups_tmp) {
-				SAFE_FREE(*unix_groups);
-				return NT_STATUS_NO_MEMORY;
-			}
-			*unix_groups = groups_tmp;
-
-			if (sys_getgrouplist(username, gid, *unix_groups, &n_unix_groups) == -1) {
-				DEBUG(0, ("get_user_groups: failed to get the unix group list\n"));
-				SAFE_FREE(*unix_groups);
-				return NT_STATUS_NO_SUCH_USER; /* what should this return value be? */
-			}
-		}
-	}
+	if ( n_unix_groups == -1 )
+		return NT_STATUS_NO_SUCH_USER; /* what should this return
+						* value be? */	
 
 	debug_unix_user_token(DBGC_CLASS, 5, uid, gid, n_unix_groups, *unix_groups);
 	
@@ -884,7 +869,7 @@ NTSTATUS make_server_info_pw(auth_serversupplied_info **server_info,
  Make (and fill) a user_info struct for a guest login.
 ***************************************************************************/
 
-NTSTATUS make_server_info_guest(auth_serversupplied_info **server_info)
+static NTSTATUS make_new_server_info_guest(auth_serversupplied_info **server_info)
 {
 	NTSTATUS nt_status;
 	SAM_ACCOUNT *sampass = NULL;
@@ -919,6 +904,49 @@ NTSTATUS make_server_info_guest(auth_serversupplied_info **server_info)
 	return nt_status;
 }
 
+static auth_serversupplied_info *copy_serverinfo(auth_serversupplied_info *src)
+{
+	auth_serversupplied_info *dst;
+
+	if (!NT_STATUS_IS_OK(make_server_info(&dst)))
+		return NULL;
+
+	dst->guest = src->guest;
+	dst->uid = src->uid;
+	dst->gid = src->gid;
+	dst->n_groups = src->n_groups;
+	if (src->n_groups != 0)
+		dst->groups = memdup(src->groups, sizeof(gid_t)*dst->n_groups);
+	else
+		dst->groups = NULL;
+	dst->ptok = dup_nt_token(src->ptok);
+	dst->user_session_key = data_blob(src->user_session_key.data,
+					  src->user_session_key.length);
+	dst->lm_session_key = data_blob(src->lm_session_key.data,
+					  src->lm_session_key.length);
+	pdb_copy_sam_account(src->sam_account, &dst->sam_account);
+	dst->pam_handle = NULL;
+	dst->unix_name = smb_xstrdup(src->unix_name);
+
+	return dst;
+}
+
+static auth_serversupplied_info *guest_info = NULL;
+
+BOOL init_guest_info(void)
+{
+	if (guest_info != NULL)
+		return True;
+
+	return NT_STATUS_IS_OK(make_new_server_info_guest(&guest_info));
+}
+
+NTSTATUS make_server_info_guest(auth_serversupplied_info **server_info)
+{
+	*server_info = copy_serverinfo(guest_info);
+	return (*server_info != NULL) ? NT_STATUS_OK : NT_STATUS_NO_MEMORY;
+}
+
 /***************************************************************************
  Purely internal function for make_server_info_info3
  Fill the sam account from getpwnam
@@ -930,6 +958,7 @@ static NTSTATUS fill_sam_account(TALLOC_CTX *mem_ctx,
 				 uid_t *uid, gid_t *gid,
 				 SAM_ACCOUNT **sam_account)
 {
+	NTSTATUS nt_status;
 	fstring dom_user, lower_username;
 	fstring real_username;
 	struct passwd *passwd;
@@ -964,7 +993,9 @@ static NTSTATUS fill_sam_account(TALLOC_CTX *mem_ctx,
 	DEBUG(5,("fill_sam_account: located username was [%s]\n",
 		*found_username));
 
-	return pdb_init_sam_pw(sam_account, passwd);
+	nt_status = pdb_init_sam_pw(sam_account, passwd);
+	passwd_free(&passwd);
+	return nt_status;
 }
 
 /****************************************************************************
@@ -996,7 +1027,7 @@ struct passwd *smb_getpwnam( char *domuser, fstring save_username, BOOL create )
 	if ( p ) {
 		fstring strip_username;
 
-		pw = Get_Pwnam( domuser );
+		pw = Get_Pwnam_alloc( domuser );
 		if ( pw ) {	
 			/* make sure we get the case of the username correct */
 			/* work around 'winbind use default domain = yes' */
@@ -1027,7 +1058,7 @@ struct passwd *smb_getpwnam( char *domuser, fstring save_username, BOOL create )
 	
 	/* just lookup a plain username */
 	
-	pw = Get_Pwnam(username);
+	pw = Get_Pwnam_alloc(username);
 		
 	/* Create local user if requested. */
 	
@@ -1036,8 +1067,8 @@ struct passwd *smb_getpwnam( char *domuser, fstring save_username, BOOL create )
 		if (username[strlen(username)-1] == '$')
 			return NULL;
 
-		auth_add_user_script(NULL, username);
-		pw = Get_Pwnam(username);
+		smb_create_user(NULL, username, NULL);
+		pw = Get_Pwnam_alloc(username);
 	}
 	
 	/* one last check for a valid passwd struct */
@@ -1125,12 +1156,20 @@ NTSTATUS make_server_info_info3(TALLOC_CTX *mem_ctx,
 
 	if (NT_STATUS_EQUAL(nt_status, NT_STATUS_NO_SUCH_USER)) {
 		DEBUG(3,("User %s does not exist, trying to add it\n", internal_username));
-		auth_add_user_script( nt_domain, sent_nt_username );
+		smb_create_user( nt_domain, sent_nt_username, NULL);
 		nt_status = fill_sam_account( mem_ctx, nt_domain, sent_nt_username, 
 			&found_username, &uid, &gid, &sam_account );
 	}
 	
+	/* if we still don't have a valid unix account check for 
+	  'map to gues = bad uid' */
+	  
 	if (!NT_STATUS_IS_OK(nt_status)) {
+		if ( lp_map_to_guest() == MAP_TO_GUEST_ON_BAD_UID ) {
+		 	make_server_info_guest(server_info); 
+			return NT_STATUS_OK;
+		}
+		
 		DEBUG(0, ("make_server_info_info3: pdb_init_sam failed!\n"));
 		return nt_status;
 	}
@@ -1288,6 +1327,9 @@ NTSTATUS make_server_info_info3(TALLOC_CTX *mem_ctx,
 		return nt_status;
 	}
 
+	(*server_info)->login_server = unistr2_tdup(mem_ctx, 
+						    &(info3->uni_logon_srv));
+
 	(*server_info)->ptok = token; 
 
 	SAFE_FREE(lgroupSIDs);
@@ -1301,11 +1343,12 @@ NTSTATUS make_server_info_info3(TALLOC_CTX *mem_ctx,
 		(*server_info)->user_session_key = data_blob(info3->user_sess_key, sizeof(info3->user_sess_key));
 	}
 
-	if (memcmp(info3->padding, zeros, sizeof(zeros)) == 0) {
+	if (memcmp(info3->lm_sess_key, zeros, 8) == 0) {
 		(*server_info)->lm_session_key = data_blob(NULL, 0);
 	} else {
-		(*server_info)->lm_session_key = data_blob(info3->padding, 16);
-	}
+		(*server_info)->lm_session_key = data_blob(info3->lm_sess_key, sizeof(info3->lm_sess_key));
+	} 
+
 	return NT_STATUS_OK;
 }
 
@@ -1386,12 +1429,13 @@ BOOL make_auth_methods(struct auth_context *auth_context, auth_methods **auth_me
 
 void delete_nt_token(NT_USER_TOKEN **pptoken)
 {
-    if (*pptoken) {
-	    NT_USER_TOKEN *ptoken = *pptoken;
-	    SAFE_FREE( ptoken->user_sids );
-	    ZERO_STRUCTP(ptoken);
-    }
-    SAFE_FREE(*pptoken);
+	if (*pptoken) {
+		NT_USER_TOKEN *ptoken = *pptoken;
+
+		SAFE_FREE( ptoken->user_sids );
+		ZERO_STRUCTP(ptoken);
+	}
+	SAFE_FREE(*pptoken);
 }
 
 /****************************************************************************
@@ -1405,21 +1449,70 @@ NT_USER_TOKEN *dup_nt_token(NT_USER_TOKEN *ptoken)
 	if (!ptoken)
 		return NULL;
 
-    if ((token = SMB_MALLOC_P(NT_USER_TOKEN)) == NULL)
-        return NULL;
+	if ((token = SMB_MALLOC_P(NT_USER_TOKEN)) == NULL)
+		return NULL;
 
-    ZERO_STRUCTP(token);
+	ZERO_STRUCTP(token);
+	
+	token->user_sids = (DOM_SID *)memdup( ptoken->user_sids, sizeof(DOM_SID) * ptoken->num_sids );
+	
+	if ( !token ) {
+		SAFE_FREE(token);
+		return NULL;
+	}
 
-    if ((token->user_sids = (DOM_SID *)memdup( ptoken->user_sids, sizeof(DOM_SID) * ptoken->num_sids )) == NULL) {
-        SAFE_FREE(token);
-        return NULL;
-    }
-
-    token->num_sids = ptoken->num_sids;
+	token->num_sids = ptoken->num_sids;
+	
+	/* copy the privileges; don't consider failure to be critical here */
+	
+	if ( !se_priv_copy( &token->privileges, &ptoken->privileges ) ) {
+		DEBUG(0,("dup_nt_token: Failure to copy SE_PRIV!.  Continuing with 0 privileges assigned.\n"));
+	}
 
 	return token;
 }
 
+/****************************************************************************
+ Check for a SID in an NT_USER_TOKEN
+****************************************************************************/
+
+BOOL nt_token_check_sid ( DOM_SID *sid, NT_USER_TOKEN *token )
+{
+	int i;
+	
+	if ( !sid || !token )
+		return False;
+	
+	for ( i=0; i<token->num_sids; i++ ) {
+		if ( sid_equal( sid, &token->user_sids[i] ) )
+			return True;
+	}
+
+	return False;
+}
+
+BOOL nt_token_check_domain_rid( NT_USER_TOKEN *token, uint32 rid ) 
+{
+	DOM_SID domain_sid;
+
+	/* if we are a domain member, the get the domain SID, else for 
+	   a DC or standalone server, use our own SID */
+
+	if ( lp_server_role() == ROLE_DOMAIN_MEMBER ) {
+		if ( !secrets_fetch_domain_sid( lp_workgroup(), &domain_sid ) ) {
+			DEBUG(1,("nt_token_check_domain_rid: Cannot lookup SID for domain [%s]\n",
+				lp_workgroup()));
+			return False;
+		}
+	} 
+	else
+		sid_copy( &domain_sid, get_global_sam_sid() );
+
+	sid_append_rid( &domain_sid, rid );
+	
+	return nt_token_check_sid( &domain_sid, token );\
+}
+
 /**
  * Verify whether or not given domain is trusted.
  *
@@ -1453,11 +1546,26 @@ BOOL is_trusted_domain(const char* dom_name)
 			return True;
 	}
 	else {
-		/* if winbindd is not up and we are a domain member) then we need to update the
-		   trustdom_cache ourselves */
+		NSS_STATUS result;
 
-		if ( !winbind_ping() )
-			update_trustdom_cache();
+		/* If winbind is around, ask it */
+
+		result = wb_is_trusted_domain(dom_name);
+
+		if (result == NSS_STATUS_SUCCESS) {
+			return True;
+		}
+
+		if (result == NSS_STATUS_NOTFOUND) {
+			/* winbind could not find the domain */
+			return False;
+		}
+
+		/* The only other possible result is that winbind is not up
+		   and running. We need to update the trustdom_cache
+		   ourselves */
+		
+		update_trustdom_cache();
 	}
 
 	/* now the trustdom cache should be available a DC could still
