@@ -44,7 +44,7 @@ static int clipfind(char **aret, int ret, char *tok);
 typedef struct file_info_struct file_info2;
 
 struct file_info_struct {
-	SMB_BIG_UINT size;
+	SMB_OFF_T size;
 	uint16 mode;
 	uid_t uid;
 	gid_t gid;
@@ -63,6 +63,7 @@ typedef struct {
 } stack;
 
 #define SEPARATORS " \t\n\r"
+extern time_t newer_than;
 extern struct cli_state *cli;
 
 /* These defines are for the do_setrattr routine, to indicate
@@ -87,9 +88,6 @@ static BOOL tar_reset=False;
 static BOOL tar_excl=True;
 /* use regular expressions for search on file names */
 static BOOL tar_re_search=False;
-#ifdef HAVE_REGEX_H
-regex_t *preg;
-#endif
 /* Do not dump anything, just calculate sizes */
 static BOOL dry_run=False;
 /* Dump files with System attribute */
@@ -797,11 +795,7 @@ static void do_tar(file_info *finfo)
 		DEBUG(5, ("...tar_re_search: %d\n", tar_re_search));
 
 		if ((!tar_re_search && clipfind(cliplist, clipn, exclaim)) ||
-#ifdef HAVE_REGEX_H
-				(tar_re_search && !regexec(preg, exclaim, 0, NULL, 0))) {
-#else
 				(tar_re_search && mask_match_list(exclaim, cliplist, clipn, True))) {
-#endif
 			DEBUG(3,("Skipping file %s\n", exclaim));
 			return;
 		}
@@ -1150,11 +1144,7 @@ static void do_tarput(void)
 		/* Well, now we have a header, process the file ...            */
 		/* Should we skip the file? We have the long name as well here */
 		skip = clipn && ((!tar_re_search && clipfind(cliplist, clipn, finfo.name) ^ tar_excl) ||
-#ifdef HAVE_REGEX_H
-					(tar_re_search && !regexec(preg, finfo.name, 0, NULL, 0)));
-#else
 					(tar_re_search && mask_match_list(finfo.name, cliplist, clipn, True)));
-#endif
 
 		DEBUG(5, ("Skip = %i, cliplist=%s, file=%s\n", skip, (cliplist?cliplist[0]:NULL), finfo.name));
 		if (skip) {
@@ -1632,7 +1622,6 @@ int tar_parseargs(int argc, char *argv[], const char *Optarg, int Optind)
 					return 0;
 				} else {
 					SMB_STRUCT_STAT stbuf;
-					extern time_t newer_than;
 	
 					if (sys_stat(argv[Optind], &stbuf) == 0) {
 						newer_than = stbuf.st_mtime;
@@ -1748,25 +1737,6 @@ int tar_parseargs(int argc, char *argv[], const char *Optarg, int Optind)
 	}
 
 	if (Optind+1<argc && tar_re_search) {  /* Doing regular expression seaches */
-#ifdef HAVE_REGEX_H
-		int errcode;
-
-		if ((preg = (regex_t *)SMB_MALLOC(65536)) == NULL) {
-
-			DEBUG(0, ("Could not allocate buffer for regular expression search\n"));
-			return;
-		}
-
-		if (errcode = regcomp(preg, argv[Optind + 1], REG_EXTENDED)) {
-			char errstr[1024];
-			size_t errlen;
-
-			errlen = regerror(errcode, preg, errstr, sizeof(errstr) - 1);
-			DEBUG(0, ("Could not compile pattern buffer for re search: %s\n%s\n", argv[Optind + 1], errstr));
-			return;
-		}
-#endif
-
 		clipn=argc-Optind-1;
 		cliplist=argv+Optind+1;
 		newOptind += clipn;
@@ -1791,11 +1761,7 @@ int tar_parseargs(int argc, char *argv[], const char *Optarg, int Optind)
 		}
 
 	} else {
-		if (tar_type=='c' && (dry_run || strcmp(argv[Optind], "/dev/null")==0)) {
-			if (!dry_run) {
-				DEBUG(0,("Output is /dev/null, assuming dry_run\n"));
-				dry_run = True;
-			}
+		if (tar_type=='c' && dry_run) {
 			tarhandle=-1;
 		} else if ((tar_type=='x' && (tarhandle = sys_open(argv[Optind], O_RDONLY, 0)) == -1)
 					|| (tar_type=='c' && (tarhandle=sys_creat(argv[Optind], 0644)) < 0)) {
