@@ -27,7 +27,7 @@
    overlap on the wire. This size gives us a nice read/write size, which
    will be a multiple of the page size on almost any system */
 #define CLI_BUFFER_SIZE (0xFFFF)
-
+#define CLI_MAX_LARGE_READX_SIZE (127*1024)
 
 /*
  * These definitions depend on smb.h
@@ -57,6 +57,27 @@ struct print_job_info
 	time_t t;
 };
 
+struct rpc_pipe_client {
+	TALLOC_CTX *mem_ctx;
+
+	struct cli_state *cli;
+
+	int pipe_idx;
+	uint16 fnum;
+
+	int pipe_auth_flags;
+
+	NTLMSSP_STATE *ntlmssp_pipe_state;
+	const char *user_name;
+	const char *domain;
+	struct pwd_info pwd;
+
+	struct netsec_auth_struct auth_info;
+
+	uint16 max_xmit_frag;
+	uint16 max_recv_frag;
+};
+
 struct cli_state {
 	int port;
 	int fd;
@@ -106,6 +127,7 @@ struct cli_state {
 	int initialised;
 	int win95;
 	uint32 capabilities;
+	BOOL dfsroot;
 
 	TALLOC_CTX *mem_ctx;
 
@@ -123,27 +145,22 @@ struct cli_state {
 					      of the pipe we're talking to, 
 					      if any */
 
-	uint16 nt_pipe_fnum;               /* Pipe handle. */
+	struct rpc_pipe_client pipes[PI_MAX_PIPES];
 
 	/* Secure pipe parameters */
 	int pipe_auth_flags;
 
-	uint16 saved_netlogon_pipe_fnum;   /* The "first" pipe to get
-                                              the session key for the
-                                              schannel. */
-	struct netsec_auth_struct auth_info;
-
-	NTLMSSP_STATE *ntlmssp_pipe_state;
-
+	struct rpc_pipe_client netlogon_pipe;  /* The "first" pipe to get
+						  the session key for the
+						  schannel. */
 	unsigned char sess_key[16];        /* Current session key. */
 	DOM_CRED clnt_cred;                /* Client credential. */
 	fstring mach_acct;                 /* MYNAME$. */
 	fstring srv_name_slash;            /* \\remote server. */
 	fstring clnt_name_slash;           /* \\local client. */
-	uint16 max_xmit_frag;
-	uint16 max_recv_frag;
 
 	BOOL use_kerberos;
+	BOOL fallback_after_kerberos;
 	BOOL use_spnego;
 
 	BOOL use_oplocks; /* should we use oplocks? */
