@@ -1,6 +1,5 @@
 /* 
-   Unix SMB/Netbios implementation.
-   Version 1.9.
+   Unix SMB/CIFS implementation.
    NT error code constants
    Copyright (C) Andrew Tridgell              1992-2000
    Copyright (C) John H Terpstra              1996-2000
@@ -28,16 +27,18 @@
 /* Win32 Status codes. */
 
 #define STATUS_BUFFER_OVERFLOW            NT_STATUS(0x80000005)
+#define STATUS_NO_MORE_FILES              NT_STATUS(0x80000006)
 #define NT_STATUS_NO_MORE_ENTRIES         NT_STATUS(0x8000001a)
 
-#define ERROR_INVALID_PARAMETER		  NT_STATUS(0x0057)
-#define ERROR_INSUFFICIENT_BUFFER	  NT_STATUS(0x007a)
 #define STATUS_MORE_ENTRIES               NT_STATUS(0x0105)
 #define STATUS_SOME_UNMAPPED              NT_STATUS(0x0107)
+#define ERROR_INVALID_PARAMETER		  NT_STATUS(0x0057)
+#define ERROR_INSUFFICIENT_BUFFER	  NT_STATUS(0x007a)
+#define STATUS_NOTIFY_ENUM_DIR            NT_STATUS(0x010c)
 #define ERROR_INVALID_DATATYPE		  NT_STATUS(0x070c)
 
-#define STATUS_MORE_ENTRIES               NT_STATUS(0x0105)
-#define STATUS_NOTIFY_ENUM_DIR            NT_STATUS(0x010c)
+/* Special "invalid" NT status code. */
+#define NT_STATUS_INVALID		  NT_STATUS(0xFFFFFFFF)
 
 /* Win32 Error codes extracted using a loop in smbclient then printing a
    netmon sniff to a file. */
@@ -561,6 +562,7 @@
 #define NT_STATUS_TOO_MANY_LINKS NT_STATUS(0xC0000000 | 0x0265)
 #define NT_STATUS_QUOTA_LIST_INCONSISTENT NT_STATUS(0xC0000000 | 0x0266)
 #define NT_STATUS_FILE_IS_OFFLINE NT_STATUS(0xC0000000 | 0x0267)
+#define NT_STATUS_NOT_A_REPARSE_POINT NT_STATUS(0xC0000000 | 0x0275)
 #define NT_STATUS_NO_SUCH_JOB NT_STATUS(0xC0000000 | 0xEDE) /* scheduler */
 
 #endif /* _NTERR_H */
