@@ -1,9 +1,10 @@
 /* 
    Unix SMB/CIFS implementation.
    SMB parameters and setup
-   Copyright (C) Andrew Tridgell 1992-1997
-   Copyright (C) Luke Kenneth Casson Leighton 1996-1997
-   Copyright (C) Paul Ashton 1997
+   Copyright (C) Andrew Tridgell               1992-1997
+   Copyright (C) Luke Kenneth Casson Leighton  1996-1997
+   Copyright (C) Paul Ashton                   1997
+   Copyright (C) Gerald (Jerry) Carter         2005
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -25,21 +26,14 @@
 
 #include "rpc_misc.h"
 
-enum SID_NAME_USE
-{
-	SID_NAME_USE_NONE = 0,/* NOTUSED */
-	SID_NAME_USER    = 1, /* user */
-	SID_NAME_DOM_GRP = 2, /* domain group */
-	SID_NAME_DOMAIN  = 3, /* domain: don't know what this is */
-	SID_NAME_ALIAS   = 4, /* local group */
-	SID_NAME_WKN_GRP = 5, /* well-known group */
-	SID_NAME_DELETED = 6, /* deleted account: needed for c2 rating */
-	SID_NAME_INVALID = 7, /* invalid account */
-	SID_NAME_UNKNOWN = 8  /* oops. */
-};
-
 /* Opcodes available on PIPE_LSARPC */
 
+#if 0	/* UNIMPLEMENTED */
+
+#define LSA_LOOKUPSIDS2		0x39
+
+#endif
+
 #define LSA_CLOSE              0x00
 #define LSA_DELETE             0x01
 #define LSA_ENUM_PRIVS         0x02
@@ -52,11 +46,11 @@ enum SID_NAME_USE
 #define LSA_CLEARAUDITLOG      0x09
 #define LSA_CREATEACCOUNT      0x0a
 #define LSA_ENUM_ACCOUNTS      0x0b
-#define LSA_CREATETRUSTDOM     0x0c
+#define LSA_CREATETRUSTDOM     0x0c	/* TODO: implement this one  -- jerry */
 #define LSA_ENUMTRUSTDOM       0x0d
 #define LSA_LOOKUPNAMES        0x0e
 #define LSA_LOOKUPSIDS         0x0f
-#define LSA_CREATESECRET       0x10
+#define LSA_CREATESECRET       0x10	/* TODO: implement this one  -- jerry */
 #define LSA_OPENACCOUNT	       0x11
 #define LSA_ENUMPRIVSACCOUNT   0x12
 #define LSA_ADDPRIVS           0x13
@@ -65,27 +59,30 @@ enum SID_NAME_USE
 #define LSA_SETQUOTAS          0x16
 #define LSA_GETSYSTEMACCOUNT   0x17
 #define LSA_SETSYSTEMACCOUNT   0x18
-#define LSA_OPENTRUSTDOM       0x19
-#define LSA_QUERYTRUSTDOM      0x1a
+#define LSA_OPENTRUSTDOM       0x19	/* TODO: implement this one  -- jerry */
+#define LSA_QUERYTRUSTDOMINFO  0x1a
 #define LSA_SETINFOTRUSTDOM    0x1b
-#define LSA_OPENSECRET         0x1c
-#define LSA_SETSECRET          0x1d
+#define LSA_OPENSECRET         0x1c	/* TODO: implement this one  -- jerry */
+#define LSA_SETSECRET          0x1d	/* TODO: implement this one  -- jerry */
 #define LSA_QUERYSECRET        0x1e
 #define LSA_LOOKUPPRIVVALUE    0x1f
 #define LSA_LOOKUPPRIVNAME     0x20
 #define LSA_PRIV_GET_DISPNAME  0x21
-#define LSA_DELETEOBJECT       0x22
-#define LSA_ENUMACCTWITHRIGHT  0x23
+#define LSA_DELETEOBJECT       0x22	/* TODO: implement this one  -- jerry */
+#define LSA_ENUMACCTWITHRIGHT  0x23	/* TODO: implement this one  -- jerry */
 #define LSA_ENUMACCTRIGHTS     0x24
 #define LSA_ADDACCTRIGHTS      0x25
 #define LSA_REMOVEACCTRIGHTS   0x26
-#define LSA_QUERYTRUSTDOMINFO  0x27
+#define LSA_QUERYTRUSTDOMINFOBYSID  0x27
 #define LSA_SETTRUSTDOMINFO    0x28
 #define LSA_DELETETRUSTDOM     0x29
 #define LSA_STOREPRIVDATA      0x2a
 #define LSA_RETRPRIVDATA       0x2b
 #define LSA_OPENPOLICY2        0x2c
 #define LSA_UNK_GET_CONNUSER   0x2d /* LsaGetConnectedCredentials ? */
+#define LSA_QUERYINFO2         0x2e
+#define LSA_QUERYTRUSTDOMINFOBYNAME 0x30
+#define LSA_OPENTRUSTDOMBYNAME 0x37
 
 /* XXXX these are here to get a compile! */
 #define LSA_LOOKUPRIDS      0xFD
@@ -213,7 +210,7 @@ typedef struct lsa_r_open_pol2_info
                             POLICY_VIEW_AUDIT_INFORMATION    |\
                             POLICY_GET_PRIVATE_INFORMATION)
 
-#define POLICY_WRITE      ( STANDARD_RIGHTS_WRITE_ACCESS     |\
+#define POLICY_WRITE      ( STD_RIGHT_READ_CONTROL_ACCESS     |\
                             POLICY_TRUST_ADMIN               |\
                             POLICY_CREATE_ACCOUNT            |\
                             POLICY_CREATE_SECRET             |\
@@ -274,32 +271,70 @@ typedef struct lsa_r_query_info
 
 } LSA_R_QUERY_INFO;
 
-/* LSA_Q_ENUM_TRUST_DOM - LSA enumerate trusted domains */
-typedef struct lsa_enum_trust_dom_info
+/* LSA_DNS_DOM_INFO - DNS domain info - info class 12*/
+typedef struct lsa_dns_dom_info
 {
-	POLICY_HND pol; /* policy handle */
-    uint32 enum_context; /* enumeration context handle */
-    uint32 preferred_len; /* preferred maximum length */
+	UNIHDR  hdr_nb_dom_name; /* netbios domain name */
+	UNIHDR  hdr_dns_dom_name;
+	UNIHDR  hdr_forest_name;
 
+	struct uuid dom_guid; /* domain GUID */
+
+	UNISTR2 uni_nb_dom_name;
+	UNISTR2 uni_dns_dom_name;
+	UNISTR2 uni_forest_name;
+
+	uint32 ptr_dom_sid;
+	DOM_SID2   dom_sid; /* domain SID */
+} LSA_DNS_DOM_INFO;
+
+typedef union lsa_info2_union
+{
+	LSA_DNS_DOM_INFO dns_dom_info;
+} LSA_INFO2_UNION;
+
+/* LSA_Q_QUERY_INFO2 - LSA query info */
+typedef struct lsa_q_query_info2
+{
+	POLICY_HND pol;    /* policy handle */
+	uint16 info_class; /* info class */
+} LSA_Q_QUERY_INFO2;
+
+typedef struct lsa_r_query_info2
+{
+	uint32 ptr;    /* pointer to info struct */
+	uint16 info_class;
+	LSA_INFO2_UNION info; /* so far the only one */
+	NTSTATUS status;
+} LSA_R_QUERY_INFO2;
+
+/*******************************************************/
+
+typedef struct {
+	POLICY_HND pol; 
+	uint32 enum_context; 
+	uint32 preferred_len; 	/* preferred maximum length */
 } LSA_Q_ENUM_TRUST_DOM;
 
-/* LSA_R_ENUM_TRUST_DOM - response to LSA enumerate trusted domains */
-typedef struct lsa_r_enum_trust_dom_info
-{
-	uint32 enum_context; /* enumeration context handle */
-	uint32 num_domains; /* number of domains */
-	uint32 ptr_enum_domains; /* buffer pointer to num domains */
+typedef struct {
+	UNISTR4	name;
+	DOM_SID2 *sid;
+} DOMAIN_INFO;
 
-	/* this lot is only added if ptr_enum_domains is non-NULL */
-	uint32 num_domains2; /* number of domains */
-	UNIHDR2 *hdr_domain_name;
-	UNISTR2 *uni_domain_name;
-	DOM_SID2 *domain_sid;
-
-	NTSTATUS status; /* return code */
+typedef struct {
+	uint32 count;
+	DOMAIN_INFO *domains;
+} DOMAIN_LIST;
 
+typedef struct {
+	uint32 enum_context;
+	uint32 count;
+	DOMAIN_LIST *domlist;
+	NTSTATUS status; 
 } LSA_R_ENUM_TRUST_DOM;
 
+/*******************************************************/
+
 /* LSA_Q_CLOSE */
 typedef struct lsa_q_close_info
 {
@@ -359,8 +394,8 @@ typedef struct lsa_trans_name_info
 
 } LSA_TRANS_NAME;
 
-/* This number purly arbitary - just to prevent a client from requesting large amounts of memory */
-#define MAX_LOOKUP_SIDS 256
+/* This number is based on Win2k and later maximum response allowed */
+#define MAX_LOOKUP_SIDS 20480
 
 /* LSA_TRANS_NAME_ENUM - LSA Translated Name Enumeration container */
 typedef struct lsa_trans_name_enum_info
@@ -392,7 +427,7 @@ typedef struct lsa_q_lookup_sids
 	POLICY_HND          pol; /* policy handle */
 	LSA_SID_ENUM        sids;
 	LSA_TRANS_NAME_ENUM names;
-	LOOKUP_LEVEL        level;
+	uint16              level;
 	uint32              mapped_count;
 
 } LSA_Q_LOOKUP_SIDS;
@@ -442,25 +477,6 @@ typedef struct lsa_r_lookup_names
 	NTSTATUS status; /* return code */
 } LSA_R_LOOKUP_NAMES;
 
-/* This is probably a policy handle but at the moment we
-   never read it - so use a dummy struct. */
-
-typedef struct lsa_q_open_secret
-{
-	uint32 dummy;
-} LSA_Q_OPEN_SECRET;
-
-/* We always return "not found" at present - so just marshal the minimum. */
-
-typedef struct lsa_r_open_secret
-{
-	uint32 dummy1;
-	uint32 dummy2;
-	uint32 dummy3;
-	uint32 dummy4;
-	NTSTATUS status;
-} LSA_R_OPEN_SECRET;
-
 typedef struct lsa_enum_priv_entry
 {
 	UNIHDR hdr_name;
@@ -490,6 +506,55 @@ typedef struct lsa_r_enum_privs
 	NTSTATUS status;
 } LSA_R_ENUM_PRIVS;
 
+/* LSA_Q_ENUM_ACCT_RIGHTS - LSA enum account rights */
+typedef struct
+{
+	POLICY_HND pol; /* policy handle */
+	DOM_SID2 sid;
+} LSA_Q_ENUM_ACCT_RIGHTS;
+
+/* LSA_R_ENUM_ACCT_RIGHTS - LSA enum account rights */
+typedef struct
+{
+	uint32 count;
+	UNISTR4_ARRAY *rights;
+	NTSTATUS status;
+} LSA_R_ENUM_ACCT_RIGHTS;
+
+
+/* LSA_Q_ADD_ACCT_RIGHTS - LSA add account rights */
+typedef struct
+{
+	POLICY_HND pol; /* policy handle */
+	DOM_SID2 sid;
+	uint32 count;
+	UNISTR4_ARRAY *rights;
+} LSA_Q_ADD_ACCT_RIGHTS;
+
+/* LSA_R_ADD_ACCT_RIGHTS - LSA add account rights */
+typedef struct
+{
+	NTSTATUS status;
+} LSA_R_ADD_ACCT_RIGHTS;
+
+
+/* LSA_Q_REMOVE_ACCT_RIGHTS - LSA remove account rights */
+typedef struct
+{
+	POLICY_HND pol; /* policy handle */
+	DOM_SID2 sid;
+	uint32 removeall;
+	uint32 count;
+	UNISTR4_ARRAY *rights;
+} LSA_Q_REMOVE_ACCT_RIGHTS;
+
+/* LSA_R_REMOVE_ACCT_RIGHTS - LSA remove account rights */
+typedef struct
+{
+	NTSTATUS status;
+} LSA_R_REMOVE_ACCT_RIGHTS;
+
+
 /* LSA_Q_PRIV_GET_DISPNAME - LSA get privilege display name */
 typedef struct lsa_q_priv_get_dispname
 {
@@ -555,6 +620,20 @@ typedef struct lsa_r_unk_get_connuser
 } LSA_R_UNK_GET_CONNUSER;
 
 
+typedef struct lsa_q_createaccount
+{
+	POLICY_HND pol; /* policy handle */
+	DOM_SID2 sid;
+	uint32 access; /* access */
+} LSA_Q_CREATEACCOUNT;
+
+typedef struct lsa_r_createaccount
+{
+	POLICY_HND pol; /* policy handle */
+	NTSTATUS status;
+} LSA_R_CREATEACCOUNT;
+
+
 typedef struct lsa_q_openaccount
 {
 	POLICY_HND pol; /* policy handle */
@@ -573,26 +652,6 @@ typedef struct lsa_q_enumprivsaccount
 	POLICY_HND pol; /* policy handle */
 } LSA_Q_ENUMPRIVSACCOUNT;
 
-
-typedef struct LUID
-{
-	uint32 low;
-	uint32 high;
-} LUID;
-
-typedef struct LUID_ATTR
-{
-	LUID luid;
-	uint32 attr;
-} LUID_ATTR ;
-
-typedef struct privilege_set
-{
-	uint32 count;
-	uint32 control;
-	LUID_ATTR *set;
-} PRIVILEGE_SET;
-
 typedef struct lsa_r_enumprivsaccount
 {
 	uint32 ptr;
@@ -624,20 +683,20 @@ typedef struct lsa_r_setsystemaccount
 	NTSTATUS status;
 } LSA_R_SETSYSTEMACCOUNT;
 
+typedef struct {
+	UNIHDR hdr;
+	UNISTR2 unistring;
+} LSA_STRING;
 
-typedef struct lsa_q_lookupprivvalue
-{
+typedef struct {
 	POLICY_HND pol; /* policy handle */
-	UNIHDR hdr_right;
-	UNISTR2 uni2_right;
-} LSA_Q_LOOKUPPRIVVALUE;
+	LSA_STRING privname;
+} LSA_Q_LOOKUP_PRIV_VALUE;
 
-typedef struct lsa_r_lookupprivvalue
-{
+typedef struct {
 	LUID luid;
 	NTSTATUS status;
-} LSA_R_LOOKUPPRIVVALUE;
-
+} LSA_R_LOOKUP_PRIV_VALUE;
 
 typedef struct lsa_q_addprivs
 {
@@ -666,7 +725,236 @@ typedef struct lsa_r_removeprivs
 	NTSTATUS status;
 } LSA_R_REMOVEPRIVS;
 
+/*******************************************************/
+#if 0 /* jerry, I think this not correct - gd */
+typedef struct {
+	POLICY_HND	handle;
+	uint32		count;	/* ??? this is what ethereal calls it */
+	DOM_SID		sid;
+} LSA_Q_OPEN_TRUSTED_DOMAIN;
+#endif
+
+/* LSA_Q_OPEN_TRUSTED_DOMAIN - LSA Query Open Trusted Domain */
+typedef struct lsa_q_open_trusted_domain
+{
+	POLICY_HND 	pol; 	/* policy handle */
+	DOM_SID2 	sid;	/* domain sid */
+	uint32 	access_mask;	/* access mask */
+	
+} LSA_Q_OPEN_TRUSTED_DOMAIN;
+
+/* LSA_R_OPEN_TRUSTED_DOMAIN - response to LSA Query Open Trusted Domain */
+typedef struct {
+	POLICY_HND	handle;	/* trustdom policy handle */
+	NTSTATUS	status; /* return code */
+} LSA_R_OPEN_TRUSTED_DOMAIN;
+
+
+/*******************************************************/
+
+typedef struct {
+	POLICY_HND	handle;	
+	UNISTR4		secretname;
+	uint32		access;
+} LSA_Q_OPEN_SECRET;
+
+typedef struct {
+	POLICY_HND	handle;
+	NTSTATUS	status;
+} LSA_R_OPEN_SECRET;
+
+
+/*******************************************************/
+
+typedef struct {
+	POLICY_HND	handle;
+} LSA_Q_DELETE_OBJECT;
+
+typedef struct {
+	NTSTATUS 	status;
+} LSA_R_DELETE_OBJECT;
+
+
+/*******************************************************/
+
+typedef struct {
+	POLICY_HND      handle;
+	UNISTR4         secretname;
+	uint32          access;
+} LSA_Q_CREATE_SECRET;
+
+typedef struct {
+	POLICY_HND      handle;
+	NTSTATUS        status;
+} LSA_R_CREATE_SECRET;
+
+
+/*******************************************************/
+
+typedef struct {
+	POLICY_HND	handle;	
+	UNISTR4		secretname;
+	uint32		access;
+} LSA_Q_CREATE_TRUSTED_DOMAIN;
+
+typedef struct {
+	POLICY_HND	handle;
+	NTSTATUS	status;
+} LSA_R_CREATE_TRUSTED_DOMAIN;
+
+
+/*******************************************************/
+
+typedef struct {
+	uint32	size;	/* size is written on the wire twice so I 
+			   can only assume that one is supposed to 
+			   be a max length and one is a size */
+	UNISTR2	*data;	/* not really a UNICODE string but the parsing 
+			   is the same */
+} LSA_DATA_BLOB;
+
+typedef struct {
+	POLICY_HND	handle;	
+	LSA_DATA_BLOB   *old_value;
+	LSA_DATA_BLOB	*new_value;
+} LSA_Q_SET_SECRET;
+
+typedef struct {
+	NTSTATUS	status;
+} LSA_R_SET_SECRET;
+
+/* LSA_Q_QUERY_TRUSTED_DOMAIN_INFO - LSA query trusted domain info */
+typedef struct lsa_query_trusted_domain_info
+{
+	POLICY_HND	pol; 		/* policy handle */
+	uint16		info_class; 	/* info class */
+
+} LSA_Q_QUERY_TRUSTED_DOMAIN_INFO;
+
+/* LSA_Q_QUERY_TRUSTED_DOMAIN_INFO_BY_SID - LSA query trusted domain info */
+typedef struct lsa_query_trusted_domain_info_by_sid
+{
+	POLICY_HND 	pol; 		/* policy handle */
+	DOM_SID2 	dom_sid;	/* domain sid */
+	uint16		info_class; 	/* info class */
+	
+} LSA_Q_QUERY_TRUSTED_DOMAIN_INFO_BY_SID;
+
+/* LSA_Q_QUERY_TRUSTED_DOMAIN_INFO_BY_NAME - LSA query trusted domain info */
+typedef struct lsa_query_trusted_domain_info_by_name
+{
+	POLICY_HND 	pol; 		/* policy handle */
+	LSA_STRING 	domain_name;	/* domain name */
+	uint16 		info_class; 	/* info class */
+	
+} LSA_Q_QUERY_TRUSTED_DOMAIN_INFO_BY_NAME;
+
+typedef struct trusted_domain_info_name {
+	LSA_STRING 	netbios_name; 
+} TRUSTED_DOMAIN_INFO_NAME;
+
+typedef struct trusted_domain_info_posix_offset {
+	uint32 		posix_offset;
+} TRUSTED_DOMAIN_INFO_POSIX_OFFSET;
+
+typedef struct lsa_data_buf {
+	uint32 size;
+	uint32 offset;
+	uint32 length;
+	uint8 *data;
+} LSA_DATA_BUF;
+
+typedef struct lsa_data_buf_hdr {
+	uint32 length;
+	uint32 size;
+	uint32 data_ptr;
+} LSA_DATA_BUF_HDR;
+
+
+typedef struct lsa_data_buf2 {
+	uint32 size;
+	uint8 *data;
+} LSA_DATA_BUF2;
+
+typedef struct trusted_domain_info_password {
+	uint32 ptr_password;
+	uint32 ptr_old_password;
+	LSA_DATA_BUF_HDR password_hdr;
+	LSA_DATA_BUF_HDR old_password_hdr;
+	LSA_DATA_BUF password;
+	LSA_DATA_BUF old_password;
+} TRUSTED_DOMAIN_INFO_PASSWORD;
+
+typedef struct trusted_domain_info_basic {
+	LSA_STRING 	netbios_name;
+	DOM_SID2 	sid;
+} TRUSTED_DOMAIN_INFO_BASIC;
+
+typedef struct trusted_domain_info_ex {
+	LSA_STRING 	domain_name;
+	LSA_STRING 	netbios_name;
+	DOM_SID2 	sid;
+	uint32 		trust_direction;
+	uint32 		trust_type;
+	uint32 		trust_attributes;
+} TRUSTED_DOMAIN_INFO_EX;
+
+typedef struct trust_domain_info_buffer {
+	NTTIME 		last_update_time;
+	uint32 		secret_type;
+	LSA_DATA_BUF2 	data;
+} LSA_TRUSTED_DOMAIN_INFO_BUFFER;
+
+typedef struct trusted_domain_info_auth_info {
+	uint32 incoming_count;
+	LSA_TRUSTED_DOMAIN_INFO_BUFFER incoming_current_auth_info;
+	LSA_TRUSTED_DOMAIN_INFO_BUFFER incoming_previous_auth_info;
+	uint32 outgoing_count;
+	LSA_TRUSTED_DOMAIN_INFO_BUFFER outgoing_current_auth_info;
+	LSA_TRUSTED_DOMAIN_INFO_BUFFER outgoing_previous_auth_info;
+} TRUSTED_DOMAIN_INFO_AUTH_INFO;
+
+typedef struct trusted_domain_info_full_info {
+	TRUSTED_DOMAIN_INFO_EX 		info_ex;
+	TRUSTED_DOMAIN_INFO_POSIX_OFFSET posix_offset;
+	TRUSTED_DOMAIN_INFO_AUTH_INFO 	auth_info;
+} TRUSTED_DOMAIN_INFO_FULL_INFO;
+
+typedef struct trusted_domain_info_11 {
+	TRUSTED_DOMAIN_INFO_EX 		info_ex;
+	LSA_DATA_BUF2 			data1;
+} TRUSTED_DOMAIN_INFO_11;
+
+typedef struct trusted_domain_info_all {
+	TRUSTED_DOMAIN_INFO_EX 		info_ex;
+	LSA_DATA_BUF2 			data1;
+	TRUSTED_DOMAIN_INFO_POSIX_OFFSET posix_offset;
+	TRUSTED_DOMAIN_INFO_AUTH_INFO 	auth_info;
+} TRUSTED_DOMAIN_INFO_ALL;
+
+/* LSA_TRUSTED_DOMAIN_INFO */
+typedef union lsa_trusted_domain_info
+{
+	uint16 					info_class;
+	TRUSTED_DOMAIN_INFO_NAME		name;
+	/* deprecated - gd
+	TRUSTED_DOMAIN_INFO_CONTROLLERS_INFO	controllers; */
+	TRUSTED_DOMAIN_INFO_POSIX_OFFSET	posix_offset;
+	TRUSTED_DOMAIN_INFO_PASSWORD		password;
+	TRUSTED_DOMAIN_INFO_BASIC		basic;
+	TRUSTED_DOMAIN_INFO_EX			info_ex;
+	TRUSTED_DOMAIN_INFO_AUTH_INFO		auth_info;
+	TRUSTED_DOMAIN_INFO_FULL_INFO		full_info;
+	TRUSTED_DOMAIN_INFO_11			info11;
+	TRUSTED_DOMAIN_INFO_ALL			info_all;
+
+} LSA_TRUSTED_DOMAIN_INFO;
+
+/* LSA_R_QUERY_TRUSTED_DOMAIN_INFO - LSA query trusted domain info */
+typedef struct r_lsa_query_trusted_domain_info
+{
+	LSA_TRUSTED_DOMAIN_INFO *info;
+	NTSTATUS status;
+} LSA_R_QUERY_TRUSTED_DOMAIN_INFO;
 
 #endif /* _RPC_LSA_H */
-
-
