@@ -4,7 +4,7 @@
    Copyright (C) Andrew Tridgell 1992-1997
    Copyright (C) Luke Kenneth Casson Leighton 1996-1997
    Copyright (C) Paul Ashton 1997
-   Copyright (C) Jean Franois Micouleau 2002
+   Copyright (C) Jean François Micouleau 2002
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -68,11 +68,22 @@
 #define SAM_DATABASE_BUILTIN   0x01 /* BUILTIN users and groups */
 #define SAM_DATABASE_PRIVS     0x02 /* Privileges */
 
+/* flags use when sending a NETLOGON_CONTROL request */
+
+#define NETLOGON_CONTROL_SYNC			0x2
 #define NETLOGON_CONTROL_REDISCOVER		0x5
 #define NETLOGON_CONTROL_TC_QUERY		0x6
 #define NETLOGON_CONTROL_TRANSPORT_NOTIFY	0x7
 #define NETLOGON_CONTROL_SET_DBFLAG		0xfffe
 
+/* Some flag values reverse engineered from NLTEST.EXE */
+/* used in the NETLOGON_CONTROL[2] reply */
+
+#define NL_CTRL_IN_SYNC          0x0000
+#define NL_CTRL_REPL_NEEDED      0x0001
+#define NL_CTRL_REPL_IN_PROGRESS 0x0002
+#define NL_CTRL_FULL_SYNC        0x0004
+
 #if 0
 /* I think this is correct - it's what gets parsed on the wire. JRA. */
 /* NET_USER_INFO_2 */
@@ -168,7 +179,9 @@ typedef struct net_user_info_3
 	UNIHDR hdr_logon_dom; /* logon domain unicode string header */
 
 	uint32 buffer_dom_id; /* undocumented logon domain id pointer */
-	uint8 padding[40];    /* unused padding bytes.  expansion room */
+	uint8 lm_sess_key[8];	/* lm session key */
+	uint32 acct_flags;	/* account flags */
+	uint32 unknown[7];	/* unknown */
 
 	uint32 num_other_sids; /* number of foreign/trusted domain sids */
 	uint32 buffer_other_sids;
@@ -188,9 +201,8 @@ typedef struct net_user_info_3
 
 	DOM_SID2 dom_sid;           /* domain SID */
 
-	uint32 num_other_groups;        /* other groups */
-	DOM_GID *other_gids; /* group info */
 	DOM_SID2 *other_sids; /* foreign/trusted domain SIDs */
+	uint32 *other_sids_attrib;
 
 } NET_USER_INFO_3;
 
@@ -595,6 +607,27 @@ typedef struct sam_delta_hdr_info
 
 } SAM_DELTA_HDR;
 
+/* LOCKOUT_STRING */
+typedef struct account_lockout_string {
+	uint32 array_size;
+	uint32 offset;
+	uint32 length;
+/*	uint16 *bindata;	*/
+	UINT64_S lockout_duration;
+	UINT64_S reset_count;
+	uint32 bad_attempt_lockout;
+	uint32 dummy;
+
+} LOCKOUT_STRING;
+
+/* HDR_LOCKOUT_STRING */
+typedef struct hdr_account_lockout_string {
+	uint16 size;
+	uint16 length;
+	uint32 buffer;
+
+} HDR_LOCKOUT_STRING;
+
 /* SAM_DOMAIN_INFO (0x1) */
 typedef struct sam_domain_info_info
 {
@@ -608,16 +641,32 @@ typedef struct sam_domain_info_info
 	UINT64_S min_pwd_age;
 	UINT64_S dom_mod_count;
 	NTTIME   creation_time;
+	uint32   security_information;
 
-	BUFHDR2 hdr_sec_desc; /* security descriptor */
-	UNIHDR hdr_unknown;
-	uint8 reserved[40];
+	BUFHDR4 hdr_sec_desc; /* security descriptor */
+
+	HDR_LOCKOUT_STRING hdr_account_lockout;
+
+	UNIHDR hdr_unknown2;
+	UNIHDR hdr_unknown3;
+	UNIHDR hdr_unknown4;
 
 	UNISTR2 uni_dom_name;
-	UNISTR2 buf_oem_info; /* never seen */
+	UNISTR2 buf_oem_info; 
+
+	RPC_DATA_BLOB buf_sec_desc;
+
+	LOCKOUT_STRING account_lockout;
+
+	UNISTR2 buf_unknown2;
+	UNISTR2 buf_unknown3;
+	UNISTR2 buf_unknown4;
+
+	uint32 logon_chgpass;
+	uint32 unknown6;
+	uint32 unknown7;
+	uint32 unknown8;
 
-	BUFFER4 buf_sec_desc;
-	UNISTR2 buf_unknown;
 
 } SAM_DOMAIN_INFO;
 
@@ -632,7 +681,7 @@ typedef struct sam_group_info_info
 
 	UNISTR2 uni_grp_name;
 	UNISTR2 uni_grp_desc;
-	BUFFER4 buf_sec_desc;
+	RPC_DATA_BLOB buf_sec_desc;
 
 } SAM_GROUP_INFO;
 
@@ -710,11 +759,11 @@ typedef struct sam_account_info_info
 	uint32 unknown1; /* 0x4EC */
 	uint32 unknown2; /* 0 */
 
-	BUFFER4 buf_logon_hrs;
+	RPC_DATA_BLOB buf_logon_hrs;
 	UNISTR2 uni_comment;
 	UNISTR2 uni_parameters;
 	SAM_PWD pass;
-	BUFFER4 buf_sec_desc;
+	RPC_DATA_BLOB buf_sec_desc;
 	UNISTR2 uni_profile;
 
 } SAM_ACCOUNT_INFO;
@@ -745,7 +794,7 @@ typedef struct sam_alias_info_info
 	uint8 reserved[40];
 
 	UNISTR2 uni_als_name;
-	BUFFER4 buf_sec_desc;
+	RPC_DATA_BLOB buf_sec_desc;
 	UNISTR2 uni_als_desc;
 
 } SAM_ALIAS_INFO;
@@ -791,7 +840,7 @@ typedef struct
 	UNISTR2  domain_name;
 	DOM_SID2 domain_sid;
 
-	BUFFER4  buf_sec_desc;
+	RPC_DATA_BLOB  buf_sec_desc;
 } SAM_DELTA_POLICY;
 
 /* SAM_DELTA_TRUST_DOMS */
@@ -843,7 +892,7 @@ typedef struct
 	UNIHDR *hdr_privslist;
 	UNISTR2 *uni_privslist;
 
-	BUFFER4 buf_sec_desc;
+	RPC_DATA_BLOB buf_sec_desc;
 } SAM_DELTA_PRIVS;
 
 /* SAM_DELTA_SECRET */
