@@ -148,6 +148,8 @@ SamrTestPrivateFunctionsUser
 
 typedef struct logon_hours_info
 {
+	uint32 max_len; /* normally 1260 bytes */
+	uint32 offset;
 	uint32 len; /* normally 21 bytes */
 	uint8 hours[32];
 
@@ -212,9 +214,6 @@ typedef struct sam_user_info_23
 	UNISTR2 uni_unknown_str ; /* don't know what this is, yet. */
 	UNISTR2 uni_munged_dial ; /* munged path name and dial-back tel no */
 
-	uint32 unknown_6; /* 0x0000 04ec */
-	uint32 padding4;
-
 	LOGON_HRS logon_hrs;
 
 } SAM_USER_INFO_23;
@@ -336,9 +335,6 @@ typedef struct sam_user_info_21
 	UNISTR2 uni_unknown_str ; /* don't know what this is, yet. */
 	UNISTR2 uni_munged_dial ; /* munged path name and dial-back tel number */
 
-	uint32 unknown_6; /* 0x0000 04ec */
-	uint32 padding4;
-
 	LOGON_HRS logon_hrs;
 
 } SAM_USER_INFO_21;
@@ -355,8 +351,8 @@ typedef struct sam_user_info_20
 
 } SAM_USER_INFO_20;
 
-/* SAM_USER_INFO_12 */
-typedef struct sam_user_info_12
+/* SAM_USER_INFO_18 */
+typedef struct sam_user_info_18
 {
 	uint8 lm_pwd[16];    /* lm user passwords */
 	uint8 nt_pwd[16];    /* nt user passwords */
@@ -364,10 +360,10 @@ typedef struct sam_user_info_12
 	uint8 lm_pwd_active; 
 	uint8 nt_pwd_active; 
 
-} SAM_USER_INFO_12;
+} SAM_USER_INFO_18;
 
-/* SAM_USER_INFO_11 */
-typedef struct sam_user_info_11
+/* SAM_USER_INFO_17 */
+typedef struct sam_user_info_17
 {
 	uint8  padding_0[16];  /* 0 - padding 16 bytes */
 	NTTIME expiry;         /* expiry time or something? */
@@ -402,16 +398,23 @@ typedef struct sam_user_info_11
 
 	uint8  padding_9[48];  /* 0 - padding 48 bytes */
 
-} SAM_USER_INFO_11;
+} SAM_USER_INFO_17;
 
 
-/* SAM_USER_INFO_10 */
-typedef struct sam_user_info_10
+/* SAM_USER_INFO_16 */
+typedef struct sam_user_info_16
 {
 	uint32 acb_info;
 
-} SAM_USER_INFO_10;
+} SAM_USER_INFO_16;
 
+/* SAM_USER_INFO_7 */
+typedef struct sam_user_info_7
+{
+	UNIHDR hdr_name;  /* unicode header for name */
+	UNISTR2 uni_name; /* unicode string for name */
+
+} SAM_USER_INFO_7;
 
 
 /* SAMR_Q_CLOSE_HND - probably a policy handle close */
@@ -533,10 +536,17 @@ typedef struct sam_unknown_info_6_info
 
 typedef struct sam_unknown_info_7_info
 {
-	uint16 unknown_0; /* 0x0003 */
+	uint16 server_role;
 
 } SAM_UNK_INFO_7;
 
+typedef struct sam_unknown_info_8_info
+{
+	UINT64_S seq_num;
+	NTTIME domain_create_time;
+
+} SAM_UNK_INFO_8;
+
 typedef struct sam_unknown_info_12_inf
 {
 	NTTIME duration;
@@ -554,11 +564,8 @@ typedef struct sam_unknown_info_5_inf
 
 typedef struct sam_unknown_info_2_inf
 {
-	uint32 unknown_0; /* 0x0000 0000 */
-	uint32 unknown_1; /* 0x8000 0000 */
-	uint32 unknown_2; /* 0x0000 0000 */
-
-	uint32 ptr_0;     /* pointer to unknown structure */
+	NTTIME logout; /* whether users are forcibly disconnected when logon hours expire */
+	UNIHDR hdr_comment; /* comment according to samba4 idl */
 	UNIHDR hdr_domain; /* domain name unicode header */
 	UNIHDR hdr_server; /* server name unicode header */
 
@@ -566,11 +573,10 @@ typedef struct sam_unknown_info_2_inf
 	   pointer is referring to
 	 */
 
-	uint32 seq_num; /* some sort of incrementing sequence number? */
-	uint32 unknown_3; /* 0x0000 0000 */
+	UINT64_S seq_num;
 	
 	uint32 unknown_4; /* 0x0000 0001 */
-	uint32 unknown_5; /* 0x0000 0003 */
+	uint32 server_role;
 	uint32 unknown_6; /* 0x0000 0001 */
 	uint32 num_domain_usrs; /* number of users in domain */
 	uint32 num_domain_grps; /* number of domain groups in domain */
@@ -578,6 +584,7 @@ typedef struct sam_unknown_info_2_inf
 
 	uint8 padding[12]; /* 12 bytes zeros */
 
+	UNISTR2 uni_comment; /* comment unicode string */
 	UNISTR2 uni_domain; /* domain name unicode string */
 	UNISTR2 uni_server; /* server name unicode string */
 
@@ -604,6 +611,7 @@ typedef struct sam_unknown_ctr_info
 		SAM_UNK_INFO_5 inf5;
 		SAM_UNK_INFO_6 inf6;
 		SAM_UNK_INFO_7 inf7;
+		SAM_UNK_INFO_8 inf8;
 		SAM_UNK_INFO_12 inf12;
 
 	} info;
@@ -1068,6 +1076,14 @@ typedef struct samr_group_info1
 
 } GROUP_INFO1;
 
+typedef struct samr_group_info2
+{
+	uint16 level;
+	UNIHDR hdr_acct_name;
+	UNISTR2 uni_acct_name;
+
+} GROUP_INFO2;
+
 typedef struct samr_group_info3
 {
 	uint32 unknown_1; /* 0x0000 0003 - number of group members? */
@@ -1090,6 +1106,7 @@ typedef struct group_info_ctr
 	union
  	{
 		GROUP_INFO1 info1;
+		GROUP_INFO2 info2;
 		GROUP_INFO3 info3;
 		GROUP_INFO4 info4;
 
@@ -1163,74 +1180,52 @@ typedef struct r_samr_create_dom_alias_info
 
 } SAMR_R_CREATE_DOM_ALIAS;
 
-/* SAMR_Q_QUERY_ALIASINFO - SAM Alias Info */
-typedef struct q_samr_query_alias_info
-{
-	POLICY_HND pol;        /* policy handle */
 
-	uint16 switch_level;    /* 0x0003 seen */
+/********************************************************/
 
-} SAMR_Q_QUERY_ALIASINFO;
-
-typedef struct samr_alias_info1
-{
-	UNIHDR hdr_acct_name;
-	UNIHDR hdr_acct_desc;
+typedef struct {
+	UNISTR4 name;
+	UNISTR4 description;
 	uint32 num_member;
-	UNISTR2 uni_acct_name;
-	UNISTR2 uni_acct_desc;
-
 } ALIAS_INFO1;
 
-typedef struct samr_alias_info3
-{
-	UNIHDR hdr_acct_desc;
-	UNISTR2 uni_acct_desc;
-
+typedef struct {
+	UNISTR4 description;
 } ALIAS_INFO3;
 
-/* ALIAS_INFO_CTR */
-typedef struct alias_info_ctr
-{
-	uint16 switch_value1;
-	uint16 switch_value2;
+typedef struct {
+	POLICY_HND pol;        /* policy handle */
+	uint16 level;    /* 0x0003 seen */
+} SAMR_Q_QUERY_ALIASINFO;
 
-	union
- 	{
+typedef struct {
+	uint16 level;
+	union {
 		ALIAS_INFO1 info1;
 		ALIAS_INFO3 info3;
-
 	} alias;
-
 } ALIAS_INFO_CTR;
 
-/* SAMR_R_QUERY_ALIASINFO - SAM alias info */
-typedef struct r_samr_query_aliasinfo_info
-{
-	uint32 ptr;        
-	ALIAS_INFO_CTR ctr;
-
+typedef struct {
+	ALIAS_INFO_CTR *ctr;
 	NTSTATUS status;
-
 } SAMR_R_QUERY_ALIASINFO;
 
 
-/* SAMR_Q_SET_ALIASINFO - SAM Alias Info */
-typedef struct q_samr_set_alias_info
-{
+/********************************************************/
+
+typedef struct {
 	POLICY_HND alias_pol;        /* policy handle */
 	ALIAS_INFO_CTR ctr;
-
 } SAMR_Q_SET_ALIASINFO;
 
-/* SAMR_R_SET_ALIASINFO - SAM alias info */
-typedef struct r_samr_set_aliasinfo_info
-{
+typedef struct {
 	NTSTATUS status;
-
 } SAMR_R_SET_ALIASINFO;
 
 
+/********************************************************/
+
 /* SAMR_Q_QUERY_USERGROUPS - */
 typedef struct q_samr_query_usergroup_info
 {
@@ -1259,14 +1254,15 @@ typedef struct sam_userinfo_ctr_info
 
 	union
 	{
-		SAM_USER_INFO_10 *id10; /* auth-level 0x10 */
-		SAM_USER_INFO_11 *id11; /* auth-level 0x11 */
-		SAM_USER_INFO_12 *id12; /* auth-level 0x12 */
-		SAM_USER_INFO_20 *id20; /* auth-level 20 */
-		SAM_USER_INFO_21 *id21; /* auth-level 21 */
-		SAM_USER_INFO_23 *id23; /* auth-level 0x17 */
-		SAM_USER_INFO_24 *id24; /* auth-level 0x18 */
-		SAM_USER_INFO_25 *id25; /* auth-level 0x19 */
+		SAM_USER_INFO_7  *id7;
+		SAM_USER_INFO_16 *id16;
+		SAM_USER_INFO_17 *id17;
+		SAM_USER_INFO_18 *id18;
+		SAM_USER_INFO_20 *id20;
+		SAM_USER_INFO_21 *id21;
+		SAM_USER_INFO_23 *id23;
+		SAM_USER_INFO_24 *id24;
+		SAM_USER_INFO_25 *id25;
 		void* id; /* to make typecasting easy */
 
 	} info;
@@ -1734,11 +1730,7 @@ typedef struct q_samr_get_dom_pwinfo
 typedef struct r_samr_get_dom_pwinfo
 {
 	/*
-	 * Previously this was 3 uint16's.  However, after some tests
-	 * it appears that the data len for the signing needs to be 16.
-	 * Not sure how 3 unit16's ever worked since the length always
-	 * turned out to 12.  3 uint32's + NT_STATUS == 16 bytes.  Tested
-	 * using NT and 2k.  --jerry
+	 * See Samba4 IDL
 	 */
 	uint16 unk_0;
 	uint32 unk_1;
