@@ -21,6 +21,8 @@
 
 #include "includes.h"
 
+extern struct current_user current_user;
+
 fstring local_machine="";
 fstring remote_arch="UNKNOWN";
 userdom_struct current_user_info;
@@ -40,24 +42,24 @@ void set_local_machine_name(const char* local_name, BOOL perm)
 	static BOOL already_perm = False;
 	fstring tmp_local_machine;
 
+	fstrcpy(tmp_local_machine,local_name);
+	trim_char(tmp_local_machine,' ',' ');
+
 	/*
 	 * Windows NT/2k uses "*SMBSERVER" and XP uses "*SMBSERV"
 	 * arrggg!!! 
 	 */
 
-	if (strequal(local_name, "*SMBSERVER")) 
-		return;
-
-	if (strequal(local_name, "*SMBSERV")) 
+	if ( strequal(tmp_local_machine, "*SMBSERVER") || strequal(tmp_local_machine, "*SMBSERV") )  {
+		fstrcpy( local_machine, client_socket_addr() );
 		return;
+	}
 
 	if (already_perm)
 		return;
 
 	already_perm = perm;
 
-	fstrcpy(tmp_local_machine,local_name);
-	trim_char(tmp_local_machine,' ',' ');
 	alpha_strcpy(local_machine,tmp_local_machine,SAFE_NETBIOS_CHARS,sizeof(local_machine)-1);
 	strlower_m(local_machine);
 }
@@ -134,6 +136,18 @@ void set_current_user_info(const userdom_struct *pcui)
 	fstrcpy(smb_user_name, current_user_info.smb_name);
 }
 
+/*******************************************************************
+ return the current active user name
+*******************************************************************/
+
+const char* get_current_username( void )
+{
+	if ( current_user_info.smb_name[0] == '\0' )
+		return smb_user_name;
+
+	return current_user_info.smb_name; 
+}
+
 /*******************************************************************
  Given a pointer to a %$(NAME) expand it as an environment variable.
  Return the number of characters by which the pointer should be advanced.
@@ -224,7 +238,7 @@ static char * realloc_expand_env_var(char *str, char *p)
 
 	r = p + 3;
 	copylen = q - r;
-	envname = (char *)malloc(copylen + 1 + 4); /* reserve space for use later add %$() chars */
+	envname = (char *)SMB_MALLOC(copylen + 1 + 4); /* reserve space for use later add %$() chars */
 	if (envname == NULL) return NULL;
 	strncpy(envname,r,copylen);
 	envname[copylen] = '\0';
@@ -371,15 +385,20 @@ void standard_sub_basic(const char *smb_name, char *str,size_t len)
 		case 'i' :
 			string_sub(p,"%i", client_socket_addr(),l);
 			break;
-		case 'L' : 
-			if (local_machine_name && *local_machine_name)
-				string_sub(p,"%L", local_machine_name,l); 
-			else {
+		case 'L' :
+			if (!StrnCaseCmp(p, "%LOGONSERVER%", strlen("%LOGONSERVER%"))) {
+				p++;
+				break;
+			}
+
+			if (local_machine_name && *local_machine_name) {
+				string_sub_once(p, "%L", local_machine_name, l); 
+			} else {
 				pstring temp_name;
 
 				pstrcpy(temp_name, global_myname());
 				strlower_m(temp_name);
-				string_sub(p,"%L", temp_name,l); 
+				string_sub_once(p, "%L", temp_name, l); 
 			}
 			break;
 		case 'M' :
@@ -446,7 +465,8 @@ static void standard_sub_advanced(int snum, const char *user,
 			string_sub(p,"%P", connectpath, l); 
 			break;
 		case 'S': 
-			string_sub(p,"%S", lp_servicename(snum), l); 
+			if ( snum != -1 )
+				string_sub(p,"%S", lp_servicename(snum), l); 
 			break;
 		case 'g': 
 			string_sub(p,"%g", gidtoname(gid), l); 
@@ -463,7 +483,8 @@ static void standard_sub_advanced(int snum, const char *user,
 			 * "path =" string in [homes] and so needs the
 			 * service name, not the username.  */
 		case 'p': 
-			string_sub(p,"%p", automount_path(lp_servicename(snum)), l); 
+			if ( snum != -1 )
+				string_sub(p,"%p", automount_path(lp_servicename(snum)), l); 
 			break;
 		case '\0': 
 			p++; 
@@ -506,7 +527,7 @@ char *alloc_sub_basic(const char *smb_name, const char *str)
 		return NULL;
 	}
 	
-	a_string = strdup(str);
+	a_string = SMB_STRDUP(str);
 	if (a_string == NULL) {
 		DEBUG(0, ("alloc_sub_specified: Out of memory!\n"));
 		return NULL;
@@ -524,7 +545,7 @@ char *alloc_sub_basic(const char *smb_name, const char *str)
 			t = realloc_string_sub(t, "%U", r);
 			break;
 		case 'G' :
-			r = strdup(smb_name);
+			r = SMB_STRDUP(smb_name);
 			if (r == NULL) goto error;
 			if ((pass = Get_Pwnam(r))!=NULL) {
 				t = realloc_string_sub(t, "%G", gidtoname(pass->pw_gid));
@@ -621,7 +642,7 @@ char *alloc_sub_specified(const char *input_string,
 	char *a_string, *ret_string;
 	char *b, *p, *s, *t;
 
-	a_string = strdup(input_string);
+	a_string = SMB_STRDUP(input_string);
 	if (a_string == NULL) {
 		DEBUG(0, ("alloc_sub_specified: Out of memory!\n"));
 		return NULL;
@@ -698,7 +719,7 @@ char *alloc_sub_advanced(int snum, const char *user,
 	char *a_string, *ret_string;
 	char *b, *p, *s, *t, *h;
 
-	a_string = strdup(str);
+	a_string = SMB_STRDUP(str);
 	if (a_string == NULL) {
 		DEBUG(0, ("alloc_sub_specified: Out of memory!\n"));
 		return NULL;
@@ -786,7 +807,6 @@ char *alloc_sub_conn(connection_struct *conn, const char *str)
 
 void standard_sub_snum(int snum, char *str, size_t len)
 {
-	extern struct current_user current_user;
 	static uid_t cached_uid = -1;
 	static fstring cached_user;
 	/* calling uidtoname() on every substitute would be too expensive, so
@@ -797,6 +817,6 @@ void standard_sub_snum(int snum, char *str, size_t len)
 		cached_uid = current_user.uid;
 	}
 
-	standard_sub_advanced(snum, cached_user, "", -1,
+	standard_sub_advanced(snum, cached_user, "", current_user.gid,
 			      smb_user_name, str, len);
 }
