@@ -20,6 +20,9 @@
 
 #include "includes.h"
 
+#undef  DBGC_CLASS
+#define DBGC_CLASS DBGC_ACLS
+
 /*
  This file wraps all differing system ACL interfaces into a consistent
  one based on the POSIX interface. It also returns the correct errors
@@ -612,7 +615,7 @@ char *sys_acl_to_text(SMB_ACL_T acl_d, ssize_t *len_p)
 	 */
 	len	= 0;
 	maxlen	= 20 * acl_d->count;
-	if ((text = malloc(maxlen)) == NULL) {
+	if ((text = SMB_MALLOC(maxlen)) == NULL) {
 		errno = ENOMEM;
 		return NULL;
 	}
@@ -690,7 +693,7 @@ char *sys_acl_to_text(SMB_ACL_T acl_d, ssize_t *len_p)
 
 			maxlen += nbytes + 20 * (acl_d->count - i);
 
-			if ((text = Realloc(oldtext, maxlen)) == NULL) {
+			if ((text = SMB_REALLOC(oldtext, maxlen)) == NULL) {
 				SAFE_FREE(oldtext);
 				errno = ENOMEM;
 				return NULL;
@@ -722,7 +725,7 @@ SMB_ACL_T sys_acl_init(int count)
 	 * acl[] array, this actually allocates an ACL with room
 	 * for (count+1) entries
 	 */
-	if ((a = malloc(sizeof(*a) + count * sizeof(struct acl))) == NULL) {
+	if ((a = SMB_MALLOC(sizeof(struct SMB_ACL_T) + count * sizeof(struct acl))) == NULL) {
 		errno = ENOMEM;
 		return NULL;
 	}
@@ -886,7 +889,7 @@ int sys_acl_set_file(const char *name, SMB_ACL_TYPE_T type, SMB_ACL_T acl_d)
 		 * allocate a temporary buffer for the complete ACL
 		 */
 		acl_count = acc_acl->count + def_acl->count;
-		acl_p = acl_buf = malloc(acl_count * sizeof(acl_buf[0]));
+		acl_p = acl_buf = SMB_MALLOC_ARRAY(struct acl, acl_count);
 
 		if (acl_buf == NULL) {
 			sys_acl_free_acl(tmp_acl);
@@ -1243,7 +1246,7 @@ char *sys_acl_to_text(SMB_ACL_T acl_d, ssize_t *len_p)
 	 */
 	len	= 0;
 	maxlen	= 20 * acl_d->count;
-	if ((text = malloc(maxlen)) == NULL) {
+	if ((text = SMB_MALLOC(maxlen)) == NULL) {
 		errno = ENOMEM;
 		return NULL;
 	}
@@ -1321,7 +1324,7 @@ char *sys_acl_to_text(SMB_ACL_T acl_d, ssize_t *len_p)
 
 			maxlen += nbytes + 20 * (acl_d->count - i);
 
-			if ((text = Realloc(oldtext, maxlen)) == NULL) {
+			if ((text = SMB_REALLOC(oldtext, maxlen)) == NULL) {
 				free(oldtext);
 				errno = ENOMEM;
 				return NULL;
@@ -1353,7 +1356,7 @@ SMB_ACL_T sys_acl_init(int count)
 	 * acl[] array, this actually allocates an ACL with room
 	 * for (count+1) entries
 	 */
-	if ((a = malloc(sizeof(*a) + count * sizeof(struct acl))) == NULL) {
+	if ((a = SMB_MALLOC(sizeof(struct SMB_ACL_T) + count * sizeof(struct acl))) == NULL) {
 		errno = ENOMEM;
 		return NULL;
 	}
@@ -1819,7 +1822,7 @@ int sys_acl_set_file(const char *name, SMB_ACL_TYPE_T type, SMB_ACL_T acl_d)
 		 * allocate a temporary buffer for the complete ACL
 		 */
 		acl_count = acc_acl->count + def_acl->count;
-		acl_p = acl_buf = malloc(acl_count * sizeof(acl_buf[0]));
+		acl_p = acl_buf = SMB_MALLOC_ARRAY(struct acl, acl_count);
 
 		if (acl_buf == NULL) {
 			sys_acl_free_acl(tmp_acl);
@@ -1982,7 +1985,7 @@ SMB_ACL_T sys_acl_get_file(const char *path_p, SMB_ACL_TYPE_T type)
 {
 	SMB_ACL_T	a;
 
-	if ((a = malloc(sizeof(*a))) == NULL) {
+	if ((a = SMB_MALLOC_P(struct SMB_ACL_T)) == NULL) {
 		errno = ENOMEM;
 		return NULL;
 	}
@@ -1999,7 +2002,7 @@ SMB_ACL_T sys_acl_get_fd(int fd)
 {
 	SMB_ACL_T	a;
 
-	if ((a = malloc(sizeof(*a))) == NULL) {
+	if ((a = SMB_MALLOC_P(struct SMB_ACL_T)) == NULL) {
 		errno = ENOMEM;
 		return NULL;
 	}
@@ -2056,7 +2059,7 @@ SMB_ACL_T sys_acl_init(int count)
 		return NULL;
 	}
 
-	if ((a = malloc(sizeof(*a) + sizeof(struct acl))) == NULL) {
+	if ((a = SMB_MALLOC(sizeof(struct SMB_ACL_T) + sizeof(struct acl))) == NULL) {
 		errno = ENOMEM;
 		return NULL;
 	}
@@ -2277,12 +2280,16 @@ SMB_ACL_T sys_acl_get_file( const char *path_p, SMB_ACL_TYPE_T type)
 	int rc = 0;
 	uid_t user_id;
 
+	/* AIX has no DEFAULT */
+	if  ( type == SMB_ACL_TYPE_DEFAULT )
+		return NULL;
+
 	/* Get the acl using statacl */
  
 	DEBUG(10,("Entering sys_acl_get_file\n"));
 	DEBUG(10,("path_p is %s\n",path_p));
 
-	file_acl = (struct acl *)malloc(BUFSIZ);
+	file_acl = (struct acl *)SMB_MALLOC(BUFSIZ);
  
 	if(file_acl == NULL) {
 		errno=ENOMEM;
@@ -2313,7 +2320,7 @@ SMB_ACL_T sys_acl_get_file( const char *path_p, SMB_ACL_TYPE_T type)
 	if(acl_entry_link_head == NULL)
 		return(NULL);
 
-	acl_entry_link->entryp = (struct new_acl_entry *)malloc(sizeof(struct new_acl_entry));
+	acl_entry_link->entryp = SMB_MALLOC_P(struct new_acl_entry);
 	if(acl_entry_link->entryp == NULL) {
 		SAFE_FREE(file_acl);
 		errno = ENOMEM;
@@ -2348,8 +2355,7 @@ SMB_ACL_T sys_acl_get_file( const char *path_p, SMB_ACL_TYPE_T type)
 			 * and already has entryp allocated.                  */
 
 			if(acl_entry_link_head->count != 0) {
-				acl_entry_link->nextp = (struct acl_entry_link *)
-											malloc(sizeof(struct acl_entry_link));
+				acl_entry_link->nextp = SMB_MALLOC_P(struct acl_entry_link);
 
 				if(acl_entry_link->nextp == NULL) {
 					SAFE_FREE(file_acl);
@@ -2360,7 +2366,7 @@ SMB_ACL_T sys_acl_get_file( const char *path_p, SMB_ACL_TYPE_T type)
 
 				acl_entry_link->nextp->prevp = acl_entry_link;
 				acl_entry_link = acl_entry_link->nextp;
-				acl_entry_link->entryp = (struct new_acl_entry *)malloc(sizeof(struct new_acl_entry));
+				acl_entry_link->entryp = SMB_MALLOC_P(struct new_acl_entry);
 				if(acl_entry_link->entryp == NULL) {
 					SAFE_FREE(file_acl);
 					errno = ENOMEM;
@@ -2419,7 +2425,7 @@ SMB_ACL_T sys_acl_get_file( const char *path_p, SMB_ACL_TYPE_T type)
 	for( i = 1; i < 4; i++) {
 		DEBUG(10,("i is %d\n",i));
 		if(acl_entry_link_head->count != 0) {
-			acl_entry_link->nextp = (struct acl_entry_link *)malloc(sizeof(struct acl_entry_link));
+			acl_entry_link->nextp = SMB_MALLOC_P(struct acl_entry_link);
 			if(acl_entry_link->nextp == NULL) {
 				SAFE_FREE(file_acl);
 				errno = ENOMEM;
@@ -2429,7 +2435,7 @@ SMB_ACL_T sys_acl_get_file( const char *path_p, SMB_ACL_TYPE_T type)
 
 			acl_entry_link->nextp->prevp = acl_entry_link;
 			acl_entry_link = acl_entry_link->nextp;
-			acl_entry_link->entryp = (struct new_acl_entry *)malloc(sizeof(struct new_acl_entry));
+			acl_entry_link->entryp = SMB_MALLOC_P(struct new_acl_entry);
 			if(acl_entry_link->entryp == NULL) {
 				SAFE_FREE(file_acl);
 				errno = ENOMEM;
@@ -2496,7 +2502,7 @@ SMB_ACL_T sys_acl_get_fd(int fd)
    
 	DEBUG(10,("Entering sys_acl_get_fd\n"));
 	DEBUG(10,("fd is %d\n",fd));
-	file_acl = (struct acl *)malloc(BUFSIZ);
+	file_acl = (struct acl *)SMB_MALLOC(BUFSIZ);
 
 	if(file_acl == NULL) {
 		errno=ENOMEM;
@@ -2529,7 +2535,7 @@ SMB_ACL_T sys_acl_get_fd(int fd)
 		return(NULL);
 	}
 
-	acl_entry_link->entryp = (struct new_acl_entry *)malloc(sizeof(struct new_acl_entry));
+	acl_entry_link->entryp = SMB_MALLOC_P(struct new_acl_entry);
 
 	if(acl_entry_link->entryp == NULL) {
 		errno = ENOMEM;
@@ -2566,7 +2572,7 @@ SMB_ACL_T sys_acl_get_fd(int fd)
 			 * and already has entryp allocated.                 */
 
 			if(acl_entry_link_head->count != 0) {
-				acl_entry_link->nextp = (struct acl_entry_link *)malloc(sizeof(struct acl_entry_link));
+				acl_entry_link->nextp = SMB_MALLOC_P(struct acl_entry_link);
 				if(acl_entry_link->nextp == NULL) {
 					errno = ENOMEM;
 					DEBUG(0,("Error in sys_acl_get_fd is %d\n",errno));
@@ -2575,7 +2581,7 @@ SMB_ACL_T sys_acl_get_fd(int fd)
 				}
 				acl_entry_link->nextp->prevp = acl_entry_link;
 				acl_entry_link = acl_entry_link->nextp;
-				acl_entry_link->entryp = (struct new_acl_entry *)malloc(sizeof(struct new_acl_entry));
+				acl_entry_link->entryp = SMB_MALLOC_P(struct new_acl_entry);
 				if(acl_entry_link->entryp == NULL) {
 					errno = ENOMEM;
 					DEBUG(0,("Error in sys_acl_get_fd is %d\n",errno));
@@ -2634,7 +2640,7 @@ SMB_ACL_T sys_acl_get_fd(int fd)
 	for( i = 1; i < 4; i++) {
 		DEBUG(10,("i is %d\n",i));
 		if(acl_entry_link_head->count != 0){
-			acl_entry_link->nextp = (struct acl_entry_link *)malloc(sizeof(struct acl_entry_link));
+			acl_entry_link->nextp = SMB_MALLOC_P(struct acl_entry_link);
 			if(acl_entry_link->nextp == NULL) {
 				errno = ENOMEM;
 				DEBUG(0,("Error in sys_acl_get_fd is %d\n",errno));
@@ -2644,7 +2650,7 @@ SMB_ACL_T sys_acl_get_fd(int fd)
 
 			acl_entry_link->nextp->prevp = acl_entry_link;
 			acl_entry_link = acl_entry_link->nextp;
-			acl_entry_link->entryp = (struct new_acl_entry *)malloc(sizeof(struct new_acl_entry));
+			acl_entry_link->entryp = SMB_MALLOC_P(struct new_acl_entry);
 
 			if(acl_entry_link->entryp == NULL) {
 				SAFE_FREE(file_acl);
@@ -2723,7 +2729,7 @@ SMB_ACL_T sys_acl_init( int count)
  
 	DEBUG(10,("Entering sys_acl_init\n"));
 
-	theacl = (struct acl_entry_link *)malloc(sizeof(struct acl_entry_link));
+	theacl = SMB_MALLOC_P(struct acl_entry_link);
 	if(theacl == NULL) {
 		errno = ENOMEM;
 		DEBUG(0,("Error in sys_acl_init is %d\n",errno));
@@ -2758,7 +2764,7 @@ int sys_acl_create_entry( SMB_ACL_T *pacl, SMB_ACL_ENTRY_T *pentry)
 	}
 
 	if(theacl->count != 0){
-		temp_entry->nextp = acl_entryp = (struct acl_entry_link *)malloc(sizeof(struct acl_entry_link));
+		temp_entry->nextp = acl_entryp = SMB_MALLOC_P(struct acl_entry_link);
 		if(acl_entryp == NULL) {
 			errno = ENOMEM;
 			DEBUG(0,("Error in sys_acl_create_entry is %d\n",errno));
@@ -2770,7 +2776,7 @@ int sys_acl_create_entry( SMB_ACL_T *pacl, SMB_ACL_ENTRY_T *pentry)
 		DEBUG(10,("The acl_entryp->prevp is %d\n",acl_entryp->prevp));
 	}
 
-	*pentry = acl_entryp->entryp = (struct new_acl_entry *)malloc(sizeof(struct new_acl_entry));
+	*pentry = acl_entryp->entryp = SMB_MALLOC_P(struct new_acl_entry);
 	if(*pentry == NULL) {
 		errno = ENOMEM;
 		DEBUG(0,("Error in sys_acl_create_entry is %d\n",errno));
@@ -2860,7 +2866,7 @@ int sys_acl_set_file( const char *name, SMB_ACL_TYPE_T acltype, SMB_ACL_T theacl
 		return(0);
 
 	acl_length = BUFSIZ;
-	file_acl = (struct acl *)malloc(BUFSIZ);
+	file_acl = (struct acl *)SMB_MALLOC(BUFSIZ);
 
 	if(file_acl == NULL) {
 		errno = ENOMEM;
@@ -2893,7 +2899,7 @@ int sys_acl_set_file( const char *name, SMB_ACL_TYPE_T acltype, SMB_ACL_T theacl
 
 		if((file_acl->acl_len + sizeof(struct acl_entry)) > acl_length) {
 			acl_length += sizeof(struct acl_entry);
-			file_acl_temp = (struct acl *)malloc(acl_length);
+			file_acl_temp = (struct acl *)SMB_MALLOC(acl_length);
 			if(file_acl_temp == NULL) {
 				SAFE_FREE(file_acl);
 				errno = ENOMEM;
@@ -2948,7 +2954,7 @@ int sys_acl_set_fd( int fd, SMB_ACL_T theacl)
  
 	DEBUG(10,("Entering sys_acl_set_fd\n"));
 	acl_length = BUFSIZ;
-	file_acl = (struct acl *)malloc(BUFSIZ);
+	file_acl = (struct acl *)SMB_MALLOC(BUFSIZ);
 
 	if(file_acl == NULL) {
 		errno = ENOMEM;
@@ -2982,7 +2988,7 @@ int sys_acl_set_fd( int fd, SMB_ACL_T theacl)
 
 		if((file_acl->acl_len + sizeof(struct acl_entry)) > acl_length) {
 			acl_length += sizeof(struct acl_entry);
-			file_acl_temp = (struct acl *)malloc(acl_length);
+			file_acl_temp = (struct acl *)SMB_MALLOC(acl_length);
 			if(file_acl_temp == NULL) {
 				SAFE_FREE(file_acl);
 				errno = ENOMEM;
