@@ -93,7 +93,7 @@ char *get_user_home_dir(const char *user)
  Returns True if username was changed, false otherwise.
 ********************************************************************/
 
-BOOL map_username(char *user)
+BOOL map_username(fstring user)
 {
 	static BOOL initialised=False;
 	static fstring last_from,last_to;
@@ -102,18 +102,11 @@ BOOL map_username(char *user)
 	char *s;
 	pstring buf;
 	BOOL mapped_user = False;
-
+	char *cmd = lp_username_map_script();
+	
 	if (!*user)
 		return False;
-
-	if (!*mapfile)
-		return False;
-
-	if (!initialised) {
-		*last_from = *last_to = 0;
-		initialised = True;
-	}
-
+		
 	if (strequal(user,last_to))
 		return False;
 
@@ -122,6 +115,52 @@ BOOL map_username(char *user)
 		fstrcpy(user,last_to);
 		return True;
 	}
+	
+	/* first try the username map script */
+	
+	if ( *cmd ) {
+		char **qlines;
+		pstring command;
+		int numlines, ret, fd;
+
+		pstr_sprintf( command, "%s \"%s\"", cmd, user );
+
+		DEBUG(10,("Running [%s]\n", command));
+		ret = smbrun(command, &fd);
+		DEBUGADD(10,("returned [%d]\n", ret));
+
+		if ( ret != 0 ) {
+			if (fd != -1)
+				close(fd);
+			return False;
+		}
+
+		numlines = 0;
+		qlines = fd_lines_load(fd, &numlines);
+		DEBUGADD(10,("Lines returned = [%d]\n", numlines));
+		close(fd);
+
+		/* should be either no lines or a single line with the mapped username */
+
+		if (numlines) {
+			DEBUG(3,("Mapped user %s to %s\n", user, qlines[0] ));
+			fstrcpy( user, qlines[0] );
+		}
+
+		file_lines_free(qlines);
+		
+		return numlines != 0;
+	}
+
+	/* ok.  let's try the mapfile */
+	
+	if (!*mapfile)
+		return False;
+
+	if (!initialised) {
+		*last_from = *last_to = 0;
+		initialised = True;
+	}
   
 	f = x_fopen(mapfile,O_RDONLY, 0);
 	if (!f) {
@@ -172,10 +211,10 @@ BOOL map_username(char *user)
 		if (strchr_m(dosname,'*') || user_in_list(user, (const char **)dosuserlist, NULL, 0)) {
 			DEBUG(3,("Mapped user %s to %s\n",user,unixname));
 			mapped_user = True;
-			fstrcpy(last_from,user);
-			sscanf(unixname,"%s",user);
-			fstrcpy(last_to,user);
-			if(return_if_mapped) {
+			fstrcpy( last_from,user );
+			fstrcpy( user, unixname );
+			fstrcpy( last_to,user );
+			if ( return_if_mapped ) {
 				str_list_free (&dosuserlist);
 				x_fclose(f);
 				return True;
@@ -250,6 +289,45 @@ static struct passwd *Get_Pwnam_internals(const char *user, char *user2)
 done:
 	DEBUG(5,("Get_Pwnam_internals %s find user [%s]!\n",ret ? "did":"didn't", user));
 
+	return ret;
+}
+
+/****************************************************************************
+ Get_Pwnam wrapper without modification.
+  NOTE: This with NOT modify 'user'! 
+  This will return an allocated structure
+****************************************************************************/
+
+struct passwd *Get_Pwnam_alloc(const char *user)
+{
+	fstring user2;
+	struct passwd *ret;
+
+	if ( *user == '\0' ) {
+		DEBUG(10,("Get_Pwnam: empty username!\n"));
+		return NULL;
+	}
+
+	fstrcpy(user2, user);
+
+	DEBUG(5,("Finding user %s\n", user));
+
+	ret = Get_Pwnam_internals(user, user2);
+	
+	return ret;  
+}
+
+/****************************************************************************
+ Get_Pwnam wrapper without modification.
+  NOTE: This with NOT modify 'user'! 
+****************************************************************************/
+
+struct passwd *Get_Pwnam(const char *user)
+{
+	struct passwd *ret;
+
+	ret = Get_Pwnam_alloc(user);
+	
 	/* This call used to just return the 'passwd' static buffer.
 	   This could then have accidental reuse implications, so 
 	   we now malloc a copy, and free it in the next use.
@@ -270,30 +348,6 @@ done:
 	
 	Get_Pwnam_ret = ret;
 
-	return ret;
-}
-
-/****************************************************************************
- Get_Pwnam wrapper without modification.
-  NOTE: This with NOT modify 'user'! 
-****************************************************************************/
-
-struct passwd *Get_Pwnam(const char *user)
-{
-	fstring user2;
-	struct passwd *ret;
-
-	if ( *user == '\0' ) {
-		DEBUG(10,("Get_Pwnam: empty username!\n"));
-		return NULL;
-	}
-
-	fstrcpy(user2, user);
-
-	DEBUG(5,("Finding user %s\n", user));
-
-	ret = Get_Pwnam_internals(user, user2);
-	
 	return ret;  
 }
 
