@@ -23,6 +23,11 @@
 
 #include "includes.h"
 
+extern fstring local_machine;
+extern char *global_clobber_region_function;
+extern unsigned int global_clobber_region_line;
+extern fstring remote_arch;
+
 /* Max allowable allococation - 256mb - 0x10000000 */
 #define MAX_ALLOC_SIZE (1024*1024*256)
 
@@ -52,7 +57,7 @@
 #endif /* WITH_NISPLUS_HOME */
 #endif /* HAVE_NETGROUP && WITH_AUTOMOUNT */
 
-int Protocol = PROTOCOL_COREPLUS;
+enum protocol_types Protocol = PROTOCOL_COREPLUS;
 
 /* a default finfo structure to ensure all fields are sensible */
 file_info def_finfo = {-1,0,0,0,0,0,0,"",""};
@@ -230,7 +235,6 @@ BOOL set_netbios_aliases(const char **str_array)
 
 BOOL init_names(void)
 {
-	extern fstring local_machine;
 	char *p;
 	int n;
 
@@ -273,29 +277,12 @@ const char *tmpdir(void)
 	return "/tmp";
 }
 
-/****************************************************************************
- Determine whether we are in the specified group.
-****************************************************************************/
-
-BOOL in_group(gid_t group, gid_t current_gid, int ngroups, const gid_t *groups)
-{
-	int i;
-
-	if (group == current_gid)
-		return(True);
-
-	for (i=0;i<ngroups;i++)
-		if (group == groups[i])
-			return(True);
-
-	return(False);
-}
-
 /****************************************************************************
  Add a gid to an array of gids if it's not already there.
 ****************************************************************************/
 
-void add_gid_to_array_unique(gid_t gid, gid_t **gids, int *num)
+void add_gid_to_array_unique(TALLOC_CTX *mem_ctx, gid_t gid,
+			     gid_t **gids, int *num)
 {
 	int i;
 
@@ -303,8 +290,11 @@ void add_gid_to_array_unique(gid_t gid, gid_t **gids, int *num)
 		if ((*gids)[i] == gid)
 			return;
 	}
-	
-	*gids = SMB_REALLOC_ARRAY(*gids, gid_t, *num+1);
+
+	if (mem_ctx != NULL)
+		*gids = TALLOC_REALLOC_ARRAY(mem_ctx, *gids, gid_t, *num+1);
+	else
+		*gids = SMB_REALLOC_ARRAY(*gids, gid_t, *num+1);
 
 	if (*gids == NULL)
 		return;
@@ -613,38 +603,6 @@ void unix_clean_name(char *s)
 	trim_string(s,NULL,"/..");
 }
 
-/****************************************************************************
- Make a dir struct.
-****************************************************************************/
-
-void make_dir_struct(char *buf, const char *mask, const char *fname,SMB_OFF_T size,int mode,time_t date, BOOL case_sensitive)
-{  
-	char *p;
-	pstring mask2;
-
-	pstrcpy(mask2,mask);
-
-	if ((mode & aDIR) != 0)
-		size = 0;
-
-	memset(buf+1,' ',11);
-	if ((p = strchr_m(mask2,'.')) != NULL) {
-		*p = 0;
-		push_ascii(buf+1,mask2,8, 0);
-		push_ascii(buf+9,p+1,3, 0);
-		*p = '.';
-	} else
-		push_ascii(buf+1,mask2,11, 0);
-
-	memset(buf+21,'\0',DIR_STRUCT_SIZE-21);
-	SCVAL(buf,21,mode);
-	put_dos_date(buf,22,date);
-	SSVAL(buf,26,size & 0xFFFF);
-	SSVAL(buf,28,(size >> 16)&0xFFFF);
-	push_ascii(buf+30,fname,12, case_sensitive ? 0 : STR_UPPER);
-	DEBUG(8,("put name [%s] from [%s] into dir struct\n",buf+30, fname));
-}
-
 /*******************************************************************
  Close the low 3 fd's and open dev/null in their place.
 ********************************************************************/
@@ -682,6 +640,46 @@ void close_low_fds(BOOL stderr_too)
 #endif
 }
 
+/*******************************************************************
+ Write data into an fd at a given offset. Ignore seek errors.
+********************************************************************/
+
+ssize_t write_data_at_offset(int fd, const char *buffer, size_t N, SMB_OFF_T pos)
+{
+	size_t total=0;
+	ssize_t ret;
+
+	if (pos == (SMB_OFF_T)-1) {
+		return write_data(fd, buffer, N);
+	}
+#if defined(HAVE_PWRITE) || defined(HAVE_PRWITE64)
+	while (total < N) {
+		ret = sys_pwrite(fd,buffer + total,N - total, pos);
+		if (ret == -1 && errno == ESPIPE) {
+			return write_data(fd, buffer + total,N - total);
+		}
+		if (ret == -1) {
+			DEBUG(0,("write_data_at_offset: write failure. Error = %s\n", strerror(errno) ));
+			return -1;
+		}
+		if (ret == 0) {
+			return total;
+		}
+		total += ret;
+		pos += ret;
+	}
+	return (ssize_t)total;
+#else
+	/* Use lseek and write_data. */
+	if (sys_lseek(fd, pos, SEEK_SET) == -1) {
+		if (errno != ESPIPE) {
+			return -1;
+		}
+	}
+	return write_data(fd, buffer, N);
+#endif
+}
+
 /****************************************************************************
  Set a fd into blocking/nonblocking mode. Uses POSIX O_NONBLOCK if available,
  else
@@ -779,12 +777,24 @@ SMB_OFF_T transfer_file(int infd,int outfd,SMB_OFF_T n)
 
 void smb_msleep(unsigned int t)
 {
+#if defined(HAVE_NANOSLEEP)
+	struct timespec tval;
+	int ret;
+
+	tval.tv_sec = t/1000;
+	tval.tv_nsec = 1000000*(t%1000);
+
+	do {
+		errno = 0;
+		ret = nanosleep(&tval, &tval);
+	} while (ret < 0 && errno == EINTR && (tval.tv_sec > 0 || tval.tv_nsec > 0));
+#else
 	unsigned int tdiff=0;
 	struct timeval tval,t1,t2;  
 	fd_set fds;
 
 	GetTimeOfDay(&t1);
-	GetTimeOfDay(&t2);
+	t2 = t1;
   
 	while (tdiff < t) {
 		tval.tv_sec = (t-tdiff)/1000;
@@ -808,6 +818,7 @@ void smb_msleep(unsigned int t)
 
 		tdiff = TvalDiff(&t1,&t2);
 	}
+#endif
 }
 
 /****************************************************************************
@@ -963,7 +974,7 @@ void *Realloc(void *p,size_t size)
 /****************************************************************************
  Type-safe realloc.
 ****************************************************************************/
-		
+
 void *realloc_array(void *p,size_t el_size, unsigned int count)
 {
 	if (count >= MAX_ALLOC_SIZE/el_size) {
@@ -972,6 +983,63 @@ void *realloc_array(void *p,size_t el_size, unsigned int count)
 	return Realloc(p,el_size*count);
 }
 
+/****************************************************************************
+ (Hopefully) efficient array append
+****************************************************************************/
+void add_to_large_array(TALLOC_CTX *mem_ctx, size_t element_size,
+			void *element, void **array, uint32 *num_elements,
+			ssize_t *array_size)
+{
+	if (*array_size < 0)
+		return;
+
+	if (*array == NULL) {
+		if (*array_size == 0) {
+			*array_size = 128;
+		}
+
+		if (*array_size >= MAX_ALLOC_SIZE/element_size) {
+			goto error;
+		}
+
+		if (mem_ctx != NULL)
+			*array = TALLOC(mem_ctx, element_size * (*array_size));
+		else
+			*array = SMB_MALLOC(element_size * (*array_size));
+
+		if (*array == NULL)
+			goto error;
+	}
+
+	if (*num_elements == *array_size) {
+		*array_size *= 2;
+
+		if (*array_size >= MAX_ALLOC_SIZE/element_size) {
+			goto error;
+		}
+
+		if (mem_ctx != NULL)
+			*array = TALLOC_REALLOC(mem_ctx, *array,
+						element_size * (*array_size));
+		else
+			*array = SMB_REALLOC(*array,
+					     element_size * (*array_size));
+
+		if (*array == NULL)
+			goto error;
+	}
+
+	memcpy((char *)(*array) + element_size*(*num_elements),
+	       element, element_size);
+	*num_elements += 1;
+
+	return;
+
+ error:
+	*num_elements = 0;
+	*array_size = -1;
+}
+
 /****************************************************************************
  Free memory, checks for NULL.
  Use directly SAFE_FREE()
@@ -1452,8 +1520,6 @@ void smb_panic2(const char *why, BOOL decrement_pid_count )
 
 #ifdef DEVELOPER
 	{
-		extern char *global_clobber_region_function;
-		extern unsigned int global_clobber_region_line;
 
 		if (global_clobber_region_function) {
 			DEBUG(0,("smb_panic: clobber_region() last called from [%s(%u)]\n",
@@ -1521,12 +1587,17 @@ void smb_panic2(const char *why, BOOL decrement_pid_count )
 		ZERO_ARRAY(names);
 		ZERO_ARRAY(namebuf);
 
+		/* We need to be root so we can open our /proc entry to walk
+		 * our stack. It also helps when we want to dump core.
+		 */
+		become_root();
+
 		for (i = 0; i < BACKTRACE_STACK_SIZE; i++) {
 			names[i] = namebuf + (i * NAMESIZE);
 		}
 
 		levels = trace_back_stack(0, addrs, names,
-				BACKTRACE_STACK_SIZE, NAMESIZE);
+				BACKTRACE_STACK_SIZE, NAMESIZE - 1);
 
 		DEBUG(0, ("BACKTRACE: %d stack frames:\n", levels));
 		for (i = 0; i < levels; i++) {
@@ -1728,6 +1799,9 @@ void free_namearray(name_compare_entry *name_array)
 	SAFE_FREE(name_array);
 }
 
+#undef DBGC_CLASS
+#define DBGC_CLASS DBGC_LOCKING
+
 /****************************************************************************
  Simple routine to do POSIX file locking. Cruft in NFS and 64->32 bit mapping
  is dealt with in posix.c
@@ -1778,6 +1852,9 @@ BOOL fcntl_lock(int fd, int op, SMB_OFF_T offset, SMB_OFF_T count, int type)
 	return(True);
 }
 
+#undef DBGC_CLASS
+#define DBGC_CLASS DBGC_ALL
+
 /*******************************************************************
  Is the name specified one of my netbios names.
  Returns true if it is equal, false otherwise.
@@ -1913,7 +1990,6 @@ void ra_lanman_string( const char *native_lanman )
 
 void set_remote_arch(enum remote_arch_types type)
 {
-	extern fstring remote_arch;
 	ra_type = type;
 	switch( type ) {
 	case RA_WFWG:
@@ -2137,52 +2213,18 @@ int set_maxfiles(int requested_max)
 #endif
 }
 
-/*****************************************************************
- Splits out the start of the key (HKLM or HKU) and the rest of the key.
-*****************************************************************/  
-
-BOOL reg_split_key(const char *full_keyname, uint32 *reg_type, char *key_name)
-{
-	pstring tmp;
-
-	if (!next_token(&full_keyname, tmp, "\\", sizeof(tmp)))
-		return False;
-
-	(*reg_type) = 0;
-
-	DEBUG(10, ("reg_split_key: hive %s\n", tmp));
-
-	if (strequal(tmp, "HKLM") || strequal(tmp, "HKEY_LOCAL_MACHINE"))
-		(*reg_type) = HKEY_LOCAL_MACHINE;
-	else if (strequal(tmp, "HKU") || strequal(tmp, "HKEY_USERS"))
-		(*reg_type) = HKEY_USERS;
-	else {
-		DEBUG(10,("reg_split_key: unrecognised hive key %s\n", tmp));
-		return False;
-	}
-	
-	if (next_token(&full_keyname, tmp, "\n\r", sizeof(tmp)))
-		fstrcpy(key_name, tmp);
-	else
-		key_name[0] = 0;
-
-	DEBUG(10, ("reg_split_key: name %s\n", key_name));
-
-	return True;
-}
-
 /*****************************************************************
  Possibly replace mkstemp if it is broken.
 *****************************************************************/  
 
-int smb_mkstemp(char *template)
+int smb_mkstemp(char *name_template)
 {
 #if HAVE_SECURE_MKSTEMP
-	return mkstemp(template);
+	return mkstemp(name_template);
 #else
 	/* have a reasonable go at emulating it. Hope that
 	   the system mktemp() isn't completly hopeless */
-	char *p = mktemp(template);
+	char *p = mktemp(name_template);
 	if (!p)
 		return -1;
 	return open(p, O_CREAT|O_EXCL|O_RDWR, 0600);
@@ -2239,6 +2281,7 @@ char *smb_xstrdup(const char *s)
 	if (!s1)
 		smb_panic("smb_xstrdup: malloc fail\n");
 	return s1;
+
 }
 
 /**
@@ -2408,6 +2451,12 @@ char *parent_dirname(const char *path)
 BOOL ms_has_wild(const char *s)
 {
 	char c;
+
+	if (lp_posix_pathnames()) {
+		/* With posix pathnames no characters are wild. */
+		return False;
+	}
+
 	while ((c = *s++)) {
 		switch (c) {
 		case '*':
@@ -2450,7 +2499,23 @@ BOOL mask_match(const char *string, char *pattern, BOOL is_case_sensitive)
 	if (strcmp(pattern,".") == 0)
 		return False;
 	
-	return ms_fnmatch(pattern, string, Protocol, is_case_sensitive) == 0;
+	return ms_fnmatch(pattern, string, Protocol <= PROTOCOL_LANMAN2, is_case_sensitive) == 0;
+}
+
+/*******************************************************************
+ A wrapper that handles case sensitivity and the special handling
+ of the ".." name. Varient that is only called by old search code which requires
+ pattern translation.
+*******************************************************************/
+
+BOOL mask_match_search(const char *string, char *pattern, BOOL is_case_sensitive)
+{
+	if (strcmp(string,"..") == 0)
+		string = ".";
+	if (strcmp(pattern,".") == 0)
+		return False;
+	
+	return ms_fnmatch(pattern, string, True, is_case_sensitive) == 0;
 }
 
 /*******************************************************************
@@ -2609,6 +2674,29 @@ void name_to_fqdn(fstring fqdn, const char *name)
 	}
 }
 
+/**********************************************************************
+ Extension to talloc_get_type: Abort on type mismatch
+***********************************************************************/
+
+void *talloc_check_name_abort(const void *ptr, const char *name)
+{
+	void *result;
+
+	if (ptr == NULL)
+		return NULL;
+
+	result = talloc_check_name(ptr, name);
+	if (result != NULL)
+		return result;
+
+	DEBUG(0, ("Talloc type mismatch, expected %s, got %s\n",
+		  name, talloc_get_name(ptr)));
+	smb_panic("aborting");
+	/* Keep the compiler happy */
+	return NULL;
+}
+
+
 #ifdef __INSURE__
 
 /*******************************************************************
@@ -2647,3 +2735,24 @@ int _Insure_trap_error(int a1, int a2, int a3, int a4, int a5, int a6)
 	return ret;
 }
 #endif
+
+uint32 map_share_mode_to_deny_mode(uint32 share_access, uint32 private_options)
+{
+	switch (share_access & ~FILE_SHARE_DELETE) {
+		case FILE_SHARE_NONE:
+			return DENY_ALL;
+		case FILE_SHARE_READ:
+			return DENY_WRITE;
+		case FILE_SHARE_WRITE:
+			return DENY_READ;
+		case FILE_SHARE_READ|FILE_SHARE_WRITE:
+			return DENY_NONE;
+	}
+	if (private_options & NTCREATEX_OPTIONS_PRIVATE_DENY_DOS) {
+		return DENY_DOS;
+	} else if (private_options & NTCREATEX_OPTIONS_PRIVATE_DENY_FCB) {
+		return DENY_FCB;
+	}
+
+	return (uint32)-1;
+}
