@@ -18,8 +18,6 @@
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
-#define NO_SYSLOG
-
 #include "includes.h"
 
 /****************************************************************************
@@ -127,6 +125,21 @@ BOOL cli_receive_smb(struct cli_state *cli)
 	return True;
 }
 
+static ssize_t write_socket(int fd, const char *buf, size_t len)
+{
+        ssize_t ret=0;
+                                                                                                                                            
+        DEBUG(6,("write_socket(%d,%d)\n",fd,(int)len));
+        ret = write_data(fd,buf,len);
+                                                                                                                                            
+        DEBUG(6,("write_socket(%d,%d) wrote %d\n",fd,(int)len,(int)ret));
+        if(ret <= 0)
+                DEBUG(0,("write_socket: Error writing %d bytes to socket %d: ERRNO = %s\n",
+                        (int)len, fd, strerror(errno) ));
+                                                                                                                                            
+        return(ret);
+}
+
 /****************************************************************************
  Send an smb to a fd.
 ****************************************************************************/
@@ -185,6 +198,8 @@ void cli_setup_packet(struct cli_state *cli)
 		flags2 = FLAGS2_LONG_PATH_COMPONENTS;
 		if (cli->capabilities & CAP_UNICODE)
 			flags2 |= FLAGS2_UNICODE_STRINGS;
+		if (cli->capabilities & CAP_DFS)
+			flags2 |= FLAGS2_DFS_PATHNAMES;
 		if (cli->capabilities & CAP_STATUS32)
 			flags2 |= FLAGS2_32_BIT_ERROR_CODES;
 		if (cli->use_spnego)
@@ -245,6 +260,7 @@ void cli_setup_signing_state(struct cli_state *cli, int signing_state)
 struct cli_state *cli_initialise(struct cli_state *cli)
 {
         BOOL alloced_cli = False;
+	int i;
 
 	/* Check the effective uid - make sure we are not setuid */
 	if (is_setuid_root()) {
@@ -279,10 +295,11 @@ struct cli_state *cli_initialise(struct cli_state *cli)
 	cli->inbuf = (char *)SMB_MALLOC(cli->bufsize+SAFETY_MARGIN);
 	cli->oplock_handler = cli_oplock_ack;
 	cli->case_sensitive = False;
+	cli->smb_rw_error = 0;
 
 	cli->use_spnego = lp_client_use_spnego();
 
-	cli->capabilities = CAP_UNICODE | CAP_STATUS32;
+	cli->capabilities = CAP_UNICODE | CAP_STATUS32 | CAP_DFS;
 
 	/* Set the CLI_FORCE_DOSERR environment variable to test
 	   client routines using DOS errors instead of STATUS32
@@ -315,8 +332,10 @@ struct cli_state *cli_initialise(struct cli_state *cli)
 	/* initialise signing */
 	cli_null_set_signing(cli);
 
-	cli->nt_pipe_fnum = 0;
-	cli->saved_netlogon_pipe_fnum = 0;
+	for (i=0; i<PI_MAX_PIPES; i++)
+		cli->pipes[i].fnum = 0;
+
+	cli->netlogon_pipe.fnum = 0;
 
 	cli->initialised = 1;
 	cli->allocated = alloced_cli;
@@ -344,14 +363,17 @@ close the session
 
 void cli_nt_session_close(struct cli_state *cli)
 {
-	if (cli->ntlmssp_pipe_state) {
-		ntlmssp_end(&cli->ntlmssp_pipe_state);
+	int i;
+
+	for (i=0; i<PI_MAX_PIPES; i++) {
+		if (cli->pipes[i].pipe_auth_flags & AUTH_PIPE_NTLMSSP) {
+			ntlmssp_end(&cli->pipes[i].ntlmssp_pipe_state);
+		}
+
+		if (cli->pipes[i].fnum != 0)
+			cli_close(cli, cli->pipes[i].fnum);
+		cli->pipes[i].fnum = 0;
 	}
-
-	if (cli->nt_pipe_fnum != 0)
-		cli_close(cli, cli->nt_pipe_fnum);
-
-	cli->nt_pipe_fnum = 0;
 	cli->pipe_idx = -1;
 }
 
@@ -361,9 +383,9 @@ close the NETLOGON session holding the session key for NETSEC
 
 void cli_nt_netlogon_netsec_session_close(struct cli_state *cli)
 {
-	if (cli->saved_netlogon_pipe_fnum != 0) {
-		cli_close(cli, cli->saved_netlogon_pipe_fnum);
-		cli->saved_netlogon_pipe_fnum = 0;
+	if (cli->netlogon_pipe.fnum != 0) {
+		cli_close(cli, cli->netlogon_pipe.fnum);
+		cli->netlogon_pipe.fnum = 0;
 	}
 }
 
@@ -398,8 +420,8 @@ void cli_close_connection(struct cli_state *cli)
 	data_blob_free(&cli->secblob);
 	data_blob_free(&cli->user_session_key);
 
-	if (cli->ntlmssp_pipe_state) 
-		ntlmssp_end(&cli->ntlmssp_pipe_state);
+	if (cli->pipes[cli->pipe_idx].pipe_auth_flags & AUTH_PIPE_NTLMSSP) 
+		ntlmssp_end(&cli->pipes[cli->pipe_idx].ntlmssp_pipe_state);
 
 	if (cli->mem_ctx) {
 		talloc_destroy(cli->mem_ctx);
