@@ -19,8 +19,6 @@
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
-#define NO_SYSLOG
-
 #include "includes.h"
 
 /****************************************************************************
@@ -167,6 +165,55 @@ static mode_t unix_filetype_from_wire(uint32 wire_type)
 	}
 }
 
+/****************************************************************************
+ Do a POSIX getfacl (UNIX extensions).
+****************************************************************************/
+
+BOOL cli_unix_getfacl(struct cli_state *cli, const char *name, size_t *prb_size, char **retbuf)
+{
+	unsigned int param_len = 0;
+	unsigned int data_len = 0;
+	uint16 setup = TRANSACT2_QPATHINFO;
+	char param[sizeof(pstring)+6];
+	char *rparam=NULL, *rdata=NULL;
+	char *p;
+
+	p = param;
+	memset(p, 0, 6);
+	SSVAL(p, 0, SMB_QUERY_POSIX_ACL);
+	p += 6;
+	p += clistr_push(cli, p, name, sizeof(pstring)-6, STR_TERMINATE);
+	param_len = PTR_DIFF(p, param);
+
+	if (!cli_send_trans(cli, SMBtrans2,
+		NULL,                        /* name */
+		-1, 0,                       /* fid, flags */
+		&setup, 1, 0,                /* setup, length, max */
+		param, param_len, 2,         /* param, length, max */
+		NULL,  0, cli->max_xmit      /* data, length, max */
+		)) {
+			return False;
+	}
+
+	if (!cli_receive_trans(cli, SMBtrans2,
+		&rparam, &param_len,
+		&rdata, &data_len)) {
+			return False;
+	}
+
+	if (data_len < 6) {
+		SAFE_FREE(rdata);
+		SAFE_FREE(rparam);
+		return False;
+	}
+
+	SAFE_FREE(rparam);
+	*retbuf = rdata;
+	*prb_size = (size_t)data_len;
+
+	return True;
+}
+
 /****************************************************************************
  Stat a file (UNIX extensions).
 ****************************************************************************/
@@ -213,12 +260,17 @@ BOOL cli_unix_stat(struct cli_state *cli, const char *name, SMB_STRUCT_STAT *sbu
 
 	sbuf->st_size = IVAL2_TO_SMB_BIG_UINT(rdata,0);     /* total size, in bytes */
 	sbuf->st_blocks = IVAL2_TO_SMB_BIG_UINT(rdata,8);   /* number of blocks allocated */
+#if defined (HAVE_STAT_ST_BLOCKS) && defined(STAT_ST_BLOCKSIZE)
 	sbuf->st_blocks /= STAT_ST_BLOCKSIZE;
+#else
+	/* assume 512 byte blocks */
+	sbuf->st_blocks /= 512;
+#endif
 	sbuf->st_ctime = interpret_long_date(rdata + 16);    /* time of last change */
 	sbuf->st_atime = interpret_long_date(rdata + 24);    /* time of last access */
 	sbuf->st_mtime = interpret_long_date(rdata + 32);    /* time of last modification */
-	sbuf->st_uid = IVAL(rdata,40);      /* user ID of owner */
-	sbuf->st_gid = IVAL(rdata,48);      /* group ID of owner */
+	sbuf->st_uid = (uid_t) IVAL(rdata,40);      /* user ID of owner */
+	sbuf->st_gid = (gid_t) IVAL(rdata,48);      /* group ID of owner */
 	sbuf->st_mode |= unix_filetype_from_wire(IVAL(rdata, 56));
 #if defined(HAVE_MAKEDEV)
 	{
@@ -651,7 +703,7 @@ int cli_nt_create_full(struct cli_state *cli, const char *fname,
 int cli_nt_create(struct cli_state *cli, const char *fname, uint32 DesiredAccess)
 {
 	return cli_nt_create_full(cli, fname, 0, DesiredAccess, 0,
-				FILE_SHARE_READ|FILE_SHARE_WRITE, FILE_EXISTS_OPEN, 0x0, 0x0);
+				FILE_SHARE_READ|FILE_SHARE_WRITE, FILE_OPEN, 0x0, 0x0);
 }
 
 /****************************************************************************
@@ -1022,7 +1074,7 @@ BOOL cli_unlock64(struct cli_state *cli, int fnum, SMB_BIG_UINT offset, SMB_BIG_
 ****************************************************************************/
 
 BOOL cli_getattrE(struct cli_state *cli, int fd, 
-		  uint16 *attr, SMB_BIG_UINT *size, 
+		  uint16 *attr, SMB_OFF_T *size, 
 		  time_t *c_time, time_t *a_time, time_t *m_time)
 {
 	memset(cli->outbuf,'\0',smb_size);
@@ -1054,15 +1106,15 @@ BOOL cli_getattrE(struct cli_state *cli, int fd,
 	}
 
 	if (c_time) {
-		*c_time = make_unix_date3(cli->inbuf+smb_vwv0);
+		*c_time = make_unix_date2(cli->inbuf+smb_vwv0);
 	}
 
 	if (a_time) {
-		*a_time = make_unix_date3(cli->inbuf+smb_vwv2);
+		*a_time = make_unix_date2(cli->inbuf+smb_vwv2);
 	}
 
 	if (m_time) {
-		*m_time = make_unix_date3(cli->inbuf+smb_vwv4);
+		*m_time = make_unix_date2(cli->inbuf+smb_vwv4);
 	}
 
 	return True;
@@ -1073,7 +1125,7 @@ BOOL cli_getattrE(struct cli_state *cli, int fd,
 ****************************************************************************/
 
 BOOL cli_getatr(struct cli_state *cli, const char *fname, 
-		uint16 *attr, size_t *size, time_t *t)
+		uint16 *attr, SMB_OFF_T *size, time_t *t)
 {
 	char *p;
 
@@ -1137,9 +1189,9 @@ BOOL cli_setattrE(struct cli_state *cli, int fd,
 	cli_setup_packet(cli);
 
 	SSVAL(cli->outbuf,smb_vwv0, fd);
-	put_dos_date3(cli->outbuf,smb_vwv1, c_time);
-	put_dos_date3(cli->outbuf,smb_vwv3, a_time);
-	put_dos_date3(cli->outbuf,smb_vwv5, m_time);
+	put_dos_date2(cli->outbuf,smb_vwv1, c_time);
+	put_dos_date2(cli->outbuf,smb_vwv3, a_time);
+	put_dos_date2(cli->outbuf,smb_vwv5, m_time);
 
 	p = smb_buf(cli->outbuf);
 	*p++ = 4;
