@@ -1,7 +1,8 @@
 /* 
    Unix SMB/CIFS implementation.
    client RAP calls
-   Copyright (C) Andrew Tridgell 1994-1998
+   Copyright (C) Andrew Tridgell         1994-1998
+   Copyright (C) Gerald (Jerry) Carter   2004
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -18,8 +19,6 @@
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
-#define NO_SYSLOG
-
 #include "includes.h"
 
 
@@ -381,7 +380,7 @@ send a qpathinfo call
 ****************************************************************************/
 BOOL cli_qpathinfo(struct cli_state *cli, const char *fname, 
 		   time_t *c_time, time_t *a_time, time_t *m_time, 
-		   size_t *size, uint16 *mode)
+		   SMB_OFF_T *size, uint16 *mode)
 {
 	unsigned int data_len = 0;
 	unsigned int param_len = 0;
@@ -456,12 +455,107 @@ BOOL cli_qpathinfo(struct cli_state *cli, const char *fname,
 	return True;
 }
 
+
+/****************************************************************************
+send a setpathinfo call
+****************************************************************************/
+BOOL cli_setpathinfo(struct cli_state *cli, const char *fname, 
+                     time_t c_time, time_t a_time, time_t m_time, uint16 mode)
+{
+	unsigned int data_len = 0;
+	unsigned int param_len = 0;
+	unsigned int rparam_len, rdata_len;
+	uint16 setup = TRANSACT2_SETPATHINFO;
+	pstring param;
+        pstring data;
+	char *rparam=NULL, *rdata=NULL;
+	int count=8;
+	BOOL ret;
+        void (*date_fn)(char *buf,int offset,time_t unixdate);
+	char *p;
+
+	memset(param, 0, sizeof(param));
+	memset(data, 0, sizeof(data));
+
+        p = param;
+
+        /* Add the information level */
+	SSVAL(p, 0, SMB_INFO_STANDARD);
+
+        /* Skip reserved */
+	p += 6;
+
+        /* Add the file name */
+	p += clistr_push(cli, p, fname, sizeof(pstring)-6, STR_TERMINATE);
+
+	param_len = PTR_DIFF(p, param);
+
+        p = data;
+
+	if (cli->win95) {
+		date_fn = put_dos_date;
+	} else {
+		date_fn = put_dos_date2;
+	}
+
+        /* Add the create, last access, and modification times */
+        (*date_fn)(p, 0, c_time);
+        (*date_fn)(p, 4, a_time);
+        (*date_fn)(p, 8, m_time);
+        p += 12;
+
+        /* Skip DataSize and AllocationSize */
+        p += 8;
+
+        /* Add attributes */
+        SSVAL(p, 0, mode);
+        p += 2;
+
+        /* Add EA size (none) */
+        SIVAL(p, 0, 0);
+        p += 4;
+
+        data_len = PTR_DIFF(p, data);
+
+	do {
+		ret = (cli_send_trans(cli, SMBtrans2, 
+				      NULL,           /* Name */
+				      -1, 0,          /* fid, flags */
+				      &setup, 1, 0,   /* setup, length, max */
+				      param, param_len, 10, /* param, length, max */
+				      data, data_len, cli->max_xmit /* data, length, max */
+				      ) &&
+		       cli_receive_trans(cli, SMBtrans2, 
+					 &rparam, &rparam_len,
+					 &rdata, &rdata_len));
+		if (!cli_is_dos_error(cli)) break;
+		if (!ret) {
+			/* we need to work around a Win95 bug - sometimes
+			   it gives ERRSRV/ERRerror temprarily */
+			uint8 eclass;
+			uint32 ecode;
+			cli_dos_error(cli, &eclass, &ecode);
+			if (eclass != ERRSRV || ecode != ERRerror) break;
+			smb_msleep(100);
+		}
+	} while (count-- && ret==False);
+
+	if (!ret) {
+		return False;
+	}
+
+	SAFE_FREE(rdata);
+	SAFE_FREE(rparam);
+	return True;
+}
+
+
 /****************************************************************************
 send a qpathinfo call with the SMB_QUERY_FILE_ALL_INFO info level
 ****************************************************************************/
 BOOL cli_qpathinfo2(struct cli_state *cli, const char *fname, 
 		    time_t *c_time, time_t *a_time, time_t *m_time, 
-		    time_t *w_time, size_t *size, uint16 *mode,
+		    time_t *w_time, SMB_OFF_T *size, uint16 *mode,
 		    SMB_INO_T *ino)
 {
 	unsigned int data_len = 0;
@@ -500,22 +594,22 @@ BOOL cli_qpathinfo2(struct cli_state *cli, const char *fname,
 	}
 
 	if (c_time) {
-		*c_time = interpret_long_date(rdata+0) - cli->serverzone;
+                *c_time = interpret_long_date(rdata+0);
 	}
 	if (a_time) {
-		*a_time = interpret_long_date(rdata+8) - cli->serverzone;
+		*a_time = interpret_long_date(rdata+8);
 	}
 	if (m_time) {
-		*m_time = interpret_long_date(rdata+16) - cli->serverzone;
+		*m_time = interpret_long_date(rdata+16);
 	}
 	if (w_time) {
-		*w_time = interpret_long_date(rdata+24) - cli->serverzone;
+		*w_time = interpret_long_date(rdata+24);
 	}
 	if (mode) {
 		*mode = SVAL(rdata, 32);
 	}
 	if (size) {
-		*size = IVAL(rdata, 48);
+                *size = IVAL2_TO_SMB_BIG_UINT(rdata,48);
 	}
 	if (ino) {
 		*ino = IVAL(rdata, 64);
@@ -545,11 +639,11 @@ BOOL cli_qfilename(struct cli_state *cli, int fnum,
 	SSVAL(param, 2, SMB_QUERY_FILE_NAME_INFO);
 
 	if (!cli_send_trans(cli, SMBtrans2, 
-                            NULL,                           /* name */
-                            -1, 0,                          /* fid, flags */
-                            &setup, 1, 0,                   /* setup, length, max */
-                            param, param_len, 2,            /* param, length, max */
-                            NULL, data_len, cli->max_xmit   /* data, length, max */
+                            NULL,                         /* name */
+                            -1, 0,                        /* fid, flags */
+                            &setup, 1, 0,                 /* setup, length, max */
+                            param, param_len, 2,          /* param, length, max */
+                            NULL, data_len, cli->max_xmit /* data, length, max */
                            )) {
 		return False;
 	}
@@ -574,7 +668,7 @@ BOOL cli_qfilename(struct cli_state *cli, int fnum,
 send a qfileinfo call
 ****************************************************************************/
 BOOL cli_qfileinfo(struct cli_state *cli, int fnum, 
-		   uint16 *mode, size_t *size,
+		   uint16 *mode, SMB_OFF_T *size,
 		   time_t *c_time, time_t *a_time, time_t *m_time, 
 		   time_t *w_time, SMB_INO_T *ino)
 {
@@ -595,11 +689,11 @@ BOOL cli_qfileinfo(struct cli_state *cli, int fnum,
 	SSVAL(param, 2, SMB_QUERY_FILE_ALL_INFO);
 
 	if (!cli_send_trans(cli, SMBtrans2, 
-                            NULL,                           /* name */
-                            -1, 0,                          /* fid, flags */
-                            &setup, 1, 0,                   /* setup, length, max */
-                            param, param_len, 2,            /* param, length, max */
-                            NULL, data_len, cli->max_xmit   /* data, length, max */
+                            NULL,                         /* name */
+                            -1, 0,                        /* fid, flags */
+                            &setup, 1, 0,                 /* setup, length, max */
+                            param, param_len, 2,          /* param, length, max */
+                            NULL, data_len, cli->max_xmit /* data, length, max */
                            )) {
 		return False;
 	}
@@ -630,7 +724,7 @@ BOOL cli_qfileinfo(struct cli_state *cli, int fnum,
 		*mode = SVAL(rdata, 32);
 	}
 	if (size) {
-		*size = IVAL(rdata, 48);
+                *size = IVAL2_TO_SMB_BIG_UINT(rdata,48);
 	}
 	if (ino) {
 		*ino = IVAL(rdata, 64);
@@ -641,9 +735,80 @@ BOOL cli_qfileinfo(struct cli_state *cli, int fnum,
 	return True;
 }
 
+
+/****************************************************************************
+send a qpathinfo BASIC_INFO call
+****************************************************************************/
+BOOL cli_qpathinfo_basic( struct cli_state *cli, const char *name, 
+                          SMB_STRUCT_STAT *sbuf, uint32 *attributes )
+{
+	unsigned int param_len = 0;
+	unsigned int data_len = 0;
+	uint16 setup = TRANSACT2_QPATHINFO;
+	char param[sizeof(pstring)+6];
+	char *rparam=NULL, *rdata=NULL;
+	char *p;
+	pstring path;
+	int len;
+	
+	/* send full paths to dfs root shares */
+	
+	if ( cli->dfsroot )
+		pstr_sprintf(path, "\\%s\\%s\\%s", cli->desthost, cli->share, name );
+	else
+		pstrcpy( path, name );
+	
+	/* cleanup */
+	
+	len = strlen( path );
+	if ( path[len] == '\\' )
+		path[len] = '\0';
+
+	p = param;
+	memset(p, 0, 6);
+	SSVAL(p, 0, SMB_QUERY_FILE_BASIC_INFO);
+	p += 6;
+	p += clistr_push(cli, p, path, sizeof(pstring)-6, STR_TERMINATE);
+	param_len = PTR_DIFF(p, param);
+
+	if (!cli_send_trans(cli, SMBtrans2,
+		NULL,                        /* name */
+		-1, 0,                       /* fid, flags */
+		&setup, 1, 0,                /* setup, length, max */
+		param, param_len, 2,         /* param, length, max */
+		NULL,  0, cli->max_xmit      /* data, length, max */
+		)) {
+			return False;
+	}
+
+	if (!cli_receive_trans(cli, SMBtrans2,
+		&rparam, &param_len,
+		&rdata, &data_len)) {
+			return False;
+	}
+
+	if (data_len < 36) {
+		SAFE_FREE(rdata);
+		SAFE_FREE(rparam);
+		return False;
+	}
+
+	sbuf->st_atime = interpret_long_date( rdata+8 );
+	sbuf->st_mtime = interpret_long_date( rdata+16 );
+	sbuf->st_ctime = interpret_long_date( rdata+24 );
+	
+	*attributes = IVAL( rdata, 32 );
+	
+	SAFE_FREE(rparam);
+	SAFE_FREE(rdata);
+	
+	return True;
+}
+
 /****************************************************************************
 send a qfileinfo call
 ****************************************************************************/
+
 BOOL cli_qfileinfo_test(struct cli_state *cli, int fnum, int level, char **poutdata, uint32 *poutlen)
 {
 	unsigned int data_len = 0;
