@@ -21,6 +21,9 @@
 
 #include "includes.h"
 
+extern struct in_addr lastip;
+extern int lastport;
+
 int num_good_sends = 0;
 int num_good_receives = 0;
 
@@ -354,7 +357,7 @@ static BOOL parse_alloc_res_rec(char *inbuf,int *offset,int length,
 {
 	int i;
 
-	*recs = (struct res_rec *)malloc(sizeof(**recs)*count);
+	*recs = SMB_MALLOC_ARRAY(struct res_rec, count);
 	if (!*recs)
 		return(False);
 
@@ -475,6 +478,11 @@ static BOOL parse_dgram(char *inbuf,int length,struct dgram_packet *dgram)
 	dgram->datasize = length-offset;
 	memcpy(dgram->data,inbuf+offset,dgram->datasize);
 
+	/* Paranioa. Ensure the last 2 bytes in the dgram buffer are
+	   zero. This should be true anyway, just enforce it for paranioa sake. JRA. */
+	SMB_ASSERT(dgram->datasize <= (sizeof(dgram->data)-2));
+	memset(&dgram->data[sizeof(dgram->data)-2], '\0', 2);
+
 	return(True);
 }
 
@@ -552,7 +560,7 @@ static struct packet_struct *copy_nmb_packet(struct packet_struct *packet)
 	struct nmb_packet *copy_nmb;
 	struct packet_struct *pkt_copy;
 
-	if(( pkt_copy = (struct packet_struct *)malloc(sizeof(*packet))) == NULL) {
+	if(( pkt_copy = SMB_MALLOC_P(struct packet_struct)) == NULL) {
 		DEBUG(0,("copy_nmb_packet: malloc fail.\n"));
 		return NULL;
 	}
@@ -575,22 +583,19 @@ static struct packet_struct *copy_nmb_packet(struct packet_struct *packet)
 	/* Now copy any resource records. */
 
 	if (nmb->answers) {
-		if((copy_nmb->answers = (struct res_rec *)
-					malloc(nmb->header.ancount * sizeof(struct res_rec))) == NULL)
+		if((copy_nmb->answers = SMB_MALLOC_ARRAY(struct res_rec,nmb->header.ancount)) == NULL)
 			goto free_and_exit;
 		memcpy((char *)copy_nmb->answers, (char *)nmb->answers, 
 				nmb->header.ancount * sizeof(struct res_rec));
 	}
 	if (nmb->nsrecs) {
-		if((copy_nmb->nsrecs = (struct res_rec *)
-					malloc(nmb->header.nscount * sizeof(struct res_rec))) == NULL)
+		if((copy_nmb->nsrecs = SMB_MALLOC_ARRAY(struct res_rec, nmb->header.nscount)) == NULL)
 			goto free_and_exit;
 		memcpy((char *)copy_nmb->nsrecs, (char *)nmb->nsrecs, 
 				nmb->header.nscount * sizeof(struct res_rec));
 	}
 	if (nmb->additional) {
-		if((copy_nmb->additional = (struct res_rec *)
-					malloc(nmb->header.arcount * sizeof(struct res_rec))) == NULL)
+		if((copy_nmb->additional = SMB_MALLOC_ARRAY(struct res_rec, nmb->header.arcount)) == NULL)
 			goto free_and_exit;
 		memcpy((char *)copy_nmb->additional, (char *)nmb->additional, 
 				nmb->header.arcount * sizeof(struct res_rec));
@@ -617,7 +622,7 @@ static struct packet_struct *copy_dgram_packet(struct packet_struct *packet)
 { 
 	struct packet_struct *pkt_copy;
 
-	if(( pkt_copy = (struct packet_struct *)malloc(sizeof(*packet))) == NULL) {
+	if(( pkt_copy = SMB_MALLOC_P(struct packet_struct)) == NULL) {
 		DEBUG(0,("copy_dgram_packet: malloc fail.\n"));
 		return NULL;
 	}
@@ -690,12 +695,10 @@ void free_packet(struct packet_struct *packet)
 struct packet_struct *parse_packet(char *buf,int length,
 				   enum packet_type packet_type)
 {
-	extern struct in_addr lastip;
-	extern int lastport;
 	struct packet_struct *p;
 	BOOL ok=False;
 
-	p = (struct packet_struct *)malloc(sizeof(*p));
+	p = SMB_MALLOC_P(struct packet_struct);
 	if (!p)
 		return(NULL);
 
@@ -1143,8 +1146,6 @@ void sort_query_replies(char *data, int n, struct in_addr ip)
 	qsort(data, n, 6, QSORT_CAST name_query_comp);
 }
 
-#define TRUNCATE_NETBIOS_NAME 1
-
 /*******************************************************************
  Convert, possibly using a stupid microsoft-ism which has destroyed
  the transport independence of netbios (for CIFS vendors that usually
@@ -1164,19 +1165,17 @@ char *dns_to_netbios_name(const char *dns_name)
 	StrnCpy(netbios_name, dns_name, MAX_NETBIOSNAME_LEN-1);
 	netbios_name[15] = 0;
 	
-#ifdef TRUNCATE_NETBIOS_NAME
 	/* ok.  this is because of a stupid microsoft-ism.  if the called host
 	   name contains a '.', microsoft clients expect you to truncate the
 	   netbios name up to and including the '.'  this even applies, by
 	   mistake, to workgroup (domain) names, which is _really_ daft.
 	 */
-	for (i = 0; i >= 15; i--) {
+	for (i = 0; i < 15; i++) {
 		if (netbios_name[i] == '.') {
 			netbios_name[i] = 0;
 			break;
 		}
 	}
-#endif /* TRUNCATE_NETBIOS_NAME */
 
 	return netbios_name;
 }
