@@ -3,6 +3,7 @@
    Locking functions
    Copyright (C) Andrew Tridgell 1992-2000
    Copyright (C) Jeremy Allison 1992-2000
+   Copyright (C) Volker Lendecke 2005
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -37,12 +38,19 @@
 #include "includes.h"
 uint16 global_smbpid;
 
+#undef DBGC_CLASS
+#define DBGC_CLASS DBGC_LOCKING
+
 /* the locking database handle */
 static TDB_CONTEXT *tdb;
+static TDB_CONTEXT *deferred_open_tdb;
 
 struct locking_data {
         union {
-                int num_share_mode_entries;
+		struct {
+			int num_share_mode_entries;
+			BOOL delete_on_close;
+		} s;
                 share_mode_entry dummy; /* Needed for alignment. */
         } u;
         /* the following two entries are implicit
@@ -62,27 +70,39 @@ static const char *lock_type_name(enum brl_type lock_type)
 
 /****************************************************************************
  Utility function called to see if a file region is locked.
- If check_self is True, then checks on our own fd with the same locking context
- are still made. If check_self is False, then checks are not made on our own fd
- with the same locking context are not made.
 ****************************************************************************/
 
 BOOL is_locked(files_struct *fsp,connection_struct *conn,
 	       SMB_BIG_UINT count,SMB_BIG_UINT offset, 
-	       enum brl_type lock_type, BOOL check_self)
+	       enum brl_type lock_type)
 {
 	int snum = SNUM(conn);
+	int strict_locking = lp_strict_locking(snum);
 	BOOL ret;
 	
 	if (count == 0)
 		return(False);
 
-	if (!lp_locking(snum) || !lp_strict_locking(snum))
+	if (!lp_locking(snum) || !strict_locking)
 		return(False);
 
-	ret = !brl_locktest(fsp->dev, fsp->inode, fsp->fnum,
-			     global_smbpid, sys_getpid(), conn->cnum, 
-			     offset, count, lock_type, check_self);
+	if (strict_locking == Auto) {
+		if  (EXCLUSIVE_OPLOCK_TYPE(fsp->oplock_type) && (lock_type == READ_LOCK || lock_type == WRITE_LOCK)) {
+			DEBUG(10,("is_locked: optimisation - exclusive oplock on file %s\n", fsp->fsp_name ));
+			ret = 0;
+		} else if (LEVEL_II_OPLOCK_TYPE(fsp->oplock_type) && (lock_type == READ_LOCK)) {
+			DEBUG(10,("is_locked: optimisation - level II oplock on file %s\n", fsp->fsp_name ));
+			ret = 0;
+		} else {
+			ret = !brl_locktest(fsp->dev, fsp->inode, fsp->fnum,
+				     global_smbpid, sys_getpid(), conn->cnum, 
+				     offset, count, lock_type);
+		}
+	} else {
+		ret = !brl_locktest(fsp->dev, fsp->inode, fsp->fnum,
+				global_smbpid, sys_getpid(), conn->cnum,
+				offset, count, lock_type);
+	}
 
 	DEBUG(10,("is_locked: brl start=%.0f len=%.0f %s for file %s\n",
 			(double)offset, (double)count, ret ? "locked" : "unlocked",
@@ -303,7 +323,21 @@ BOOL locking_init(int read_only)
 		DEBUG(0,("ERROR: Failed to initialise locking database\n"));
 		return False;
 	}
-	
+
+	if (!read_only && !deferred_open_tdb) {
+		deferred_open_tdb = tdb_open_log(lock_path("deferred_open.tdb"), 
+		       0, TDB_DEFAULT|TDB_CLEAR_IF_FIRST, 
+		       O_RDWR|O_CREAT,
+		       0644);
+
+		if (!deferred_open_tdb) {
+			DEBUG(0,("ERROR: Failed to initialise deferred open database\n"));
+			tdb_close(tdb);
+			tdb = NULL;
+			return False;
+		}
+	}
+
 	if (!posix_locking_init(read_only))
 		return False;
 
@@ -318,15 +352,20 @@ BOOL locking_init(int read_only)
 
 BOOL locking_end(void)
 {
+	BOOL ret = True;
 
 	brl_shutdown(open_read_only);
 	if (tdb) {
-
 		if (tdb_close(tdb) != 0)
-			return False;
+			ret = False;
 	}
 
-	return True;
+	if (deferred_open_tdb) {
+		if (tdb_close(deferred_open_tdb) != 0)
+			ret = False;
+	}
+		
+	return ret;
 }
 
 /*******************************************************************
@@ -397,10 +436,14 @@ char *share_mode_str(int num, share_mode_entry *e)
 {
 	static pstring share_str;
 
-	slprintf(share_str, sizeof(share_str)-1, "share_mode_entry[%d]: \
-pid = %lu, share_mode = 0x%x, desired_access = 0x%x, port = 0x%x, type= 0x%x, file_id = %lu, dev = 0x%x, inode = %.0f",
-	num, (unsigned long)e->pid, e->share_mode, (unsigned int)e->desired_access, e->op_port, e->op_type, e->share_file_id,
-	(unsigned int)e->dev, (double)e->inode );
+	slprintf(share_str, sizeof(share_str)-1, "share_mode_entry[%d]: "
+		 "pid = %lu, share_access = 0x%x, private_options = 0x%x, "
+		 "access_mask = 0x%x, port = 0x%x, type= 0x%x, file_id = %lu, "
+		 "dev = 0x%x, inode = %.0f",
+		 num, (unsigned long)e->pid,
+		 e->share_access, e->private_options,
+		 e->access_mask, e->op_port, e->op_type, e->share_file_id,
+		 (unsigned int)e->dev, (double)e->inode );
 
 	return share_str;
 }
@@ -411,7 +454,7 @@ pid = %lu, share_mode = 0x%x, desired_access = 0x%x, port = 0x%x, type= 0x%x, fi
 
 static void print_share_mode_table(struct locking_data *data)
 {
-	int num_share_modes = data->u.num_share_mode_entries;
+	int num_share_modes = data->u.s.num_share_mode_entries;
 	share_mode_entry *shares = (share_mode_entry *)(data + 1);
 	int i;
 
@@ -425,9 +468,9 @@ static void print_share_mode_table(struct locking_data *data)
  Get all share mode entries for a dev/inode pair.
 ********************************************************************/
 
-int get_share_modes(connection_struct *conn, 
-		    SMB_DEV_T dev, SMB_INO_T inode, 
-		    share_mode_entry **pp_shares)
+int get_share_modes(SMB_DEV_T dev, SMB_INO_T inode, 
+		    share_mode_entry **pp_shares,
+		    BOOL *delete_on_close)
 {
 	TDB_DATA dbuf;
 	struct locking_data *data;
@@ -435,13 +478,18 @@ int get_share_modes(connection_struct *conn,
 	share_mode_entry *shares = NULL;
 	TDB_DATA key = locking_key(dev, inode);
 	*pp_shares = NULL;
+	*delete_on_close = False;
 
 	dbuf = tdb_fetch(tdb, key);
 	if (!dbuf.dptr)
 		return 0;
 
 	data = (struct locking_data *)dbuf.dptr;
-	num_share_modes = data->u.num_share_mode_entries;
+
+	*delete_on_close = data->u.s.delete_on_close;
+	DEBUG(10, ("get_share_modes: delete_on_close: %d\n",
+		   *delete_on_close));
+	num_share_modes = data->u.s.num_share_mode_entries;
 	if(num_share_modes) {
 		pstring fname;
 		int i;
@@ -480,7 +528,7 @@ int get_share_modes(connection_struct *conn,
 
 		/* Did we delete any ? If so, re-store in tdb. */
 		if (del_count) {
-			data->u.num_share_mode_entries = num_share_modes;
+			data->u.s.num_share_mode_entries = num_share_modes;
 			
 			if (num_share_modes) {
 				memcpy(dbuf.dptr + sizeof(*data), shares,
@@ -492,7 +540,7 @@ int get_share_modes(connection_struct *conn,
 			/* The record has shrunk a bit */
 			dbuf.dsize -= del_count * sizeof(share_mode_entry);
 
-			if (data->u.num_share_mode_entries == 0) {
+			if (data->u.s.num_share_mode_entries == 0) {
 				if (tdb_delete(tdb, key) == -1) {
 					SAFE_FREE(shares);
 					SAFE_FREE(dbuf.dptr);
@@ -513,6 +561,15 @@ int get_share_modes(connection_struct *conn,
 	return num_share_modes;
 }
 
+BOOL get_delete_on_close_flag(SMB_DEV_T dev, SMB_INO_T inode)
+{
+	share_mode_entry *shares;
+	BOOL result;
+	get_share_modes(dev, inode, &shares, &result);
+	SAFE_FREE(shares);
+	return result;
+}
+
 /*******************************************************************
  Fill a share mode entry.
 ********************************************************************/
@@ -524,8 +581,9 @@ static void fill_share_mode(char *p, files_struct *fsp, uint16 port, uint16 op_t
 
 	memset(e, '\0', sizeof(share_mode_entry));
 	e->pid = sys_getpid();
-	e->share_mode = fsp->share_mode;
-	e->desired_access = fsp->desired_access;
+	e->share_access = fsp->share_access;
+	e->private_options = fsp->fh->private_options;
+	e->access_mask = fsp->access_mask;
 	e->op_port = port;
 	e->op_type = op_type;
 	memcpy(x, &fsp->open_time, sizeof(struct timeval));
@@ -541,8 +599,22 @@ static void fill_share_mode(char *p, files_struct *fsp, uint16 port, uint16 op_t
 
 BOOL share_modes_identical( share_mode_entry *e1, share_mode_entry *e2)
 {
+#if 1 /* JRA PARANOIA TEST - REMOVE LATER */
+	if (e1->pid == e2->pid &&
+		e1->share_file_id == e2->share_file_id &&
+		e1->dev == e2->dev &&
+		e1->inode == e2->inode &&
+		(e1->share_access) != (e2->share_access)) {
+			DEBUG(0,("PANIC: share_modes_identical: share_mode "
+				 "mismatch (e1 = 0x%x, e2 = 0x%x). Logic error.\n",
+				 (unsigned int)e1->share_access,
+				 (unsigned int)e2->share_access ));
+		smb_panic("PANIC: share_modes_identical logic error.\n");
+	}
+#endif
+
 	return (e1->pid == e2->pid &&
-		(e1->share_mode & ~DELETE_ON_CLOSE_FLAG) == (e2->share_mode & ~DELETE_ON_CLOSE_FLAG) &&
+		(e1->share_access) == (e2->share_access) &&
 		e1->dev == e2->dev &&
 		e1->inode == e2->inode &&
 		e1->share_file_id == e2->share_file_id );
@@ -554,8 +626,9 @@ BOOL share_modes_identical( share_mode_entry *e1, share_mode_entry *e2)
  Ignore if no entry deleted.
 ********************************************************************/
 
-ssize_t del_share_entry( SMB_DEV_T dev, SMB_INO_T inode,
-			share_mode_entry *entry, share_mode_entry **ppse)
+ssize_t del_share_entry(SMB_DEV_T dev, SMB_INO_T inode,
+			share_mode_entry *entry, share_mode_entry **ppse,
+			BOOL *delete_on_close)
 {
 	TDB_DATA dbuf;
 	struct locking_data *data;
@@ -573,6 +646,7 @@ ssize_t del_share_entry( SMB_DEV_T dev, SMB_INO_T inode,
 		return -1;
 
 	data = (struct locking_data *)dbuf.dptr;
+	*delete_on_close = data->u.s.delete_on_close;
 	shares = (share_mode_entry *)(dbuf.dptr + sizeof(*data));
 
 	/*
@@ -581,15 +655,15 @@ ssize_t del_share_entry( SMB_DEV_T dev, SMB_INO_T inode,
 	 * from the record.
 	 */
 
-	DEBUG(10,("del_share_entry: num_share_modes = %d\n", data->u.num_share_mode_entries ));
+	DEBUG(10,("del_share_entry: num_share_modes = %d\n", data->u.s.num_share_mode_entries ));
 
-	for (i=0;i<data->u.num_share_mode_entries;) {
+	for (i=0;i<data->u.s.num_share_mode_entries;) {
 		if (share_modes_identical(&shares[i], entry)) {
 			DEBUG(10,("del_share_entry: deleted %s\n",
 				share_mode_str(i, &shares[i]) ));
 			if (ppse)
 				*ppse = memdup(&shares[i], sizeof(*shares));
-			data->u.num_share_mode_entries--;
+			data->u.s.num_share_mode_entries--;
 			if ((dbuf.dsize - (sizeof(*data) + (i+1)*sizeof(*shares))) > 0) {
 				memmove(&shares[i], &shares[i+1], 
 					dbuf.dsize - (sizeof(*data) + (i+1)*sizeof(*shares)));
@@ -607,10 +681,10 @@ ssize_t del_share_entry( SMB_DEV_T dev, SMB_INO_T inode,
 		/* the record may have shrunk a bit */
 		dbuf.dsize -= del_count * sizeof(*shares);
 
-		count = (ssize_t)data->u.num_share_mode_entries;
+		count = (ssize_t)data->u.s.num_share_mode_entries;
 
 		/* store it back in the database */
-		if (data->u.num_share_mode_entries == 0) {
+		if (data->u.s.num_share_mode_entries == 0) {
 			if (tdb_delete(tdb, key) == -1)
 				count = -1;
 		} else {
@@ -629,7 +703,8 @@ ssize_t del_share_entry( SMB_DEV_T dev, SMB_INO_T inode,
  of entries left, and a memdup'ed copy of the entry deleted.
 ********************************************************************/
 
-ssize_t del_share_mode(files_struct *fsp, share_mode_entry **ppse)
+ssize_t del_share_mode(files_struct *fsp, share_mode_entry **ppse,
+		       BOOL *delete_on_close)
 {
 	share_mode_entry entry;
 
@@ -638,7 +713,8 @@ ssize_t del_share_mode(files_struct *fsp, share_mode_entry **ppse)
 	 */
 
 	fill_share_mode((char *)&entry, fsp, 0, 0);
-	return del_share_entry(fsp->dev, fsp->inode, &entry, ppse);
+	return del_share_entry(fsp->dev, fsp->inode, &entry, ppse,
+			       delete_on_close);
 }
 
 /*******************************************************************
@@ -670,7 +746,8 @@ BOOL set_share_mode(files_struct *fsp, uint16 port, uint16 op_type)
 		if (!p)
 			return False;
 		data = (struct locking_data *)p;
-		data->u.num_share_mode_entries = 1;
+		ZERO_STRUCT(data->u); /* Keep valgrind happy */
+		data->u.s.num_share_mode_entries = 1;
 	
 		DEBUG(10,("set_share_mode: creating entry for file %s. num_share_modes = 1\n",
 			fsp->fsp_name ));
@@ -692,10 +769,10 @@ BOOL set_share_mode(files_struct *fsp, uint16 port, uint16 op_type)
 	/* we're adding to an existing entry - this is a bit fiddly */
 	data = (struct locking_data *)dbuf.dptr;
 
-	data->u.num_share_mode_entries++;
+	data->u.s.num_share_mode_entries++;
 	
 	DEBUG(10,("set_share_mode: adding entry for file %s. new num_share_modes = %d\n",
-		fsp->fsp_name, data->u.num_share_mode_entries ));
+		fsp->fsp_name, data->u.s.num_share_mode_entries ));
 
 	size = dbuf.dsize + sizeof(share_mode_entry);
 	p = SMB_MALLOC(size);
@@ -742,7 +819,7 @@ static BOOL mod_share_mode( SMB_DEV_T dev, SMB_INO_T inode, share_mode_entry *en
 	shares = (share_mode_entry *)(dbuf.dptr + sizeof(*data));
 
 	/* find any with our pid and call the supplied function */
-	for (i=0;i<data->u.num_share_mode_entries;i++) {
+	for (i=0;i<data->u.s.num_share_mode_entries;i++) {
 		if (share_modes_identical(entry, &shares[i])) {
 			mod_fn(&shares[i], dev, inode, param);
 			need_store=True;
@@ -751,7 +828,7 @@ static BOOL mod_share_mode( SMB_DEV_T dev, SMB_INO_T inode, share_mode_entry *en
 
 	/* if the mod fn was called then store it back */
 	if (need_store) {
-		if (data->u.num_share_mode_entries == 0) {
+		if (data->u.s.num_share_mode_entries == 0) {
 			if (tdb_delete(tdb, key) == -1)
 				ret = False;
 		} else {
@@ -829,8 +906,7 @@ BOOL modify_delete_flag( SMB_DEV_T dev, SMB_INO_T inode, BOOL delete_on_close)
 {
 	TDB_DATA dbuf;
 	struct locking_data *data;
-	int i;
-	share_mode_entry *shares;
+	BOOL res;
 	TDB_DATA key = locking_key(dev, inode);
 
 	/* read in the existing share modes */
@@ -839,25 +915,14 @@ BOOL modify_delete_flag( SMB_DEV_T dev, SMB_INO_T inode, BOOL delete_on_close)
 		return False;
 
 	data = (struct locking_data *)dbuf.dptr;
-	shares = (share_mode_entry *)(dbuf.dptr + sizeof(*data));
 
 	/* Set/Unset the delete on close element. */
-	for (i=0;i<data->u.num_share_mode_entries;i++,shares++) {
-		shares->share_mode = (delete_on_close ?
-                            (shares->share_mode | DELETE_ON_CLOSE_FLAG) :
-                            (shares->share_mode & ~DELETE_ON_CLOSE_FLAG) );
-	}
+	data->u.s.delete_on_close = delete_on_close;
 
-	/* store it back */
-	if (data->u.num_share_mode_entries) {
-		if (tdb_store(tdb, key, dbuf, TDB_REPLACE)==-1) {
-			SAFE_FREE(dbuf.dptr);
-			return False;
-		}
-	}
+	res = (tdb_store(tdb, key, dbuf, TDB_REPLACE)!=-1);
 
 	SAFE_FREE(dbuf.dptr);
-	return True;
+	return res;
 }
 
 /*******************************************************************
@@ -912,6 +977,7 @@ static void print_deferred_open_table(struct deferred_open_data *data)
 	}
 }
 
+
 /*******************************************************************
  Form a static deferred open locking key for a dev/inode pair.
 ******************************************************************/
@@ -946,7 +1012,7 @@ int get_deferred_opens(connection_struct *conn,
 
 	*pp_de_entries = NULL;
 
-	dbuf = tdb_fetch(tdb, key);
+	dbuf = tdb_fetch(deferred_open_tdb, key);
 	if (!dbuf.dptr)
 		return 0;
 
@@ -1003,13 +1069,13 @@ int get_deferred_opens(connection_struct *conn,
 			dbuf.dsize -= del_count * sizeof(deferred_open_entry);
 
 			if (data->u.num_deferred_open_entries == 0) {
-				if (tdb_delete(tdb, key) == -1) {
+				if (tdb_delete(deferred_open_tdb, key) == -1) {
 					SAFE_FREE(de_entries);
 					SAFE_FREE(dbuf.dptr);
 					return 0;
 				}
 			} else {
-				if (tdb_store(tdb, key, dbuf, TDB_REPLACE) == -1) {
+				if (tdb_store(deferred_open_tdb, key, dbuf, TDB_REPLACE) == -1) {
 					SAFE_FREE(de_entries);
 					SAFE_FREE(dbuf.dptr);
 					return 0;
@@ -1053,7 +1119,7 @@ BOOL delete_deferred_open_entry(deferred_open_entry *entry)
 	TDB_DATA key = deferred_open_locking_key(entry->dev, entry->inode);
 
 	/* read in the existing share modes */
-	dbuf = tdb_fetch(tdb, key);
+	dbuf = tdb_fetch(deferred_open_tdb, key);
 	if (!dbuf.dptr)
 		return -1;
 
@@ -1096,10 +1162,10 @@ BOOL delete_deferred_open_entry(deferred_open_entry *entry)
 
 		/* store it back in the database */
 		if (data->u.num_deferred_open_entries == 0) {
-			if (tdb_delete(tdb, key) == -1)
+			if (tdb_delete(deferred_open_tdb, key) == -1)
 				ret = False;
 		} else {
-			if (tdb_store(tdb, key, dbuf, TDB_REPLACE) == -1)
+			if (tdb_store(deferred_open_tdb, key, dbuf, TDB_REPLACE) == -1)
 				ret = False;
 		}
 	}
@@ -1141,7 +1207,7 @@ BOOL add_deferred_open(uint16 mid, struct timeval *ptv, SMB_DEV_T dev, SMB_INO_T
 	BOOL ret = True;
 		
 	/* read in the existing deferred open records if any */
-	dbuf = tdb_fetch(tdb, key);
+	dbuf = tdb_fetch(deferred_open_tdb, key);
 	if (!dbuf.dptr) {
 		size_t offset;
 		/* we'll need to create a new record */
@@ -1151,6 +1217,7 @@ BOOL add_deferred_open(uint16 mid, struct timeval *ptv, SMB_DEV_T dev, SMB_INO_T
 		if (!p)
 			return False;
 		data = (struct deferred_open_data *)p;
+		ZERO_STRUCT(data->u.dummy); /* Keep valgrind happy */
 		data->u.num_deferred_open_entries = 1;
 	
 		DEBUG(10,("add_deferred_open: creating entry for file %s. num_deferred_open_entries = 1\n",
@@ -1161,7 +1228,7 @@ BOOL add_deferred_open(uint16 mid, struct timeval *ptv, SMB_DEV_T dev, SMB_INO_T
 		fill_deferred_open(p + sizeof(*data), mid, ptv, dev, inode, port);
 		dbuf.dptr = p;
 		dbuf.dsize = size;
-		if (tdb_store(tdb, key, dbuf, TDB_REPLACE) == -1)
+		if (tdb_store(deferred_open_tdb, key, dbuf, TDB_REPLACE) == -1)
 			ret = False;
 
 		print_deferred_open_table((struct deferred_open_data *)p);
@@ -1191,7 +1258,7 @@ BOOL add_deferred_open(uint16 mid, struct timeval *ptv, SMB_DEV_T dev, SMB_INO_T
 	SAFE_FREE(dbuf.dptr);
 	dbuf.dptr = p;
 	dbuf.dsize = size;
-	if (tdb_store(tdb, key, dbuf, TDB_REPLACE) == -1)
+	if (tdb_store(deferred_open_tdb, key, dbuf, TDB_REPLACE) == -1)
 		ret = False;
 	print_deferred_open_table((struct deferred_open_data *)p);
 	SAFE_FREE(p);
@@ -1219,9 +1286,9 @@ static int traverse_fn(TDB_CONTEXT *the_tdb, TDB_DATA kbuf, TDB_DATA dbuf,
 
 	data = (struct locking_data *)dbuf.dptr;
 	shares = (share_mode_entry *)(dbuf.dptr + sizeof(*data));
-	name = dbuf.dptr + sizeof(*data) + data->u.num_share_mode_entries*sizeof(*shares);
+	name = dbuf.dptr + sizeof(*data) + data->u.s.num_share_mode_entries*sizeof(*shares);
 
-	for (i=0;i<data->u.num_share_mode_entries;i++) {
+	for (i=0;i<data->u.s.num_share_mode_entries;i++) {
 		traverse_callback(&shares[i], name);
 	}
 	return 0;
