@@ -864,6 +864,7 @@ void reply_netbios_packet(struct packet_struct *orig_packet,
 	struct res_rec answers;
 	struct nmb_packet *orig_nmb = &orig_packet->packet.nmb;
 	BOOL loopback_this_packet = False;
+	int rr_type = RR_TYPE_NB;
 	const char *packet_type = "unknown";
   
 	/* Check if we are sending to or from ourselves. */
@@ -885,11 +886,15 @@ void reply_netbios_packet(struct packet_struct *orig_packet,
 			packet_type = "nmb_status";
 			nmb->header.nm_flags.recursion_desired = False;
 			nmb->header.nm_flags.recursion_available = False;
+			rr_type = RR_TYPE_NBSTAT;
 			break;
 		case NMB_QUERY:
 			packet_type = "nmb_query";
 			nmb->header.nm_flags.recursion_desired = True;
 			nmb->header.nm_flags.recursion_available = True;
+			if (rcode) {
+				rr_type = RR_TYPE_NULL;
+			}
 			break;
 		case NMB_REG:
 		case NMB_REG_REFRESH:
@@ -906,6 +911,7 @@ void reply_netbios_packet(struct packet_struct *orig_packet,
 			packet_type = "nmb_wack";
 			nmb->header.nm_flags.recursion_desired = False;
 			nmb->header.nm_flags.recursion_available = False;
+			rr_type = RR_TYPE_NULL;
 			break;
 		case WINS_REG:
 			packet_type = "wins_reg";
@@ -916,6 +922,9 @@ void reply_netbios_packet(struct packet_struct *orig_packet,
 			packet_type = "wins_query";
 			nmb->header.nm_flags.recursion_desired = True;
 			nmb->header.nm_flags.recursion_available = True;
+			if (rcode) {
+				rr_type = RR_TYPE_NULL;
+			}
 			break;
 		default:
 			DEBUG(0,("reply_netbios_packet: Unknown packet type: %s %s to ip %s\n",
@@ -947,8 +956,8 @@ for id %hu\n", packet_type, nmb_namestr(&orig_nmb->question.question_name),
 	memset((char*)nmb->answers,'\0',sizeof(*nmb->answers));
   
 	nmb->answers->rr_name  = orig_nmb->question.question_name;
-	nmb->answers->rr_type  = orig_nmb->question.question_type;
-	nmb->answers->rr_class = orig_nmb->question.question_class;
+	nmb->answers->rr_type  = rr_type;
+	nmb->answers->rr_class = RR_CLASS_IN;
 	nmb->answers->ttl      = ttl;
   
 	if (data && len) {
@@ -1639,7 +1648,7 @@ on subnet %s\n", rrec->response_id, inet_ntoa(rrec->packet->ip), subrec->subnet_
   plus the broadcast sockets.
 ***************************************************************************/
 
-static BOOL create_listen_fdset(fd_set **ppset, int **psock_array, int *listen_number)
+static BOOL create_listen_fdset(fd_set **ppset, int **psock_array, int *listen_number, int *maxfd)
 {
 	int *sock_array = NULL;
 	struct subnet_record *subrec = NULL;
@@ -1672,21 +1681,25 @@ only use %d.\n", (count*2) + 2, FD_SETSIZE));
 	/* Add in the broadcast socket on 137. */
 	FD_SET(ClientNMB,pset);
 	sock_array[num++] = ClientNMB;
+	*maxfd = MAX( *maxfd, ClientNMB);
 
 	/* Add in the 137 sockets on all the interfaces. */
 	for (subrec = FIRST_SUBNET; subrec; subrec = NEXT_SUBNET_EXCLUDING_UNICAST(subrec)) {
 		FD_SET(subrec->nmb_sock,pset);
 		sock_array[num++] = subrec->nmb_sock;
+		*maxfd = MAX( *maxfd, subrec->nmb_sock);
 	}
 
 	/* Add in the broadcast socket on 138. */
 	FD_SET(ClientDGRAM,pset);
 	sock_array[num++] = ClientDGRAM;
+	*maxfd = MAX( *maxfd, ClientDGRAM);
 
 	/* Add in the 138 sockets on all the interfaces. */
 	for (subrec = FIRST_SUBNET; subrec; subrec = NEXT_SUBNET_EXCLUDING_UNICAST(subrec)) {
 		FD_SET(subrec->dgram_sock,pset);
 		sock_array[num++] = subrec->dgram_sock;
+		*maxfd = MAX( *maxfd, subrec->dgram_sock);
 	}
 
 	*listen_number = (count*2) + 2;
@@ -1711,6 +1724,7 @@ BOOL listen_for_packets(BOOL run_election)
 	static int listen_number = 0;
 	static int *sock_array = NULL;
 	int i;
+	static int maxfd = 0;
 
 	fd_set fds;
 	int selrtn;
@@ -1720,7 +1734,7 @@ BOOL listen_for_packets(BOOL run_election)
 #endif
 
 	if(listen_set == NULL || rescan_listen_set) {
-		if(create_listen_fdset(&listen_set, &sock_array, &listen_number)) {
+		if(create_listen_fdset(&listen_set, &sock_array, &listen_number, &maxfd)) {
 			DEBUG(0,("listen_for_packets: Fatal error. unable to create listen set. Exiting.\n"));
 			return True;
 		}
@@ -1733,6 +1747,7 @@ BOOL listen_for_packets(BOOL run_election)
 	dns_fd = asyncdns_fd();
 	if (dns_fd != -1) {
 		FD_SET(dns_fd, &fds);
+		maxfd = MAX( maxfd, dns_fd);
 	}
 #endif
 
@@ -1750,7 +1765,7 @@ BOOL listen_for_packets(BOOL run_election)
 
 	BlockSignals(False, SIGTERM);
 
-	selrtn = sys_select(FD_SETSIZE,&fds,NULL,NULL,&timeout);
+	selrtn = sys_select(maxfd+1,&fds,NULL,NULL,&timeout);
 
 	/* We can only take signals when we are in the select - block them again here. */
 
