@@ -278,7 +278,7 @@ static struct group *wb_aix_getgrgid(gid_t gid)
 	
 	request.data.gid = gid;
 
-	ret = winbindd_request(WINBINDD_GETGRGID, &request, &response);
+	ret = winbindd_request_response(WINBINDD_GETGRGID, &request, &response);
 
 	logit("getgrgid ret=%d\n", ret);
 
@@ -310,7 +310,7 @@ static struct group *wb_aix_getgrnam(const char *name)
 
 	STRCPY_RETNULL(request.data.groupname, name);
 
-	ret = winbindd_request(WINBINDD_GETGRNAM, &request, &response);
+	ret = winbindd_request_response(WINBINDD_GETGRNAM, &request, &response);
 	
 	HANDLE_ERRORS(ret);
 
@@ -366,7 +366,7 @@ static char *wb_aix_getgrset(char *user)
 		free(r_user);
 	}
 
-	ret = winbindd_request(WINBINDD_GETGROUPS, &request, &response);
+	ret = winbindd_request_response(WINBINDD_GETGROUPS, &request, &response);
 
 	HANDLE_ERRORS(ret);
 
@@ -405,7 +405,7 @@ static struct passwd *wb_aix_getpwuid(uid_t uid)
 		
 	request.data.uid = uid;
 	
-	ret = winbindd_request(WINBINDD_GETPWUID, &request, &response);
+	ret = winbindd_request_response(WINBINDD_GETPWUID, &request, &response);
 
 	HANDLE_ERRORS(ret);
 
@@ -438,7 +438,7 @@ static struct passwd *wb_aix_getpwnam(const char *name)
 
 	STRCPY_RETNULL(request.data.username, name);
 
-	ret = winbindd_request(WINBINDD_GETPWNAM, &request, &response);
+	ret = winbindd_request_response(WINBINDD_GETPWNAM, &request, &response);
 
 	HANDLE_ERRORS(ret);
 	
@@ -471,7 +471,7 @@ static int wb_aix_lsuser(char *attributes[], attrval_t results[], int size)
 	ZERO_STRUCT(request);
 	ZERO_STRUCT(response);
 	
-	ret = winbindd_request(WINBINDD_LIST_USERS, &request, &response);
+	ret = winbindd_request_response(WINBINDD_LIST_USERS, &request, &response);
 	if (ret != 0) {
 		errno = EINVAL;
 		return -1;
@@ -519,7 +519,7 @@ static int wb_aix_lsgroup(char *attributes[], attrval_t results[], int size)
 	ZERO_STRUCT(request);
 	ZERO_STRUCT(response);
 	
-	ret = winbindd_request(WINBINDD_LIST_GROUPS, &request, &response);
+	ret = winbindd_request_response(WINBINDD_LIST_GROUPS, &request, &response);
 	if (ret != 0) {
 		errno = EINVAL;
 		return -1;
@@ -600,7 +600,7 @@ static attrval_t pwd_to_sid(struct passwd *pwd)
 
 	request.data.uid = pwd->pw_uid;
 
-	if (winbindd_request(WINBINDD_UID_TO_SID, &request, &response) !=
+	if (winbindd_request_response(WINBINDD_UID_TO_SID, &request, &response) !=
 	    NSS_STATUS_SUCCESS) {
 		r.attr_flag = ENOENT;
 	} else {
@@ -834,7 +834,7 @@ static int wb_aix_authenticate(char *user, char *pass,
 		free(r_user);
 	}
 
-	result = winbindd_request(WINBINDD_PAM_AUTH, &request, &response);
+	result = winbindd_request_response(WINBINDD_PAM_AUTH, &request, &response);
 
 	free_response(&response);
 
@@ -883,7 +883,7 @@ static int wb_aix_chpass(char *user, char *oldpass, char *newpass, char **messag
 		free(r_user);
 	}
 
-	result = winbindd_request(WINBINDD_PAM_CHAUTHTOK, &request, &response);
+	result = winbindd_request_response(WINBINDD_PAM_CHAUTHTOK, &request, &response);
 
 	free_response(&response);
 
