@@ -5,6 +5,7 @@
 
    Copyright (C) Andrew Tridgell 2001
    Copyright (C) Gerald Carter   2003
+   Copyright (C) Volker Lendecke 2005
    
    
    This program is free software; you can redistribute it and/or modify
@@ -100,8 +101,13 @@ static struct winbind_cache *get_cache(struct winbindd_domain *domain)
 {
 	struct winbind_cache *ret = wcache;
 
+	/* we have to know what type of domain we are dealing with first */
+
+	if ( !domain->initialized )
+		set_dc_type_and_flags( domain );
+
 	if (!domain->backend) {
-		extern struct winbindd_methods msrpc_methods;
+		extern struct winbindd_methods reconnect_methods;
 		switch (lp_security()) {
 #ifdef HAVE_ADS
 		case SEC_ADS: {
@@ -129,7 +135,7 @@ static struct winbind_cache *get_cache(struct winbindd_domain *domain)
 		default:
 			DEBUG(5,("get_cache: Setting MS-RPC methods for domain %s\n",
 				domain->name));
-			domain->backend = &msrpc_methods;
+			domain->backend = &reconnect_methods;
 		}
 	}
 
@@ -209,7 +215,10 @@ static char *centry_string(struct cache_entry *centry, TALLOC_CTX *mem_ctx)
 		smb_panic("centry_string");
 	}
 
-	ret = TALLOC(mem_ctx, len+1);
+	if (mem_ctx != NULL)
+		ret = TALLOC(mem_ctx, len+1);
+	else
+		ret = SMB_MALLOC(len+1);
 	if (!ret) {
 		smb_panic("centry_string out of memory\n");
 	}
@@ -222,20 +231,15 @@ static char *centry_string(struct cache_entry *centry, TALLOC_CTX *mem_ctx)
 /* pull a string from a cache entry, using the supplied
    talloc context 
 */
-static DOM_SID *centry_sid(struct cache_entry *centry, TALLOC_CTX *mem_ctx)
+static BOOL centry_sid(struct cache_entry *centry, DOM_SID *sid)
 {
-	DOM_SID *sid;
 	char *sid_string;
-
-	sid = TALLOC_P(mem_ctx, DOM_SID);
-	if (!sid)
-		return NULL;
-	
-	sid_string = centry_string(centry, mem_ctx);
+	sid_string = centry_string(centry, NULL);
 	if (!string_to_sid(sid, sid_string)) {
-		return NULL;
+		return False;
 	}
-	return sid;
+	SAFE_FREE(sid_string);
+	return True;
 }
 
 /* the server is considered down if it can't give us a sequence number */
@@ -366,9 +370,6 @@ static void refresh_sequence_number(struct winbindd_domain *domain, BOOL force)
 	/* important! make sure that we know if this is a native 
 	   mode domain or not */
 
-	if ( !domain->initialized )
-		set_dc_type_and_flags( domain );
-
 	status = domain->backend->sequence_number(domain, &domain->sequence_number);
 
 	if (!NT_STATUS_IS_OK(status)) {
@@ -468,21 +469,12 @@ static struct cache_entry *wcache_fetch(struct winbind_cache *cache,
 	centry->sequence_number = centry_uint32(centry);
 
 	if (centry_expired(domain, kstr, centry)) {
-		extern BOOL opt_dual_daemon;
-
 		DEBUG(10,("wcache_fetch: entry %s expired for domain %s\n",
 			 kstr, domain->name ));
 
-		if (opt_dual_daemon) {
-			extern BOOL background_process;
-			background_process = True;
-			DEBUG(10,("wcache_fetch: background processing expired entry %s for domain %s\n",
-				 kstr, domain->name ));
-		} else {
-			centry_free(centry);
-			free(kstr);
-			return NULL;
-		}
+		centry_free(centry);
+		free(kstr);
+		return NULL;
 	}
 
 	DEBUG(10,("wcache_fetch: returning entry %s for domain %s\n",
@@ -653,9 +645,11 @@ static void wcache_save_user(struct winbindd_domain *domain, NTSTATUS status, WI
 		return;
 	centry_put_string(centry, info->acct_name);
 	centry_put_string(centry, info->full_name);
-	centry_put_sid(centry, info->user_sid);
-	centry_put_sid(centry, info->group_sid);
-	centry_end(centry, "U/%s", sid_to_string(sid_string, info->user_sid));
+	centry_put_string(centry, info->homedir);
+	centry_put_string(centry, info->shell);
+	centry_put_sid(centry, &info->user_sid);
+	centry_put_sid(centry, &info->group_sid);
+	centry_end(centry, "U/%s", sid_to_string(sid_string, &info->user_sid));
 	DEBUG(10,("wcache_save_user: %s (acct_name %s)\n", sid_string, info->acct_name));
 	centry_free(centry);
 }
@@ -690,8 +684,10 @@ static NTSTATUS query_user_list(struct winbindd_domain *domain,
 	for (i=0; i<(*num_entries); i++) {
 		(*info)[i].acct_name = centry_string(centry, mem_ctx);
 		(*info)[i].full_name = centry_string(centry, mem_ctx);
-		(*info)[i].user_sid = centry_sid(centry, mem_ctx);
-		(*info)[i].group_sid = centry_sid(centry, mem_ctx);
+		(*info)[i].homedir = centry_string(centry, mem_ctx);
+		(*info)[i].shell = centry_string(centry, mem_ctx);
+		centry_sid(centry, &(*info)[i].user_sid);
+		centry_sid(centry, &(*info)[i].group_sid);
 	}
 
 do_cached:	
@@ -728,10 +724,12 @@ do_query:
 
 		status = domain->backend->query_user_list(domain, mem_ctx, num_entries, info);
 		if (!NT_STATUS_IS_OK(status))
-			DEBUG(3, ("query_user_list: returned 0x%08x, retrying\n", NT_STATUS_V(status)));
-			if (NT_STATUS_V(status) == NT_STATUS_V(NT_STATUS_UNSUCCESSFUL)) {
-				DEBUG(3, ("query_user_list: flushing connection cache\n"));
-				winbindd_cm_flush();
+			DEBUG(3, ("query_user_list: returned 0x%08x, "
+				  "retrying\n", NT_STATUS_V(status)));
+			if (NT_STATUS_EQUAL(status, NT_STATUS_UNSUCCESSFUL)) {
+				DEBUG(3, ("query_user_list: flushing "
+					  "connection cache\n"));
+				invalidate_cm_connection(&domain->conn);
 			}
 
 	} while (NT_STATUS_V(status) == NT_STATUS_V(NT_STATUS_UNSUCCESSFUL) && 
@@ -746,17 +744,19 @@ do_query:
 	for (i=0; i<(*num_entries); i++) {
 		centry_put_string(centry, (*info)[i].acct_name);
 		centry_put_string(centry, (*info)[i].full_name);
-		centry_put_sid(centry, (*info)[i].user_sid);
-		centry_put_sid(centry, (*info)[i].group_sid);
+		centry_put_string(centry, (*info)[i].homedir);
+		centry_put_string(centry, (*info)[i].shell);
+		centry_put_sid(centry, &(*info)[i].user_sid);
+		centry_put_sid(centry, &(*info)[i].group_sid);
 		if (domain->backend->consistent) {
 			/* when the backend is consistent we can pre-prime some mappings */
 			wcache_save_name_to_sid(domain, NT_STATUS_OK, 
 						domain->name,
 						(*info)[i].acct_name, 
-						(*info)[i].user_sid,
+						&(*info)[i].user_sid,
 						SID_NAME_USER);
 			wcache_save_sid_to_name(domain, NT_STATUS_OK, 
-						(*info)[i].user_sid,
+						&(*info)[i].user_sid,
 						domain->name,
 						(*info)[i].acct_name, 
 						SID_NAME_USER);
@@ -938,7 +938,6 @@ static NTSTATUS name_to_sid(struct winbindd_domain *domain,
 	struct cache_entry *centry = NULL;
 	NTSTATUS status;
 	fstring uname;
-	DOM_SID *sid2;
 
 	if (!cache->tdb)
 		goto do_query;
@@ -949,13 +948,7 @@ static NTSTATUS name_to_sid(struct winbindd_domain *domain,
 	if (!centry)
 		goto do_query;
 	*type = (enum SID_NAME_USE)centry_uint32(centry);
-	sid2 = centry_sid(centry, mem_ctx);
-	if (!sid2) {
-		ZERO_STRUCTP(sid);
-	} else {
-		sid_copy(sid, sid2);
-	}
-
+	centry_sid(centry, sid);
 	status = centry->status;
 
 	DEBUG(10,("name_to_sid: [Cached] - cached name for domain %s status %s\n",
@@ -1088,8 +1081,10 @@ static NTSTATUS query_user(struct winbindd_domain *domain,
 
 	info->acct_name = centry_string(centry, mem_ctx);
 	info->full_name = centry_string(centry, mem_ctx);
-	info->user_sid = centry_sid(centry, mem_ctx);
-	info->group_sid = centry_sid(centry, mem_ctx);
+	info->homedir = centry_string(centry, mem_ctx);
+	info->shell = centry_string(centry, mem_ctx);
+	centry_sid(centry, &info->user_sid);
+	centry_sid(centry, &info->group_sid);
 	status = centry->status;
 
 	DEBUG(10,("query_user: [Cached] - cached info for domain %s status %s\n",
@@ -1123,7 +1118,7 @@ do_query:
 static NTSTATUS lookup_usergroups(struct winbindd_domain *domain,
 				  TALLOC_CTX *mem_ctx,
 				  const DOM_SID *user_sid, 
-				  uint32 *num_groups, DOM_SID ***user_gids)
+				  uint32 *num_groups, DOM_SID **user_gids)
 {
 	struct winbind_cache *cache = get_cache(domain);
 	struct cache_entry *centry = NULL;
@@ -1156,11 +1151,11 @@ static NTSTATUS lookup_usergroups(struct winbindd_domain *domain,
 	if (*num_groups == 0)
 		goto do_cached;
 
-	(*user_gids) = TALLOC_ARRAY(mem_ctx, DOM_SID *, *num_groups);
+	(*user_gids) = TALLOC_ARRAY(mem_ctx, DOM_SID, *num_groups);
 	if (! (*user_gids))
 		smb_panic("lookup_usergroups out of memory");
 	for (i=0; i<(*num_groups); i++) {
-		(*user_gids)[i] = centry_sid(centry, mem_ctx);
+		centry_sid(centry, &(*user_gids)[i]);
 	}
 
 do_cached:	
@@ -1193,7 +1188,7 @@ do_query:
 		goto skip_save;
 	centry_put_uint32(centry, *num_groups);
 	for (i=0; i<(*num_groups); i++) {
-		centry_put_sid(centry, (*user_gids)[i]);
+		centry_put_sid(centry, &(*user_gids)[i]);
 	}	
 	centry_end(centry, "UG/%s", sid_to_string(sid_string, user_sid));
 	centry_free(centry);
@@ -1202,11 +1197,95 @@ skip_save:
 	return status;
 }
 
+static NTSTATUS lookup_useraliases(struct winbindd_domain *domain,
+				   TALLOC_CTX *mem_ctx,
+				   uint32 num_sids, const DOM_SID *sids,
+				   uint32 *num_aliases, uint32 **alias_rids)
+{
+	struct winbind_cache *cache = get_cache(domain);
+	struct cache_entry *centry = NULL;
+	NTSTATUS status;
+	char *sidlist = talloc_strdup(mem_ctx, "");
+	int i;
+
+	if (!cache->tdb)
+		goto do_query;
+
+	if (num_sids == 0) {
+		*num_aliases = 0;
+		*alias_rids = NULL;
+		return NT_STATUS_OK;
+	}
+
+	/* We need to cache indexed by the whole list of SIDs, the aliases
+	 * resulting might come from any of the SIDs. */
+
+	for (i=0; i<num_sids; i++) {
+		sidlist = talloc_asprintf(mem_ctx, "%s/%s", sidlist,
+					  sid_string_static(&sids[i]));
+		if (sidlist == NULL)
+			return NT_STATUS_NO_MEMORY;
+	}
+
+	centry = wcache_fetch(cache, domain, "UA%s", sidlist);
+
+	if (!centry)
+		goto do_query;
+
+	*num_aliases = centry_uint32(centry);
+	*alias_rids = NULL;
+
+	(*alias_rids) = TALLOC_ARRAY(mem_ctx, uint32, *num_aliases);
+
+	if ((*num_aliases != 0) && ((*alias_rids) == NULL))
+		return NT_STATUS_NO_MEMORY;
+
+	for (i=0; i<(*num_aliases); i++)
+		(*alias_rids)[i] = centry_uint32(centry);
+
+	status = centry->status;
+
+	DEBUG(10,("lookup_useraliases: [Cached] - cached info for domain %s "
+		  "status %s\n", domain->name,
+		  get_friendly_nt_error_msg(status)));
+
+	centry_free(centry);
+	return status;
+
+ do_query:
+	(*num_aliases) = 0;
+	(*alias_rids) = NULL;
+
+	if (!NT_STATUS_IS_OK(domain->last_status))
+		return domain->last_status;
+
+	DEBUG(10,("lookup_usergroups: [Cached] - doing backend query for info "
+		  "for domain %s\n", domain->name ));
+
+	status = domain->backend->lookup_useraliases(domain, mem_ctx,
+						     num_sids, sids,
+						     num_aliases, alias_rids);
+
+	/* and save it */
+	refresh_sequence_number(domain, False);
+	centry = centry_start(domain, status);
+	if (!centry)
+		goto skip_save;
+	centry_put_uint32(centry, *num_aliases);
+	for (i=0; i<(*num_aliases); i++)
+		centry_put_uint32(centry, (*alias_rids)[i]);
+	centry_end(centry, "UA%s", sidlist);
+	centry_free(centry);
+
+ skip_save:
+	return status;
+}
+
 
 static NTSTATUS lookup_groupmem(struct winbindd_domain *domain,
 				TALLOC_CTX *mem_ctx,
 				const DOM_SID *group_sid, uint32 *num_names, 
-				DOM_SID ***sid_mem, char ***names, 
+				DOM_SID **sid_mem, char ***names, 
 				uint32 **name_types)
 {
 	struct winbind_cache *cache = get_cache(domain);
@@ -1227,7 +1306,7 @@ static NTSTATUS lookup_groupmem(struct winbindd_domain *domain,
 	if (*num_names == 0)
 		goto do_cached;
 
-	(*sid_mem) = TALLOC_ARRAY(mem_ctx, DOM_SID *, *num_names);
+	(*sid_mem) = TALLOC_ARRAY(mem_ctx, DOM_SID, *num_names);
 	(*names) = TALLOC_ARRAY(mem_ctx, char *, *num_names);
 	(*name_types) = TALLOC_ARRAY(mem_ctx, uint32, *num_names);
 
@@ -1236,7 +1315,7 @@ static NTSTATUS lookup_groupmem(struct winbindd_domain *domain,
 	}
 
 	for (i=0; i<(*num_names); i++) {
-		(*sid_mem)[i] = centry_sid(centry, mem_ctx);
+		centry_sid(centry, &(*sid_mem)[i]);
 		(*names)[i] = centry_string(centry, mem_ctx);
 		(*name_types)[i] = centry_uint32(centry);
 	}
@@ -1274,7 +1353,7 @@ do_query:
 		goto skip_save;
 	centry_put_uint32(centry, *num_names);
 	for (i=0; i<(*num_names); i++) {
-		centry_put_sid(centry, (*sid_mem)[i]);
+		centry_put_sid(centry, &(*sid_mem)[i]);
 		centry_put_string(centry, (*names)[i]);
 		centry_put_uint32(centry, (*name_types)[i]);
 	}	
@@ -1313,18 +1392,6 @@ static NTSTATUS trusted_domains(struct winbindd_domain *domain,
 					       names, alt_names, dom_sids);
 }
 
-/* find the domain sid */
-static NTSTATUS domain_sid(struct winbindd_domain *domain, DOM_SID *sid)
-{
-	get_cache(domain);
-
-	DEBUG(10,("domain_sid: [Cached] - doing backend query for info for domain %s\n",
-		domain->name ));
-
-	/* we don't cache this call */
-	return domain->backend->domain_sid(domain, sid);
-}
-
 /* find the alternate names for the domain, if any */
 static NTSTATUS alternate_name(struct winbindd_domain *domain)
 {
@@ -1387,9 +1454,172 @@ struct winbindd_methods cache_methods = {
 	sid_to_name,
 	query_user,
 	lookup_usergroups,
+	lookup_useraliases,
 	lookup_groupmem,
 	sequence_number,
 	trusted_domains,
-	domain_sid,
 	alternate_name
 };
+
+static BOOL init_wcache(void)
+{
+	if (wcache == NULL) {
+		wcache = SMB_XMALLOC_P(struct winbind_cache);
+		ZERO_STRUCTP(wcache);
+	}
+
+	if (wcache->tdb != NULL)
+		return True;
+
+	wcache->tdb = tdb_open_log(lock_path("winbindd_cache.tdb"), 5000, 
+				   TDB_CLEAR_IF_FIRST, O_RDWR|O_CREAT, 0600);
+
+	if (wcache->tdb == NULL) {
+		DEBUG(0,("Failed to open winbindd_cache.tdb!\n"));
+		return False;
+	}
+
+	return True;
+}
+
+void cache_store_response(pid_t pid, struct winbindd_response *response)
+{
+	fstring key_str;
+
+	if (!init_wcache())
+		return;
+
+	DEBUG(10, ("Storing response for pid %d, len %d\n",
+		   pid, response->length));
+
+	fstr_sprintf(key_str, "DR/%d", pid);
+	if (tdb_store(wcache->tdb, string_tdb_data(key_str), 
+		      make_tdb_data((void *)response, sizeof(*response)),
+		      TDB_REPLACE) == -1)
+		return;
+
+	if (response->length == sizeof(*response))
+		return;
+
+	/* There's extra data */
+
+	DEBUG(10, ("Storing extra data: len=%d\n",
+		   response->length - sizeof(*response)));
+
+	fstr_sprintf(key_str, "DE/%d", pid);
+	if (tdb_store(wcache->tdb, string_tdb_data(key_str),
+		      make_tdb_data(response->extra_data,
+				    response->length - sizeof(*response)),
+		      TDB_REPLACE) == 0)
+		return;
+
+	/* We could not store the extra data, make sure the tdb does not
+	 * contain a main record with wrong dangling extra data */
+
+	fstr_sprintf(key_str, "DR/%d", pid);
+	tdb_delete(wcache->tdb, string_tdb_data(key_str));
+
+	return;
+}
+
+BOOL cache_retrieve_response(pid_t pid, struct winbindd_response * response)
+{
+	TDB_DATA data;
+	fstring key_str;
+
+	if (!init_wcache())
+		return False;
+
+	DEBUG(10, ("Retrieving response for pid %d\n", pid));
+
+	fstr_sprintf(key_str, "DR/%d", pid);
+	data = tdb_fetch(wcache->tdb, string_tdb_data(key_str));
+
+	if (data.dptr == NULL)
+		return False;
+
+	if (data.dsize != sizeof(*response))
+		return False;
+
+	memcpy(response, data.dptr, data.dsize);
+	SAFE_FREE(data.dptr);
+
+	if (response->length == sizeof(*response)) {
+		response->extra_data = NULL;
+		return True;
+	}
+
+	/* There's extra data */
+
+	DEBUG(10, ("Retrieving extra data length=%d\n",
+		   response->length - sizeof(*response)));
+
+	fstr_sprintf(key_str, "DE/%d", pid);
+	data = tdb_fetch(wcache->tdb, string_tdb_data(key_str));
+
+	if (data.dptr == NULL) {
+		DEBUG(0, ("Did not find extra data\n"));
+		return False;
+	}
+
+	if (data.dsize != (response->length - sizeof(*response))) {
+		DEBUG(0, ("Invalid extra data length: %d\n", data.dsize));
+		SAFE_FREE(data.dptr);
+		return False;
+	}
+
+	response->extra_data = data.dptr;
+	return True;
+}
+
+char *cache_store_request_data(TALLOC_CTX *mem_ctx, char *request_string)
+{
+	int i;
+
+	if (!init_wcache())
+		return NULL;
+
+	for (i=0; i<2; i++) {
+		char *key = talloc_strdup(mem_ctx, generate_random_str(16));
+		if (key == NULL)
+			return NULL;
+		DEBUG(10, ("Storing request key %s\n", key));
+		if (tdb_store_bystring(wcache->tdb, key,
+			      	       string_tdb_data(request_string),
+			      	       TDB_INSERT) == 0)
+			return key;
+	}
+	return NULL;
+}
+
+char *cache_retrieve_request_data(TALLOC_CTX *mem_ctx, char *key)
+{
+	TDB_DATA data;
+	char *result = NULL;
+
+	if (!init_wcache())
+		return NULL;
+
+	DEBUG(10, ("Retrieving key %s\n", key));
+
+	data = tdb_fetch_bystring(wcache->tdb, key);
+	if (data.dptr == NULL)
+		return NULL;
+
+	if (strnlen(data.dptr, data.dsize) != (data.dsize)) {
+		DEBUG(0, ("Received invalid request string\n"));
+		goto done;
+	}
+	result = TALLOC_ARRAY(mem_ctx, char, data.dsize+1);
+	if (result != NULL) {
+		memcpy(result, data.dptr, data.dsize);
+		result[data.dsize] = '\0';
+	}
+	if (tdb_delete_bystring(wcache->tdb, key) != 0) {
+		DEBUG(0, ("Could not delete key %s\n", key));
+		result = NULL;
+	}
+  done:
+	SAFE_FREE(data.dptr);
+	return result;
+}
