@@ -76,6 +76,13 @@ BOOL lookup_sid(const DOM_SID *sid, fstring dom_name, fstring name, enum SID_NAM
 	/* Check if this is our own sid.  This should perhaps be done by
 	   winbind?  For the moment handle it here. */
 
+	if (sid->num_auths == 4 && sid_equal(get_global_sam_sid(), sid)) {
+		DOM_SID tmp_sid;
+		sid_copy(&tmp_sid, sid);
+		return map_domain_sid_to_name(&tmp_sid, dom_name) && 
+			local_lookup_sid(sid, name, name_type);
+	}
+
 	if (sid->num_auths == 5) {
 		DOM_SID tmp_sid;
 		uint32 rid;
@@ -105,7 +112,6 @@ BOOL lookup_sid(const DOM_SID *sid, fstring dom_name, fstring name, enum SID_NAM
 	return True;
 }
 
-
 /*****************************************************************
  Id mapping cache.  This is to avoid Winbind mappings already
  seen by smbd to be queried too frequently, keeping winbindd
@@ -243,8 +249,8 @@ static BOOL fetch_gid_from_cache(gid_t *pgid, const DOM_SID *psid)
 		if (sid_compare(&pc->sid, psid) == 0) {
 			fstring sid;
 			*pgid = pc->gid;
-			DEBUG(3,("fetch uid from cache %u -> %s\n",
-				(unsigned int)*pgid, sid_to_string(sid, psid)));
+			DEBUG(3,("fetch gid from cache %u -> %s\n",
+				 (unsigned int)*pgid, sid_to_string(sid, psid)));
 			DLIST_PROMOTE(gid_sid_cache_head, pc);
 			return True;
 		}
@@ -472,7 +478,7 @@ NTSTATUS sid_to_gid(const DOM_SID *psid, gid_t *pgid)
 	   or we are dead in the water */
 
 	if ( !winbind_sid_to_gid(pgid, psid) ) {
-		DEBUG(10,("sid_to_uid: winbind failed to allocate a new gid for sid %s\n",
+		DEBUG(10,("sid_to_gid: winbind failed to allocate a new gid for sid %s\n",
 			sid_to_string(sid_str, psid) ));
 		return NT_STATUS_UNSUCCESSFUL;
 	}
