@@ -52,7 +52,7 @@ const char *get_default_sam_name(void)
 
 void pdb_fill_default_sam(SAM_ACCOUNT *user)
 {
-	ZERO_STRUCT(user->private); /* Don't touch the talloc context */
+	ZERO_STRUCT(user->private_u); /* Don't touch the talloc context */
 
 	/* no initial methods */
 	user->methods = NULL;
@@ -60,36 +60,36 @@ void pdb_fill_default_sam(SAM_ACCOUNT *user)
         /* Don't change these timestamp settings without a good reason.
            They are important for NT member server compatibility. */
 
-	user->private.logon_time            = (time_t)0;
-	user->private.pass_last_set_time    = (time_t)0;
-	user->private.pass_can_change_time  = (time_t)0;
-	user->private.logoff_time           = 
-	user->private.kickoff_time          = 
-	user->private.pass_must_change_time = get_time_t_max();
-	user->private.fields_present        = 0x00ffffff;
-	user->private.logon_divs = 168; 	/* hours per week */
-	user->private.hours_len = 21; 		/* 21 times 8 bits = 168 */
-	memset(user->private.hours, 0xff, user->private.hours_len); /* available at all hours */
-	user->private.bad_password_count = 0;
-	user->private.logon_count = 0;
-	user->private.unknown_6 = 0x000004ec; /* don't know */
+	user->private_u.logon_time            = (time_t)0;
+	user->private_u.pass_last_set_time    = (time_t)0;
+	user->private_u.pass_can_change_time  = (time_t)0;
+	user->private_u.logoff_time           = 
+	user->private_u.kickoff_time          = 
+	user->private_u.pass_must_change_time = get_time_t_max();
+	user->private_u.fields_present        = 0x00ffffff;
+	user->private_u.logon_divs = 168; 	/* hours per week */
+	user->private_u.hours_len = 21; 		/* 21 times 8 bits = 168 */
+	memset(user->private_u.hours, 0xff, user->private_u.hours_len); /* available at all hours */
+	user->private_u.bad_password_count = 0;
+	user->private_u.logon_count = 0;
+	user->private_u.unknown_6 = 0x000004ec; /* don't know */
 
 	/* Some parts of samba strlen their pdb_get...() returns, 
 	   so this keeps the interface unchanged for now. */
 	   
-	user->private.username = "";
-	user->private.domain = "";
-	user->private.nt_username = "";
-	user->private.full_name = "";
-	user->private.home_dir = "";
-	user->private.logon_script = "";
-	user->private.profile_path = "";
-	user->private.acct_desc = "";
-	user->private.workstations = "";
-	user->private.unknown_str = "";
-	user->private.munged_dial = "";
+	user->private_u.username = "";
+	user->private_u.domain = "";
+	user->private_u.nt_username = "";
+	user->private_u.full_name = "";
+	user->private_u.home_dir = "";
+	user->private_u.logon_script = "";
+	user->private_u.profile_path = "";
+	user->private_u.acct_desc = "";
+	user->private_u.workstations = "";
+	user->private_u.unknown_str = "";
+	user->private_u.munged_dial = "";
 
-	user->private.plaintext_pw = NULL;
+	user->private_u.plaintext_pw = NULL;
 
 	/* 
 	   Unless we know otherwise have a Account Control Bit
@@ -97,17 +97,17 @@ void pdb_fill_default_sam(SAM_ACCOUNT *user)
 	   asks for a filtered list of users.
 	*/
 
-	user->private.acct_ctrl = ACB_NORMAL;
+	user->private_u.acct_ctrl = ACB_NORMAL;
 }	
 
 static void destroy_pdb_talloc(SAM_ACCOUNT **user) 
 {
 	if (*user) {
-		data_blob_clear_free(&((*user)->private.lm_pw));
-		data_blob_clear_free(&((*user)->private.nt_pw));
+		data_blob_clear_free(&((*user)->private_u.lm_pw));
+		data_blob_clear_free(&((*user)->private_u.nt_pw));
 
-		if((*user)->private.plaintext_pw!=NULL)
-			memset((*user)->private.plaintext_pw,'\0',strlen((*user)->private.plaintext_pw));
+		if((*user)->private_u.plaintext_pw!=NULL)
+			memset((*user)->private_u.plaintext_pw,'\0',strlen((*user)->private_u.plaintext_pw));
 		talloc_destroy((*user)->mem_ctx);
 		*user = NULL;
 	}
@@ -396,13 +396,13 @@ static void pdb_free_sam_contents(SAM_ACCOUNT *user)
 	/* Kill off sensitive data.  Free()ed by the
 	   talloc mechinism */
 
-	data_blob_clear_free(&(user->private.lm_pw));
-	data_blob_clear_free(&(user->private.nt_pw));
-	if (user->private.plaintext_pw!=NULL)
-		memset(user->private.plaintext_pw,'\0',strlen(user->private.plaintext_pw));
+	data_blob_clear_free(&(user->private_u.lm_pw));
+	data_blob_clear_free(&(user->private_u.nt_pw));
+	if (user->private_u.plaintext_pw!=NULL)
+		memset(user->private_u.plaintext_pw,'\0',strlen(user->private_u.plaintext_pw));
 
-	if (user->private.backend_private_data && user->private.backend_private_data_free_fn) {
-		user->private.backend_private_data_free_fn(&user->private.backend_private_data);
+	if (user->private_u.backend_private_data && user->private_u.backend_private_data_free_fn) {
+		user->private_u.backend_private_data_free_fn(&user->private_u.backend_private_data);
 	}
 }
 
@@ -742,6 +742,14 @@ BOOL local_lookup_sid(const DOM_SID *sid, char *name, enum SID_NAME_USE *psid_na
 	GROUP_MAP map;
 	BOOL ret;
 
+	if (sid_equal(get_global_sam_sid(), sid)) {
+		*psid_name_use = SID_NAME_DOMAIN;
+		fstrcpy(name, "");
+		DEBUG(5,("local_lookup_sid: SID is our own domain-sid: %s.\n", 
+			sid_string_static(sid)));
+		return True;
+	}
+
 	if (!sid_peek_check_rid(get_global_sam_sid(), sid, &rid)){
 		DEBUG(0,("local_lookup_sid: sid_peek_check_rid return False! SID: %s\n",
 			sid_string_static(&map.sid)));
@@ -751,24 +759,11 @@ BOOL local_lookup_sid(const DOM_SID *sid, char *name, enum SID_NAME_USE *psid_na
 	
 	DEBUG(5,("local_lookup_sid: looking up RID %u.\n", (unsigned int)rid));
 	
-	if (rid == DOMAIN_USER_RID_ADMIN) {
-		const char **admin_list = lp_admin_users(-1);
-		*psid_name_use = SID_NAME_USER;
-		if (admin_list) {
-			const char *p = *admin_list;
-			if(!next_token(&p, name, NULL, sizeof(fstring)))
-				fstrcpy(name, "Administrator");
-		} else {
-			fstrcpy(name, "Administrator");
-		}
-		return True;
-	}
 
+	/* see if the passdb can help us with the name of the user */
 	if (!NT_STATUS_IS_OK(pdb_init_sam(&sam_account))) {
 		return False;
 	}
-	
-	/* see if the passdb can help us with the name of the user */
 
 	/* BEING ROOT BLLOCK */
 	become_root();
@@ -799,6 +794,12 @@ BOOL local_lookup_sid(const DOM_SID *sid, char *name, enum SID_NAME_USE *psid_na
 		return True;
 	}
 
+	if (rid == DOMAIN_USER_RID_ADMIN) {
+		*psid_name_use = SID_NAME_USER;
+		fstrcpy(name, "Administrator");
+		return True;
+	}
+
 	if (algorithmic_pdb_rid_is_user(rid)) {
 		uid_t uid;
 		struct passwd *pw = NULL;
@@ -831,8 +832,6 @@ BOOL local_lookup_sid(const DOM_SID *sid, char *name, enum SID_NAME_USE *psid_na
 		gid = pdb_group_rid_to_gid(rid);
 		gr = getgrgid(gid);
 			
-		*psid_name_use = SID_NAME_ALIAS;
-			
 		DEBUG(5,("local_lookup_sid: looking up gid %u %s\n", (unsigned int)gid,
 			 gr ? "succeeded" : "failed" ));
 			
@@ -1151,9 +1150,11 @@ DOM_SID *local_uid_to_sid(DOM_SID *psid, uid_t uid)
 		DEBUG(4,("local_uid_to_sid: User %s [uid == %lu] has no samba account\n",
 			unix_pw->pw_name, (unsigned long)uid));
 
-		return algorithmic_uid_to_sid( psid, uid);
+		algorithmic_uid_to_sid( psid, uid);
 	}
 
+	pdb_free_sam(&sampw);
+
 	DEBUG(10,("local_uid_to_sid:  uid (%d) -> SID %s (%s).\n", 
 		(unsigned int)uid, sid_string_static(psid), unix_pw->pw_name));
 	
@@ -1191,6 +1192,7 @@ BOOL local_sid_to_uid(uid_t *puid, const DOM_SID *psid, enum SID_NAME_USE *name_
 	become_root();
 	if ( !pdb_getsampwsid(sampw, psid) ) {
 		unbecome_root();
+		pdb_free_sam(&sampw);
 		DEBUG(8,("local_sid_to_uid: Could not find SID %s in passdb\n",
 			sid_string_static(psid)));
 		return False;
@@ -1214,28 +1216,10 @@ BOOL local_sid_to_uid(uid_t *puid, const DOM_SID *psid, enum SID_NAME_USE *name_
 		(unsigned int)*puid, user_name ));
 
 	*name_type = SID_NAME_USER;
-	
+	pdb_free_sam( &sampw );
 	return True;
 }
 
-/****************************************************************************
- Convert a gid to SID - algorithmic.
-****************************************************************************/
-
-DOM_SID *algorithmic_gid_to_sid(DOM_SID *psid, uid_t gid)
-{
-	if ( !lp_enable_rid_algorithm() )
-		return NULL;
-
-	DEBUG(8,("algorithmic_gid_to_sid: falling back to RID algorithm\n"));
-	sid_copy( psid, get_global_sam_sid() );
-	sid_append_rid( psid, pdb_gid_to_group_rid(gid) );
-	DEBUG(10,("algorithmic_gid_to_sid:  gid (%d) -> SID %s.\n",
-		(unsigned int)gid, sid_string_static(psid) ));
-
-	return psid;
-}
-
 /****************************************************************************
  Convert a gid to SID - locally.
 ****************************************************************************/
@@ -1256,14 +1240,16 @@ DOM_SID *local_gid_to_sid(DOM_SID *psid, gid_t gid)
 	
 	if ( !ret ) {
 
-		/* algorithmic to rid mapping if enabled */
+		/* fallback to rid mapping if enabled */
 
 		if ( lp_enable_rid_algorithm() ) {
+			sid_copy(psid, get_global_sam_sid());
+			sid_append_rid(psid, pdb_gid_to_group_rid(gid));
 
 			DEBUG(10,("local_gid_to_sid: Fall back to algorithmic mapping: %u -> %s\n", 
 				(unsigned int)gid, sid_string_static(psid)));
 				
-			return algorithmic_gid_to_sid(psid, gid);
+			return psid;
 		}
 		else
 			return NULL;
@@ -1378,18 +1364,18 @@ BOOL init_sam_from_buffer_v0(SAM_ACCOUNT *sampass, uint8 *buf, uint32 buflen)
 		pass_last_set_time,
 		pass_can_change_time,
 		pass_must_change_time;
-	char *username;
-	char *domain;
-	char *nt_username;
-	char *dir_drive;
-	char *unknown_str;
-	char *munged_dial;
-	char *fullname;
-	char *homedir;
-	char *logon_script;
-	char *profile_path;
-	char *acct_desc;
-	char *workstations;
+	char *username = NULL;
+	char *domain = NULL;
+	char *nt_username = NULL;
+	char *dir_drive = NULL;
+	char *unknown_str = NULL;
+	char *munged_dial = NULL;
+	char *fullname = NULL;
+	char *homedir = NULL;
+	char *logon_script = NULL;
+	char *profile_path = NULL;
+	char *acct_desc = NULL;
+	char *workstations = NULL;
 	uint32	username_len, domain_len, nt_username_len,
 		dir_drive_len, unknown_str_len, munged_dial_len,
 		fullname_len, homedir_len, logon_script_len,
@@ -1398,8 +1384,8 @@ BOOL init_sam_from_buffer_v0(SAM_ACCOUNT *sampass, uint8 *buf, uint32 buflen)
 	uint32	user_rid, group_rid, remove_me, hours_len, unknown_6;
 	uint16	acct_ctrl, logon_divs;
 	uint16	bad_password_count, logon_count;
-	uint8	*hours;
-	uint8	*lm_pw_ptr, *nt_pw_ptr;
+	uint8	*hours = NULL;
+	uint8	*lm_pw_ptr = NULL, *nt_pw_ptr = NULL;
 	uint32		len = 0;
 	uint32		lm_pw_len, nt_pw_len, hourslen;
 	BOOL ret = True;
@@ -1557,18 +1543,18 @@ BOOL init_sam_from_buffer_v1(SAM_ACCOUNT *sampass, uint8 *buf, uint32 buflen)
 		pass_last_set_time,
 		pass_can_change_time,
 		pass_must_change_time;
-	char *username;
-	char *domain;
-	char *nt_username;
-	char *dir_drive;
-	char *unknown_str;
-	char *munged_dial;
-	char *fullname;
-	char *homedir;
-	char *logon_script;
-	char *profile_path;
-	char *acct_desc;
-	char *workstations;
+	char *username = NULL;
+	char *domain = NULL;
+	char *nt_username = NULL;
+	char *dir_drive = NULL;
+	char *unknown_str = NULL;
+	char *munged_dial = NULL;
+	char *fullname = NULL;
+	char *homedir = NULL;
+	char *logon_script = NULL;
+	char *profile_path = NULL;
+	char *acct_desc = NULL;
+	char *workstations = NULL;
 	uint32	username_len, domain_len, nt_username_len,
 		dir_drive_len, unknown_str_len, munged_dial_len,
 		fullname_len, homedir_len, logon_script_len,
@@ -1577,8 +1563,8 @@ BOOL init_sam_from_buffer_v1(SAM_ACCOUNT *sampass, uint8 *buf, uint32 buflen)
 	uint32	user_rid, group_rid, remove_me, hours_len, unknown_6;
 	uint16	acct_ctrl, logon_divs;
 	uint16	bad_password_count, logon_count;
-	uint8	*hours;
-	uint8	*lm_pw_ptr, *nt_pw_ptr;
+	uint8	*hours = NULL;
+	uint8	*lm_pw_ptr = NULL, *nt_pw_ptr = NULL;
 	uint32		len = 0;
 	uint32		lm_pw_len, nt_pw_len, hourslen;
 	BOOL ret = True;
@@ -1743,18 +1729,18 @@ BOOL init_sam_from_buffer_v2(SAM_ACCOUNT *sampass, uint8 *buf, uint32 buflen)
 		pass_last_set_time,
 		pass_can_change_time,
 		pass_must_change_time;
-	char *username;
-	char *domain;
-	char *nt_username;
-	char *dir_drive;
-	char *unknown_str;
-	char *munged_dial;
-	char *fullname;
-	char *homedir;
-	char *logon_script;
-	char *profile_path;
-	char *acct_desc;
-	char *workstations;
+	char *username = NULL;
+	char *domain = NULL;
+	char *nt_username = NULL;
+	char *dir_drive = NULL;
+	char *unknown_str = NULL;
+	char *munged_dial = NULL;
+	char *fullname = NULL;
+	char *homedir = NULL;
+	char *logon_script = NULL;
+	char *profile_path = NULL;
+	char *acct_desc = NULL;
+	char *workstations = NULL;
 	uint32	username_len, domain_len, nt_username_len,
 		dir_drive_len, unknown_str_len, munged_dial_len,
 		fullname_len, homedir_len, logon_script_len,
@@ -1763,12 +1749,13 @@ BOOL init_sam_from_buffer_v2(SAM_ACCOUNT *sampass, uint8 *buf, uint32 buflen)
 	uint32	user_rid, group_rid, hours_len, unknown_6;
 	uint16	acct_ctrl, logon_divs;
 	uint16	bad_password_count, logon_count;
-	uint8	*hours;
-	uint8	*lm_pw_ptr, *nt_pw_ptr, *nt_pw_hist_ptr;
+	uint8	*hours = NULL;
+	uint8	*lm_pw_ptr = NULL, *nt_pw_ptr = NULL, *nt_pw_hist_ptr = NULL;
 	uint32		len = 0;
 	uint32		lm_pw_len, nt_pw_len, nt_pw_hist_len, hourslen;
 	uint32 pwHistLen = 0;
 	BOOL ret = True;
+	fstring tmpstring;
 	
 	if(sampass == NULL || buf == NULL) {
 		DEBUG(0, ("init_sam_from_buffer_v2: NULL parameters found!\n"));
@@ -1832,7 +1819,9 @@ BOOL init_sam_from_buffer_v2(SAM_ACCOUNT *sampass, uint8 *buf, uint32 buflen)
 	pdb_set_fullname(sampass, fullname, PDB_SET);
 
 	if (homedir) {
-		pdb_set_homedir(sampass, homedir, PDB_SET);
+		fstrcpy( tmpstring, homedir );
+		standard_sub_basic( username, tmpstring, sizeof(tmpstring) );
+		pdb_set_homedir(sampass, tmpstring, PDB_SET);
 	}
 	else {
 		pdb_set_homedir(sampass, 
@@ -1842,14 +1831,14 @@ BOOL init_sam_from_buffer_v2(SAM_ACCOUNT *sampass, uint8 *buf, uint32 buflen)
 
 	if (dir_drive) 	
 		pdb_set_dir_drive(sampass, dir_drive, PDB_SET);
-	else {
-		pdb_set_dir_drive(sampass, 
-			talloc_sub_basic(sampass->mem_ctx,  username, lp_logon_drive()),
-			PDB_DEFAULT);
-	}
+	else
+		pdb_set_dir_drive(sampass, lp_logon_drive(), PDB_DEFAULT );
 
-	if (logon_script) 
-		pdb_set_logon_script(sampass, logon_script, PDB_SET);
+	if (logon_script) {
+		fstrcpy( tmpstring, logon_script );
+		standard_sub_basic( username, tmpstring, sizeof(tmpstring) );
+		pdb_set_logon_script(sampass, tmpstring, PDB_SET);
+	}
 	else {
 		pdb_set_logon_script(sampass, 
 			talloc_sub_basic(sampass->mem_ctx, username, lp_logon_script()),
@@ -1857,8 +1846,11 @@ BOOL init_sam_from_buffer_v2(SAM_ACCOUNT *sampass, uint8 *buf, uint32 buflen)
 	}
 	
 	if (profile_path) {	
-		pdb_set_profile_path(sampass, profile_path, PDB_SET);
-	} else {
+		fstrcpy( tmpstring, profile_path );
+		standard_sub_basic( username, tmpstring, sizeof(tmpstring) );
+		pdb_set_profile_path(sampass, tmpstring, PDB_SET);
+	} 
+	else {
 		pdb_set_profile_path(sampass, 
 			talloc_sub_basic(sampass->mem_ctx, username, lp_logon_path()),
 			PDB_DEFAULT);
@@ -1885,7 +1877,7 @@ BOOL init_sam_from_buffer_v2(SAM_ACCOUNT *sampass, uint8 *buf, uint32 buflen)
 	/* Change from V1 is addition of password history field. */
 	account_policy_get(AP_PASSWORD_HISTORY, &pwHistLen);
 	if (pwHistLen) {
-		char *pw_hist = SMB_MALLOC(pwHistLen * PW_HISTORY_ENTRY_LEN);
+		uint8 *pw_hist = SMB_MALLOC(pwHistLen * PW_HISTORY_ENTRY_LEN);
 		if (!pw_hist) {
 			ret = False;
 			goto done;
@@ -2215,6 +2207,28 @@ uint32 init_buffer_from_sam_v2 (uint8 **buf, const SAM_ACCOUNT *sampass, BOOL si
 	return (buflen);
 }
 
+BOOL pdb_copy_sam_account(const SAM_ACCOUNT *src, SAM_ACCOUNT **dst)
+{
+	BOOL result;
+	uint8 *buf;
+	int len;
+
+	if ((*dst == NULL) && (!NT_STATUS_IS_OK(pdb_init_sam(dst))))
+		return False;
+
+	len = init_buffer_from_sam_v2(&buf, src, False);
+
+	if (len == -1)
+		return False;
+
+	result = init_sam_from_buffer_v2(*dst, buf, len);
+	(*dst)->methods = src->methods;
+
+	free(buf);
+
+	return result;
+}
+
 /**********************************************************************
 **********************************************************************/
 
