@@ -43,11 +43,6 @@ static BOOL remove_from_jobs_changed(const char* sharename, uint32 jobid);
    jobids are assigned when a job starts spooling. 
 */
 
-/***************************************************************************
- Nightmare. LANMAN jobid's are 16 bit numbers..... We must map them to 32
- bit RPC jobids.... JRA.
-***************************************************************************/
-
 static TDB_CONTEXT *rap_tdb;
 static uint16 next_rap_jobid;
 struct rap_jobid_key {
@@ -55,11 +50,17 @@ struct rap_jobid_key {
 	uint32  jobid;
 };
 
+/***************************************************************************
+ Nightmare. LANMAN jobid's are 16 bit numbers..... We must map them to 32
+ bit RPC jobids.... JRA.
+***************************************************************************/
+
 uint16 pjobid_to_rap(const char* sharename, uint32 jobid)
 {
 	uint16 rap_jobid;
 	TDB_DATA data, key;
 	struct rap_jobid_key jinfo;
+	uint8 buf[2];
 
 	DEBUG(10,("pjobid_to_rap: called.\n"));
 
@@ -89,7 +90,8 @@ uint16 pjobid_to_rap(const char* sharename, uint32 jobid)
 	rap_jobid = ++next_rap_jobid;
 	if (rap_jobid == 0)
 		rap_jobid = ++next_rap_jobid;
-	data.dptr = (char *)&rap_jobid;
+	SSVAL(buf,0,rap_jobid);
+	data.dptr = buf;
 	data.dsize = sizeof(rap_jobid);
 	tdb_store(rap_tdb, key, data, TDB_REPLACE);
 	tdb_store(rap_tdb, data, key, TDB_REPLACE);
@@ -102,13 +104,15 @@ uint16 pjobid_to_rap(const char* sharename, uint32 jobid)
 BOOL rap_to_pjobid(uint16 rap_jobid, fstring sharename, uint32 *pjobid)
 {
 	TDB_DATA data, key;
+	uint8 buf[2];
 
 	DEBUG(10,("rap_to_pjobid called.\n"));
 
 	if (!rap_tdb)
 		return False;
 
-	key.dptr = (char *)&rap_jobid;
+	SSVAL(buf,0,rap_jobid);
+	key.dptr = buf;
 	key.dsize = sizeof(rap_jobid);
 	data = tdb_fetch(rap_tdb, key);
 	if ( data.dptr && data.dsize == sizeof(struct rap_jobid_key) ) 
@@ -133,6 +137,7 @@ static void rap_jobid_delete(const char* sharename, uint32 jobid)
 	TDB_DATA key, data;
 	uint16 rap_jobid;
 	struct rap_jobid_key jinfo;
+	uint8 buf[2];
 
 	DEBUG(10,("rap_jobid_delete: called.\n"));
 
@@ -158,7 +163,8 @@ static void rap_jobid_delete(const char* sharename, uint32 jobid)
 
 	rap_jobid = SVAL(data.dptr, 0);
 	SAFE_FREE(data.dptr);
-	data.dptr = (char *)&rap_jobid;
+	SSVAL(buf,0,rap_jobid);
+	data.dptr=buf;
 	data.dsize = sizeof(rap_jobid);
 	tdb_delete(rap_tdb, key);
 	tdb_delete(rap_tdb, data);
@@ -228,7 +234,7 @@ void printing_end(void)
  when asked for (and only when supported)
 ****************************************************************************/
 
-static struct printif *get_printer_fns_from_type( enum printing_types type )
+static struct printif *get_printer_fns_from_type( int type )
 {
 	struct printif *printer_fns = &generic_printif;
 
@@ -393,8 +399,10 @@ uint32 sysjob_to_jobid(int unix_jobid)
 		if (!lp_print_ok(snum))
 			continue;
 		pdb = get_print_db_byname(lp_const_servicename(snum));
-		if (pdb)
-			tdb_traverse(pdb->tdb, unixjob_traverse_fn, &unix_jobid);
+		if (!pdb) {
+			continue;
+		}
+		tdb_traverse(pdb->tdb, unixjob_traverse_fn, &unix_jobid);
 		release_print_db(pdb);
 		if (sysjob_to_jobid_value != (uint32)-1)
 			return sysjob_to_jobid_value;
@@ -803,7 +811,7 @@ static void print_cache_flush(int snum)
  Check if someone already thinks they are doing the update.
 ****************************************************************************/
 
-static pid_t get_updating_pid(fstring sharename)
+static pid_t get_updating_pid(const char *sharename)
 {
 	fstring keystr;
 	TDB_DATA data, key;
@@ -837,7 +845,7 @@ static pid_t get_updating_pid(fstring sharename)
  in the tdb.
 ****************************************************************************/
 
-static void set_updating_pid(const fstring sharename, BOOL delete)
+static void set_updating_pid(const fstring sharename, BOOL updating)
 {
 	fstring keystr;
 	TDB_DATA key;
@@ -853,8 +861,12 @@ static void set_updating_pid(const fstring sharename, BOOL delete)
 	slprintf(keystr, sizeof(keystr)-1, "UPDATING/%s", sharename);
     	key.dptr = keystr;
 	key.dsize = strlen(keystr);
+	
+	DEBUG(5, ("set_updating_pid: %s updating lpq cache for print share %s\n", 
+		updating ? "" : "not ",
+		sharename ));
 
-	if (delete) {
+	if ( !updating ) {
 		tdb_delete(pdb->tdb, key);
 		release_print_db(pdb);
 		return;
@@ -896,7 +908,7 @@ static int printjob_comp(print_queue_struct *j1, print_queue_struct *j2)
 
 static void store_queue_struct(struct tdb_print_db *pdb, struct traverse_struct *pts)
 {
-	TDB_DATA data, key;
+	TDB_DATA data;
 	int max_reported_jobs = lp_max_reported_jobs(pts->snum);
 	print_queue_struct *queue = pts->queue;
 	size_t len;
@@ -940,22 +952,19 @@ static void store_queue_struct(struct tdb_print_db *pdb, struct traverse_struct
 				queue[i].fs_file);
 	}
 
-	key.dptr = "INFO/linear_queue_array";
-	key.dsize = strlen(key.dptr);
-	tdb_store(pdb->tdb, key, data, TDB_REPLACE);
+	tdb_store(pdb->tdb, string_tdb_data("INFO/linear_queue_array"), data,
+		  TDB_REPLACE);
 	SAFE_FREE(data.dptr);
 	return;
 }
 
 static TDB_DATA get_jobs_changed_data(struct tdb_print_db *pdb)
 {
-	TDB_DATA data, key;
+	TDB_DATA data;
 
-	key.dptr = "INFO/jobs_changed";
-	key.dsize = strlen(key.dptr);
 	ZERO_STRUCT(data);
 
-	data = tdb_fetch(pdb->tdb, key);
+	data = tdb_fetch(pdb->tdb, string_tdb_data("INFO/jobs_changed"));
 	if (data.dptr == NULL || data.dsize == 0 || (data.dsize % 4 != 0)) {
 		SAFE_FREE(data.dptr);
 		ZERO_STRUCT(data);
@@ -978,11 +987,69 @@ static void check_job_changed(const char *sharename, TDB_DATA data, uint32 jobid
 	}
 }
 
-struct print_queue_update_context {
-	fstring sharename;
-	enum printing_types printing_type;
-	pstring lpqcommand;
-};
+/****************************************************************************
+ Check if the print queue has been updated recently enough.
+****************************************************************************/
+
+static BOOL print_cache_expired(const char *sharename, BOOL check_pending)
+{
+	fstring key;
+	time_t last_qscan_time, time_now = time(NULL);
+	struct tdb_print_db *pdb = get_print_db_byname(sharename);
+	BOOL result = False;
+
+	if (!pdb)
+		return False;
+
+	snprintf(key, sizeof(key), "CACHE/%s", sharename);
+	last_qscan_time = (time_t)tdb_fetch_int32(pdb->tdb, key);
+
+	/*
+	 * Invalidate the queue for 3 reasons.
+	 * (1). last queue scan time == -1.
+	 * (2). Current time - last queue scan time > allowed cache time.
+	 * (3). last queue scan time > current time + MAX_CACHE_VALID_TIME (1 hour by default).
+	 * This last test picks up machines for which the clock has been moved
+	 * forward, an lpq scan done and then the clock moved back. Otherwise
+	 * that last lpq scan would stay around for a loooong loooong time... :-). JRA.
+	 */
+
+	if (last_qscan_time == ((time_t)-1) 
+		|| (time_now - last_qscan_time) >= lp_lpqcachetime() 
+		|| last_qscan_time > (time_now + MAX_CACHE_VALID_TIME)) 
+	{
+		time_t msg_pending_time;
+
+		DEBUG(4, ("print_cache_expired: cache expired for queue %s " 
+			"(last_qscan_time = %d, time now = %d, qcachetime = %d)\n", 
+			sharename, (int)last_qscan_time, (int)time_now, 
+			(int)lp_lpqcachetime() ));
+
+		/* check if another smbd has already sent a message to update the 
+		   queue.  Give the pending message one minute to clear and 
+		   then send another message anyways.  Make sure to check for 
+		   clocks that have been run forward and then back again. */
+
+		snprintf(key, sizeof(key), "MSG_PENDING/%s", sharename);
+
+		if ( check_pending 
+			&& tdb_fetch_uint32( pdb->tdb, key, (uint32*)&msg_pending_time ) 
+			&& msg_pending_time > 0
+			&& msg_pending_time <= time_now 
+			&& (time_now - msg_pending_time) < 60 ) 
+		{
+			DEBUG(4,("print_cache_expired: message already pending for %s.  Accepting cache\n",
+				sharename));
+			goto done;
+		}
+		
+		result = True;
+	}
+
+done:
+	release_print_db(pdb);
+	return result;
+}
 
 /****************************************************************************
  main work for updating the lpq cahe for a printer queue
@@ -1002,13 +1069,20 @@ static void print_queue_update_internal( const char *sharename,
 	TDB_DATA jcdata;
 	fstring keystr, cachestr;
 	struct tdb_print_db *pdb = get_print_db_byname(sharename);
+
+	if (!pdb) {
+		return;
+	}
 	
+	DEBUG(5,("print_queue_update_internal: printer = %s, type = %d, lpq command = [%s]\n",
+		sharename, current_printif->type, lpq_command));
+
 	/*
 	 * Update the cache time FIRST ! Stops others even
 	 * attempting to get the lock and doing this
 	 * if the lpq takes a long time.
 	 */
-
+	 
 	slprintf(cachestr, sizeof(cachestr)-1, "CACHE/%s", sharename);
 	tdb_store_int32(pdb->tdb, cachestr, (int)time(NULL));
 
@@ -1019,8 +1093,8 @@ static void print_queue_update_internal( const char *sharename,
 		current_printif->type, 
 		lpq_command, &queue, &status);
 
-	DEBUG(3, ("%d job%s in queue for %s\n", qcount, (qcount != 1) ?
-		"s" : "", sharename));
+	DEBUG(3, ("print_queue_update_internal: %d job%s in queue for %s\n", 
+		qcount, (qcount != 1) ?	"s" : "", sharename));
 
 	/* Sort the queue by submission time otherwise they are displayed
 	   in hash order. */
@@ -1084,14 +1158,14 @@ static void print_queue_update_internal( const char *sharename,
 
 	SAFE_FREE(tstruct.queue);
 
-	DEBUG(10,("print_queue_update: printer %s INFO/total_jobs = %d\n",
+	DEBUG(10,("print_queue_update_internal: printer %s INFO/total_jobs = %d\n",
 				sharename, tstruct.total_jobs ));
 
 	tdb_store_int32(pdb->tdb, "INFO/total_jobs", tstruct.total_jobs);
 
 	get_queue_status(sharename, &old_status);
 	if (old_status.qcount != qcount)
-		DEBUG(10,("print_queue_update: queue status change %d jobs -> %d jobs for printer %s\n",
+		DEBUG(10,("print_queue_update_internal: queue status change %d jobs -> %d jobs for printer %s\n",
 					old_status.qcount, qcount, sharename));
 
 	/* store the new queue status structure */
@@ -1112,6 +1186,20 @@ static void print_queue_update_internal( const char *sharename,
 	slprintf(keystr, sizeof(keystr)-1, "CACHE/%s", sharename);
 	tdb_store_int32(pdb->tdb, keystr, (int32)time(NULL));
 
+	/* clear the msg pending record for this queue */
+
+	snprintf(keystr, sizeof(keystr), "MSG_PENDING/%s", sharename);
+
+	if ( !tdb_store_uint32( pdb->tdb, keystr, 0 ) ) {
+		/* log a message but continue on */
+
+		DEBUG(0,("print_queue_update: failed to store MSG_PENDING flag for [%s]!\n",
+			sharename));
+	}
+
+	release_print_db( pdb );
+
+	return;
 }
 
 /****************************************************************************
@@ -1120,18 +1208,24 @@ static void print_queue_update_internal( const char *sharename,
  smbd processes maytry to update the lpq cache concurrently).
 ****************************************************************************/
 
-static void print_queue_update_with_lock(int snum)
+static void print_queue_update_with_lock( const char *sharename, 
+                                          struct printif *current_printif,
+                                          char *lpq_command )
 {
-	fstring sharename, keystr;
-	pstring lpq_command;
+	fstring keystr;
 	struct tdb_print_db *pdb;
-	struct printif *current_printif = get_printer_fns( snum );
 
-	fstrcpy(sharename, lp_const_servicename(snum));
+	DEBUG(5,("print_queue_update_with_lock: printer share = %s\n", sharename));
 	pdb = get_print_db_byname(sharename);
 	if (!pdb)
 		return;
 
+	if ( !print_cache_expired(sharename, False) ) {
+		DEBUG(5,("print_queue_update_with_lock: print cache for %s is still ok\n", sharename));
+		release_print_db(pdb);
+		return;
+	}
+	
 	/*
 	 * Check to see if someone else is doing this update.
 	 * This is essentially a mutex on the update.
@@ -1147,7 +1241,7 @@ static void print_queue_update_with_lock(int snum)
 	slprintf(keystr, sizeof(keystr) - 1, "LOCK/%s", sharename);
 	/* Only wait 10 seconds for this. */
 	if (tdb_lock_bystring(pdb->tdb, keystr, 10) == -1) {
-		DEBUG(0,("print_queue_update: Failed to lock printer %s database\n", sharename));
+		DEBUG(0,("print_queue_update_with_lock: Failed to lock printer %s database\n", sharename));
 		release_print_db(pdb);
 		return;
 	}
@@ -1172,7 +1266,7 @@ static void print_queue_update_with_lock(int snum)
 	 */
 
 	/* Tell others we're doing the update. */
-	set_updating_pid(sharename, False);
+	set_updating_pid(sharename, True);
 
 	/*
 	 * Allow others to enter and notice we're doing
@@ -1182,36 +1276,39 @@ static void print_queue_update_with_lock(int snum)
 	tdb_unlock_bystring(pdb->tdb, keystr);
 
 	/* do the main work now */
-	/* have to substitute any variables here since 
-           print_queue_get_internal() will not */
-	
-	pstrcpy( lpq_command, lp_lpqcommand(snum) );
-	pstring_sub( lpq_command, "%p", PRINTERNAME(snum) );
-	standard_sub_snum( snum, lpq_command, sizeof(lpq_command) );
 	
 	print_queue_update_internal( sharename, current_printif, lpq_command );
 	
 	/* Delete our pid from the db. */
-	set_updating_pid(sharename, True);
+	set_updating_pid(sharename, False);
 	release_print_db(pdb);
 }
 
 /****************************************************************************
 this is the receive function of the background lpq updater
 ****************************************************************************/
-static void print_queue_receive(int msg_type, pid_t src, void *buf, size_t len)
+static void print_queue_receive(int msg_type, pid_t src, void *buf, size_t msglen)
 {
-	struct print_queue_update_context *ctx;
+	fstring sharename;
+	pstring lpqcommand;
+	int printing_type;
+	size_t len;
 
-	if (len != sizeof(struct print_queue_update_context)) {
-		DEBUG(1, ("Got invalid print queue update message\n"));
+	len = tdb_unpack( buf, msglen, "fdP",
+		sharename,
+		&printing_type,
+		lpqcommand );
+
+	if ( len == -1 ) {
+		DEBUG(0,("print_queue_receive: Got invalid print queue update message\n"));
 		return;
 	}
 
-	ctx = (struct print_queue_update_context*)buf;
-	print_queue_update_internal(ctx->sharename, 
-		get_printer_fns_from_type(ctx->printing_type),
-		ctx->lpqcommand );
+	print_queue_update_with_lock(sharename, 
+		get_printer_fns_from_type(printing_type),
+		lpqcommand );
+
+	return;
 }
 
 static pid_t background_lpq_updater_pid = -1;
@@ -1274,30 +1371,93 @@ void start_background_queue(void)
 /****************************************************************************
 update the internal database from the system print queue for a queue
 ****************************************************************************/
-static void print_queue_update(int snum)
-{
-	struct print_queue_update_context ctx;
 
+static void print_queue_update(int snum, BOOL force)
+{
+	fstring key;
+	fstring sharename;
+	pstring lpqcommand;
+	char *buffer = NULL;
+	size_t len = 0;
+	size_t newlen;
+	struct tdb_print_db *pdb;
+	int type;
+	struct printif *current_printif;
+
+	fstrcpy( sharename, lp_const_servicename(snum));
+
+	/* don't strip out characters like '$' from the printername */
+	
+	pstrcpy( lpqcommand, lp_lpqcommand(snum));
+	string_sub2( lpqcommand, "%p", PRINTERNAME(snum), sizeof(lpqcommand), False, False );
+	standard_sub_snum( snum, lpqcommand, sizeof(lpqcommand) );
+	
 	/* 
 	 * Make sure that the background queue process exists.  
 	 * Otherwise just do the update ourselves 
 	 */
-	   
-	if ( background_lpq_updater_pid != -1 ) {
-		fstrcpy(ctx.sharename, lp_const_servicename(snum));
-		ctx.printing_type = lp_printing(snum);
-
-		pstrcpy(ctx.lpqcommand, lp_lpqcommand(snum));
-		pstring_sub( ctx.lpqcommand, "%p", PRINTERNAME(snum) );
-		standard_sub_snum( snum, ctx.lpqcommand, sizeof(ctx.lpqcommand) );
 	
-		become_root();
-		message_send_pid(background_lpq_updater_pid,
-				 MSG_PRINTER_UPDATE, &ctx, sizeof(ctx),
-				 False);
-		unbecome_root();
-	} else
-		print_queue_update_with_lock( snum );
+	if ( force || background_lpq_updater_pid == -1 ) {
+		DEBUG(4,("print_queue_update: updating queue [%s] myself\n", sharename));
+		current_printif = get_printer_fns( snum );
+		print_queue_update_with_lock( sharename, current_printif, lpqcommand );
+
+		return;
+	}
+
+	type = lp_printing(snum);
+	
+	/* get the length */
+
+	len = tdb_pack( buffer, len, "fdP",
+		sharename,
+		type,
+		lpqcommand );
+
+	buffer = SMB_XMALLOC_ARRAY( char, len );
+
+	/* now pack the buffer */
+	newlen = tdb_pack( buffer, len, "fdP",
+		sharename,
+		type,
+		lpqcommand );
+
+	SMB_ASSERT( newlen == len );
+
+	DEBUG(10,("print_queue_update: Sending message -> printer = %s, "
+		"type = %d, lpq command = [%s]\n", sharename, type, lpqcommand ));
+
+	/* here we set a msg pending record for other smbd processes 
+	   to throttle the number of duplicate print_queue_update msgs
+	   sent.  */
+
+	pdb = get_print_db_byname(sharename);
+	if (!pdb) {
+		SAFE_FREE(buffer);
+		return;
+	}
+
+	snprintf(key, sizeof(key), "MSG_PENDING/%s", sharename);
+
+	if ( !tdb_store_uint32( pdb->tdb, key, time(NULL) ) ) {
+		/* log a message but continue on */
+
+		DEBUG(0,("print_queue_update: failed to store MSG_PENDING flag for [%s]!\n",
+			sharename));
+	}
+
+	release_print_db( pdb );
+
+	/* finally send the message */
+	
+	become_root();
+	message_send_pid(background_lpq_updater_pid,
+		 MSG_PRINTER_UPDATE, buffer, len, False);
+	unbecome_root();
+
+	SAFE_FREE( buffer );
+
+	return;
 }
 
 /****************************************************************************
@@ -1582,10 +1742,14 @@ static BOOL remove_from_jobs_changed(const char* sharename, uint32 jobid)
 	BOOL ret = False;
 	BOOL gotlock = False;
 
-	key.dptr = "INFO/jobs_changed";
-	key.dsize = strlen(key.dptr);
+	if (!pdb) {
+		return False;
+	}
+
 	ZERO_STRUCT(data);
 
+	key = string_tdb_data("INFO/jobs_changed");
+
 	if (tdb_chainlock_with_timeout(pdb->tdb, key, 5) == -1)
 		goto out;
 
@@ -1757,7 +1921,7 @@ pause, or resume print job. User name: %s. Printer name: %s.",
 	/* force update the database and say the delete failed if the
            job still exists */
 
-	print_queue_update(snum);
+	print_queue_update(snum, True);
 	
 	deleted = !print_job_exists(sharename, jobid);
 	if ( !deleted )
@@ -1874,7 +2038,7 @@ pause, or resume print job. User name: %s. Printer name: %s.",
  Write to a print file.
 ****************************************************************************/
 
-int print_job_write(int snum, uint32 jobid, const char *buf, int size)
+ssize_t print_job_write(int snum, uint32 jobid, const char *buf, SMB_OFF_T pos, size_t size)
 {
 	const char* sharename = lp_const_servicename(snum);
 	int return_code;
@@ -1888,7 +2052,8 @@ int print_job_write(int snum, uint32 jobid, const char *buf, int size)
 	if (pjob->pid != sys_getpid())
 		return -1;
 
-	return_code = write(pjob->fd, buf, size);
+	return_code = write_data_at_offset(pjob->fd, buf, size, pos);
+
 	if (return_code>0) {
 		pjob->size += size;
 		pjob_store(sharename, jobid, pjob);
@@ -1896,45 +2061,6 @@ int print_job_write(int snum, uint32 jobid, const char *buf, int size)
 	return return_code;
 }
 
-/****************************************************************************
- Check if the print queue has been updated recently enough.
-****************************************************************************/
-
-static BOOL print_cache_expired(int snum)
-{
-	fstring key;
-	time_t last_qscan_time, time_now = time(NULL);
-	const char *printername = lp_const_servicename(snum);
-	struct tdb_print_db *pdb = get_print_db_byname(printername);
-
-	if (!pdb)
-		return False;
-
-	slprintf(key, sizeof(key), "CACHE/%s", printername);
-	last_qscan_time = (time_t)tdb_fetch_int32(pdb->tdb, key);
-
-	/*
-	 * Invalidate the queue for 3 reasons.
-	 * (1). last queue scan time == -1.
-	 * (2). Current time - last queue scan time > allowed cache time.
-	 * (3). last queue scan time > current time + MAX_CACHE_VALID_TIME (1 hour by default).
-	 * This last test picks up machines for which the clock has been moved
-	 * forward, an lpq scan done and then the clock moved back. Otherwise
-	 * that last lpq scan would stay around for a loooong loooong time... :-). JRA.
-	 */
-
-	if (last_qscan_time == ((time_t)-1) || (time_now - last_qscan_time) >= lp_lpqcachetime() ||
-			last_qscan_time > (time_now + MAX_CACHE_VALID_TIME)) {
-		DEBUG(3, ("print cache expired for queue %s \
-(last_qscan_time = %d, time now = %d, qcachetime = %d)\n", printername,
-			(int)last_qscan_time, (int)time_now, (int)lp_lpqcachetime() ));
-		release_print_db(pdb);
-		return True;
-	}
-	release_print_db(pdb);
-	return False;
-}
-
 /****************************************************************************
  Get the queue status - do not update if db is out of date.
 ****************************************************************************/
@@ -1942,19 +2068,20 @@ static BOOL print_cache_expired(int snum)
 static int get_queue_status(const char* sharename, print_status_struct *status)
 {
 	fstring keystr;
-	TDB_DATA data, key;
+	TDB_DATA data;
 	struct tdb_print_db *pdb = get_print_db_byname(sharename);
 	int len;
 
+	if (status) {
+		ZERO_STRUCTP(status);
+	}
+
 	if (!pdb)
 		return 0;
 
 	if (status) {
-		ZERO_STRUCTP(status);
-		slprintf(keystr, sizeof(keystr)-1, "STATUS/%s", sharename);
-		key.dptr = keystr;
-		key.dsize = strlen(keystr);
-		data = tdb_fetch(pdb->tdb, key);
+		fstr_sprintf(keystr, "STATUS/%s", sharename);
+		data = tdb_fetch(pdb->tdb, string_tdb_data(keystr));
 		if (data.dptr) {
 			if (data.dsize == sizeof(print_status_struct))
 				/* this memcpy is ok since the status struct was 
@@ -1977,10 +2104,12 @@ int print_queue_length(int snum, print_status_struct *pstatus)
 	const char* sharename = lp_const_servicename( snum );
 	print_status_struct status;
 	int len;
+
+	ZERO_STRUCT( status );
  
 	/* make sure the database is up to date */
-	if (print_cache_expired(snum))
-		print_queue_update(snum);
+	if (print_cache_expired(lp_const_servicename(snum), True))
+		print_queue_update(snum, False);
  
 	/* also fetch the queue status */
 	memset(&status, 0, sizeof(status));
@@ -2064,18 +2193,17 @@ static BOOL allocate_print_jobid(struct tdb_print_db *pdb, int snum, const char
 
 static BOOL add_to_jobs_changed(struct tdb_print_db *pdb, uint32 jobid)
 {
-	TDB_DATA data, key;
+	TDB_DATA data;
 	uint32 store_jobid;
 
-	key.dptr = "INFO/jobs_changed";
-	key.dsize = strlen(key.dptr);
 	SIVAL(&store_jobid, 0, jobid);
 	data.dptr = (char *)&store_jobid;
 	data.dsize = 4;
 
 	DEBUG(10,("add_to_jobs_changed: Added jobid %u\n", (unsigned int)jobid ));
 
-	return (tdb_append(pdb->tdb, key, data) == 0);
+	return (tdb_append(pdb->tdb, string_tdb_data("INFO/jobs_changed"),
+			   data) == 0);
 }
 
 /***************************************************************************
@@ -2124,7 +2252,7 @@ uint32 print_job_start(struct current_user *user, int snum, char *jobname, NT_DE
 	}
 
 	/* for autoloaded printers, check that the printcap entry still exists */
-	if (lp_autoloaded(snum) && !pcap_printername_ok(lp_const_servicename(snum), NULL)) {
+	if (lp_autoloaded(snum) && !pcap_printername_ok(lp_const_servicename(snum))) {
 		DEBUG(3, ("print_job_start: printer name %s check failed.\n", lp_const_servicename(snum) ));
 		release_print_db(pdb);
 		errno = ENOENT;
@@ -2294,8 +2422,8 @@ BOOL print_job_end(int snum, uint32 jobid, BOOL normal_close)
 	pjob_store(sharename, jobid, pjob);
 	
 	/* make sure the database is up to date */
-	if (print_cache_expired(snum))
-		print_queue_update(snum);
+	if (print_cache_expired(lp_const_servicename(snum), True))
+		print_queue_update(snum, False);
 	
 	return True;
 
@@ -2314,7 +2442,7 @@ fail:
 
 static BOOL get_stored_queue_info(struct tdb_print_db *pdb, int snum, int *pcount, print_queue_struct **ppqueue)
 {
-	TDB_DATA data, key, cgdata;
+	TDB_DATA data, cgdata;
 	print_queue_struct *queue = NULL;
 	uint32 qcount = 0;
 	uint32 extra_count = 0;
@@ -2326,28 +2454,23 @@ static BOOL get_stored_queue_info(struct tdb_print_db *pdb, int snum, int *pcoun
 	const char* sharename = lp_servicename(snum);
 
 	/* make sure the database is up to date */
-	if (print_cache_expired(snum))
-		print_queue_update(snum);
+	if (print_cache_expired(lp_const_servicename(snum), True))
+		print_queue_update(snum, False);
  
 	*pcount = 0;
 	*ppqueue = NULL;
 
 	ZERO_STRUCT(data);
 	ZERO_STRUCT(cgdata);
-	key.dptr = "INFO/linear_queue_array";
-	key.dsize = strlen(key.dptr);
 
 	/* Get the stored queue data. */
-	data = tdb_fetch(pdb->tdb, key);
+	data = tdb_fetch(pdb->tdb, string_tdb_data("INFO/linear_queue_array"));
 	
 	if (data.dptr && data.dsize >= sizeof(qcount))
 		len += tdb_unpack(data.dptr + len, data.dsize - len, "d", &qcount);
 		
 	/* Get the changed jobs list. */
-	key.dptr = "INFO/jobs_changed";
-	key.dsize = strlen(key.dptr);
-
-	cgdata = tdb_fetch(pdb->tdb, key);
+	cgdata = tdb_fetch(pdb->tdb, string_tdb_data("INFO/jobs_changed"));
 	if (cgdata.dptr != NULL && (cgdata.dsize % 4 == 0))
 		extra_count = cgdata.dsize/4;
 
@@ -2447,8 +2570,8 @@ int print_queue_status(int snum,
 
 	/* make sure the database is up to date */
 
-	if (print_cache_expired(snum))
-		print_queue_update(snum);
+	if (print_cache_expired(lp_const_servicename(snum), True))
+		print_queue_update(snum, False);
 
 	/* return if we are done */
 	if ( !ppqueue || !status )
@@ -2507,9 +2630,14 @@ BOOL print_queue_pause(struct current_user *user, int snum, WERROR *errcode)
 		*errcode = WERR_ACCESS_DENIED;
 		return False;
 	}
+	
 
+	become_root();
+		
 	ret = (*(current_printif->queue_pause))(snum);
 
+	unbecome_root();
+		
 	if (ret != 0) {
 		*errcode = WERR_INVALID_PARAM;
 		return False;
@@ -2538,17 +2666,21 @@ BOOL print_queue_resume(struct current_user *user, int snum, WERROR *errcode)
 		*errcode = WERR_ACCESS_DENIED;
 		return False;
 	}
-
+	
+	become_root();
+		
 	ret = (*(current_printif->queue_resume))(snum);
 
+	unbecome_root();
+		
 	if (ret != 0) {
 		*errcode = WERR_INVALID_PARAM;
 		return False;
 	}
 
 	/* make sure the database is up to date */
-	if (print_cache_expired(snum))
-		print_queue_update(snum);
+	if (print_cache_expired(lp_const_servicename(snum), True))
+		print_queue_update(snum, True);
 
 	/* Send a printer notify message */
 
@@ -2569,10 +2701,13 @@ BOOL print_queue_purge(struct current_user *user, int snum, WERROR *errcode)
 	BOOL can_job_admin;
 
 	/* Force and update so the count is accurate (i.e. not a cached count) */
-	print_queue_update(snum);
+	print_queue_update(snum, True);
 	
 	can_job_admin = print_access_check(user, snum, JOB_ACCESS_ADMINISTER);
 	njobs = print_queue_status(snum, &queue, &status);
+	
+	if ( can_job_admin )
+		become_root();
 
 	for (i=0;i<njobs;i++) {
 		BOOL owner = is_owner(user, snum, queue[i].job);
@@ -2581,6 +2716,12 @@ BOOL print_queue_purge(struct current_user *user, int snum, WERROR *errcode)
 			print_job_delete1(snum, queue[i].job);
 		}
 	}
+	
+	if ( can_job_admin )
+		unbecome_root();
+
+	/* update the cache */
+	print_queue_update( snum, True );
 
 	SAFE_FREE(queue);
 
