@@ -1,6 +1,5 @@
 /* 
- *  Unix SMB/Netbios implementation.
- *  Version 1.9.
+ *  Unix SMB/CIFS implementation.
  *  MSDfs RPC Pipe client / server routines
  *  Copyright (C) Andrew Tridgell              1992-2000,
  *  Copyright (C) Luke Kenneth Casson Leighton 1996-2000,
@@ -26,6 +25,9 @@
 #include "nterr.h"
 #include "rpc_parse.h"   
 
+#undef DBGC_CLASS
+#define DBGC_CLASS DBGC_RPC_PARSE
+
 /******************************************************************* 
 Make a DFS_Q_DFS_QUERY structure
 *******************************************************************/
@@ -78,9 +80,9 @@ BOOL init_dfs_q_dfs_remove(DFS_Q_DFS_REMOVE *q_d, const char *entrypath,
 			   const char *servername, const char *sharename)
 {
 	DEBUG(5,("init_dfs_q_dfs_remove\n"));
-	init_unistr2(&q_d->DfsEntryPath, entrypath,  strlen(entrypath)+1);
-	init_unistr2(&q_d->ServerName,   servername, strlen(servername)+1);
-	init_unistr2(&q_d->ShareName,    sharename,  strlen(sharename)+1);
+	init_unistr2(&q_d->DfsEntryPath, entrypath, UNI_STR_TERMINATE);
+	init_unistr2(&q_d->ServerName, servername, UNI_STR_TERMINATE);
+	init_unistr2(&q_d->ShareName, sharename, UNI_STR_TERMINATE);
 	q_d->ptr_ServerName = q_d->ptr_ShareName = 1;
 	return True;
 }
@@ -147,16 +149,17 @@ BOOL dfs_io_r_dfs_remove(const char *desc, DFS_R_DFS_REMOVE *r_d, prs_struct *ps
 Make a DFS_Q_DFS_ADD structure
 *******************************************************************/
 
-BOOL init_dfs_q_dfs_add(DFS_Q_DFS_ADD *q_d, const char *entrypath, const char *servername,
-			const char *sharename, const char *comment, uint32 flags)
+BOOL init_dfs_q_dfs_add(DFS_Q_DFS_ADD *q_d, const char *entrypath, 
+			const char *servername, const char *sharename, 
+			const char *comment, uint32 flags)
 {
 	DEBUG(5,("init_dfs_q_dfs_add\n"));
 	q_d->ptr_DfsEntryPath = q_d->ptr_ServerName = q_d->ptr_ShareName = 1;
-	init_unistr2(&q_d->DfsEntryPath, entrypath,  strlen(entrypath)+1);
-	init_unistr2(&q_d->ServerName,   servername, strlen(servername)+1);
-	init_unistr2(&q_d->ShareName,    sharename,  strlen(sharename)+1);
+	init_unistr2(&q_d->DfsEntryPath, entrypath, UNI_STR_TERMINATE);
+	init_unistr2(&q_d->ServerName, servername, UNI_STR_TERMINATE);
+	init_unistr2(&q_d->ShareName, sharename, UNI_STR_TERMINATE);
 	if(comment != NULL) {
-		init_unistr2(&q_d->Comment,      comment,    strlen(comment)+1);
+		init_unistr2(&q_d->Comment, comment,UNI_STR_TERMINATE);
 		q_d->ptr_Comment = 1;
 	} else {
 		q_d->ptr_Comment = 0;
@@ -234,9 +237,9 @@ BOOL init_dfs_q_dfs_get_info(DFS_Q_DFS_GET_INFO *q_d, const char *entrypath,
 			     uint32 info_level)
 {
 	DEBUG(5,("init_dfs_q2_get_info\n"));
-	init_unistr2(&q_d->uni_path, entrypath,  strlen(entrypath)+1);
-	init_unistr2(&q_d->uni_server,   servername, strlen(servername)+1);
-	init_unistr2(&q_d->uni_share,    sharename,  strlen(sharename)+1);
+	init_unistr2(&q_d->uni_path, entrypath, UNI_STR_TERMINATE);
+	init_unistr2(&q_d->uni_server, servername, UNI_STR_TERMINATE);
+	init_unistr2(&q_d->uni_share, sharename, UNI_STR_TERMINATE);
 	q_d->level = info_level;
 	q_d->ptr_server = q_d->ptr_share = 1;
 	return True;
@@ -246,7 +249,7 @@ BOOL init_dfs_q_dfs_get_info(DFS_Q_DFS_GET_INFO *q_d, const char *entrypath,
  Read/write a DFS_Q_GET_INFO structure
  ************************************************************/
 
-BOOL dfs_io_q_dfs_get_info(const char* desc, DFS_Q_DFS_GET_INFO* q_i, prs_struct* ps, int depth)
+BOOL dfs_io_q_dfs_get_info(const char *desc, DFS_Q_DFS_GET_INFO* q_i, prs_struct* ps, int depth)
 {
 	if(q_i == NULL)
 		return False;
@@ -286,7 +289,7 @@ BOOL dfs_io_q_dfs_get_info(const char* desc, DFS_Q_DFS_GET_INFO* q_i, prs_struct
  Read/write a DFS_R_GET_INFO structure
  ************************************************************/
 
-BOOL dfs_io_r_dfs_get_info(const char* desc, DFS_R_DFS_GET_INFO* r_i, prs_struct* ps, int depth)
+BOOL dfs_io_r_dfs_get_info(const char *desc, DFS_R_DFS_GET_INFO* r_i, prs_struct* ps, int depth)
 {
 	if(r_i == NULL)
 		return False;
@@ -371,7 +374,7 @@ BOOL dfs_io_dfs_info_ctr(const char *desc, DFS_INFO_CTR* ctr, uint32 num_entries
 		depth++;
 		/* should depend on whether marshalling or unmarshalling! */
 		if(UNMARSHALLING(ps)) {
-			ctr->dfs.info1 = (DFS_INFO_1 *)prs_alloc_mem(ps, sizeof(DFS_INFO_1)*num_entries);
+			ctr->dfs.info1 = PRS_ALLOC_MEM(ps, DFS_INFO_1, num_entries);
 			if (!ctr->dfs.info1)
 				return False;
 		}
@@ -391,7 +394,7 @@ BOOL dfs_io_dfs_info_ctr(const char *desc, DFS_INFO_CTR* ctr, uint32 num_entries
 	case 2:
 		depth++;
 		if(UNMARSHALLING(ps)) {
-			ctr->dfs.info2 = (DFS_INFO_2 *)prs_alloc_mem(ps, num_entries*sizeof(DFS_INFO_2));
+			ctr->dfs.info2 = PRS_ALLOC_MEM(ps, DFS_INFO_2, num_entries);
 			if (!ctr->dfs.info2)
 				return False;
 		}
@@ -421,7 +424,7 @@ BOOL dfs_io_dfs_info_ctr(const char *desc, DFS_INFO_CTR* ctr, uint32 num_entries
 	case 3:
 		depth++;
 		if(UNMARSHALLING(ps)) {
-			ctr->dfs.info3 = (DFS_INFO_3 *)prs_alloc_mem(ps, num_entries*sizeof(DFS_INFO_3));
+			ctr->dfs.info3 = PRS_ALLOC_MEM(ps, DFS_INFO_3, num_entries);
 			if (!ctr->dfs.info3)
 				return False;
 		}
@@ -514,7 +517,7 @@ BOOL dfs_io_dfs_storage_info(const char *desc, DFS_INFO_3* info3, prs_struct *ps
 	depth++;
 
 	if(UNMARSHALLING(ps)) {
-		info3->storages = (DFS_STORAGE_INFO *)prs_alloc_mem(ps, info3->num_storage_infos*sizeof(DFS_STORAGE_INFO));
+		info3->storages = PRS_ALLOC_MEM(ps, DFS_STORAGE_INFO, info3->num_storage_infos);
 		if (!info3->storages)
 			return False;
 	}
