@@ -36,7 +36,7 @@ interface/version dce/rpc pipe identification
 		0x8a885d04, 0x1ceb, 0x11c9, \
 		{ 0x9f, 0xe8 },             \
 		{ 0x08, 0x00,               \
-		0x2b, 0x10, 0x48, 0x60 }    \
+       		  0x2b, 0x10, 0x48, 0x60 }  \
 	}, 0x02                             \
 }
 
@@ -46,7 +46,7 @@ interface/version dce/rpc pipe identification
 		0x8a885d04, 0x1ceb, 0x11c9, \
 		{ 0x9f, 0xe8 },             \
 		{ 0x08, 0x00,               \
-		0x2b, 0x10, 0x48, 0x60 }    \
+		  0x2b, 0x10, 0x48, 0x60 }  \
 	}, 0x02                             \
 }
 
@@ -56,7 +56,7 @@ interface/version dce/rpc pipe identification
 		0x6bffd098, 0xa112, 0x3610, \
 		{ 0x98, 0x33 },             \
 		{ 0x46, 0xc3,               \
-		0xf8, 0x7e, 0x34, 0x5a }    \
+		  0xf8, 0x7e, 0x34, 0x5a }  \
 	}, 0x01                             \
 }
 
@@ -66,7 +66,7 @@ interface/version dce/rpc pipe identification
 		0x4b324fc8, 0x1670, 0x01d3, \
 		{ 0x12, 0x78 },             \
 		{ 0x5a, 0x47,               \
-		0xbf, 0x6e, 0xe1, 0x88 }    \
+		  0xbf, 0x6e, 0xe1, 0x88 }  \
 	}, 0x03                             \
 }
 
@@ -76,7 +76,7 @@ interface/version dce/rpc pipe identification
 		0x12345778, 0x1234, 0xabcd, \
 		{ 0xef, 0x00 },             \
 		{ 0x01, 0x23,               \
-		0x45, 0x67, 0x89, 0xab }    \
+		  0x45, 0x67, 0x89, 0xab }  \
 	}, 0x00                             \
 }
 
@@ -86,7 +86,7 @@ interface/version dce/rpc pipe identification
 		0x3919286a, 0xb10c, 0x11d0, \
 		{ 0x9b, 0xa8 },             \
 		{ 0x00, 0xc0,               \
-		0x4f, 0xd9, 0x2e, 0xf5 }    \
+		  0x4f, 0xd9, 0x2e, 0xf5 }  \
 	}, 0x00                             \
 }
 
@@ -96,7 +96,7 @@ interface/version dce/rpc pipe identification
 		0x12345778, 0x1234, 0xabcd, \
 		{ 0xef, 0x00 },             \
 		{ 0x01, 0x23,               \
-		0x45, 0x67, 0x89, 0xac }    \
+		  0x45, 0x67, 0x89, 0xac }  \
 	}, 0x01                             \
 }
 
@@ -106,7 +106,7 @@ interface/version dce/rpc pipe identification
 		0x12345678, 0x1234, 0xabcd, \
 		{ 0xef, 0x00 },             \
 		{ 0x01, 0x23,               \
-		0x45, 0x67, 0xcf, 0xfb }    \
+		  0x45, 0x67, 0xcf, 0xfb }  \
 	}, 0x01                             \
 }
 
@@ -116,7 +116,7 @@ interface/version dce/rpc pipe identification
 		0x338cd001, 0x2244, 0x31f1, \
 		{ 0xaa, 0xaa },             \
 		{ 0x90, 0x00,               \
-		0x38, 0x00, 0x10, 0x03 }    \
+		  0x38, 0x00, 0x10, 0x03 }  \
 	}, 0x01                             \
 }
 
@@ -126,7 +126,7 @@ interface/version dce/rpc pipe identification
 		0x12345678, 0x1234, 0xabcd, \
 		{ 0xef, 0x00 },             \
 		{ 0x01, 0x23,               \
-		0x45, 0x67, 0x89, 0xab }    \
+		  0x45, 0x67, 0x89, 0xab }  \
 	}, 0x01                             \
 }
 
@@ -136,7 +136,7 @@ interface/version dce/rpc pipe identification
 		0x0, 0x0, 0x0,              \
 		{ 0x00, 0x00 },             \
 		{ 0x00, 0x00,               \
-		0x00, 0x00, 0x00, 0x00 }    \
+		  0x00, 0x00, 0x00, 0x00 }  \
 	}, 0x00                             \
 }
 
@@ -170,6 +170,27 @@ interface/version dce/rpc pipe identification
         }, 0x01                             \
 }
 
+#define SYNT_SVCCTL_V2                      \
+{                                           \
+	{                                   \
+		0x367abb81, 0x9844, 0x35f1, \
+                { 0xad, 0x32 },             \
+                { 0x98, 0xf0,               \
+                  0x38, 0x00, 0x10, 0x03 }  \
+	}, 0x02                             \
+}
+
+
+#define SYNT_EVENTLOG_V0		    \
+{					    \
+	{				    \
+		0x82273fdc, 0xe32a, 0x18c3, \
+		{ 0x3f, 0x78 },		    \
+		{ 0x82, 0x79,		    \
+		  0x29, 0xdc, 0x23, 0xea }  \
+	}, 0x00				    \
+}
+
 /*
  * IMPORTANT!!  If you update this structure, make sure to
  * update the index #defines in smb.h.
@@ -189,6 +210,8 @@ const struct pipe_id_info pipe_names [] =
 	{ PIPE_NETDFS  , SYNT_NETDFS_V3        , PIPE_NETDFS   , TRANS_SYNT_V2 },
 	{ PIPE_ECHO    , SYNT_ECHO_V1          , PIPE_ECHO     , TRANS_SYNT_V2 },
 	{ PIPE_SHUTDOWN, SYNT_SHUTDOWN_V1      , PIPE_SHUTDOWN , TRANS_SYNT_V2 },
+	{ PIPE_SVCCTL  , SYNT_SVCCTL_V2        , PIPE_NTSVCS   , TRANS_SYNT_V2 },
+	{ PIPE_EVENTLOG, SYNT_EVENTLOG_V0      , PIPE_EVENTLOG , TRANS_SYNT_V2 },
 	{ NULL         , SYNT_NONE_V0          , NULL          , SYNT_NONE_V0  }
 };
 
@@ -353,34 +376,84 @@ static BOOL smb_io_rpc_hdr_bba(const char *desc,  RPC_HDR_BBA *rpc, prs_struct *
 	return True;
 }
 
+/*******************************************************************
+ Inits an RPC_CONTEXT structure.
+ Note the transfer pointer must remain valid until this is marshalled.
+********************************************************************/
+
+void init_rpc_context(RPC_CONTEXT *rpc_ctx, uint16 context_id, RPC_IFACE *abstract, RPC_IFACE *transfer)
+{
+	rpc_ctx->context_id   = context_id   ; /* presentation context identifier (0x0) */
+	rpc_ctx->num_transfer_syntaxes = 1 ; /* the number of syntaxes (has always been 1?)(0x1) */
+
+	/* num and vers. of interface client is using */
+	rpc_ctx->abstract = *abstract;
+
+	/* vers. of interface to use for replies */
+	rpc_ctx->transfer = transfer;
+}
+
 /*******************************************************************
  Inits an RPC_HDR_RB structure.
+ Note the context pointer must remain valid until this is marshalled.
 ********************************************************************/
 
 void init_rpc_hdr_rb(RPC_HDR_RB *rpc, 
 				uint16 max_tsize, uint16 max_rsize, uint32 assoc_gid,
-				uint32 num_elements, uint16 context_id, uint8 num_syntaxes,
-				RPC_IFACE *abstract, RPC_IFACE *transfer)
+				RPC_CONTEXT *context)
 {
 	init_rpc_hdr_bba(&rpc->bba, max_tsize, max_rsize, assoc_gid);
 
-	rpc->num_elements = num_elements ; /* the number of elements (0x1) */
-	rpc->context_id   = context_id   ; /* presentation context identifier (0x0) */
-	rpc->num_syntaxes = num_syntaxes ; /* the number of syntaxes (has always been 1?)(0x1) */
-
-	/* num and vers. of interface client is using */
-	rpc->abstract = *abstract;
-
-	/* num and vers. of interface to use for replies */
-	rpc->transfer = *transfer;
+	rpc->num_contexts = 1;
+	rpc->rpc_context = context;
 }
 
+/*******************************************************************
+ Reads or writes an RPC_CONTEXT structure.
+********************************************************************/
+
+BOOL smb_io_rpc_context(const char *desc, RPC_CONTEXT *rpc_ctx, prs_struct *ps, int depth)
+{
+	int i;
+
+	if (rpc_ctx == NULL)
+		return False;
+
+	if(!prs_align(ps))
+		return False;
+	if(!prs_uint16("context_id  ", ps, depth, &rpc_ctx->context_id ))
+		return False;
+	if(!prs_uint8 ("num_transfer_syntaxes", ps, depth, &rpc_ctx->num_transfer_syntaxes))
+		return False;
+
+	/* num_transfer_syntaxes must not be zero. */
+	if (rpc_ctx->num_transfer_syntaxes == 0)
+		return False;
+
+	if(!smb_io_rpc_iface("", &rpc_ctx->abstract, ps, depth))
+		return False;
+
+	if (UNMARSHALLING(ps)) {
+		if (!(rpc_ctx->transfer = PRS_ALLOC_MEM(ps, RPC_IFACE, rpc_ctx->num_transfer_syntaxes))) {
+			return False;
+		}
+	}
+
+	for (i = 0; i < rpc_ctx->num_transfer_syntaxes; i++ ) {
+		if (!smb_io_rpc_iface("", &rpc_ctx->transfer[i], ps, depth))
+			return False;
+	}
+	return True;
+} 
+
 /*******************************************************************
  Reads or writes an RPC_HDR_RB structure.
 ********************************************************************/
 
 BOOL smb_io_rpc_hdr_rb(const char *desc, RPC_HDR_RB *rpc, prs_struct *ps, int depth)
 {
+	int i;
+	
 	if (rpc == NULL)
 		return False;
 
@@ -390,18 +463,26 @@ BOOL smb_io_rpc_hdr_rb(const char *desc, RPC_HDR_RB *rpc, prs_struct *ps, int de
 	if(!smb_io_rpc_hdr_bba("", &rpc->bba, ps, depth))
 		return False;
 
-	if(!prs_uint32("num_elements", ps, depth, &rpc->num_elements))
-		return False;
-	if(!prs_uint16("context_id  ", ps, depth, &rpc->context_id ))
-		return False;
-	if(!prs_uint8 ("num_syntaxes", ps, depth, &rpc->num_syntaxes))
+	if(!prs_uint8("num_contexts", ps, depth, &rpc->num_contexts))
 		return False;
 
-	if(!smb_io_rpc_iface("", &rpc->abstract, ps, depth))
-		return False;
-	if(!smb_io_rpc_iface("", &rpc->transfer, ps, depth))
+	/* 3 pad bytes following - will be mopped up by the prs_align in smb_io_rpc_context(). */
+
+	/* num_contexts must not be zero. */
+	if (rpc->num_contexts == 0)
 		return False;
 
+	if (UNMARSHALLING(ps)) {
+		if (!(rpc->rpc_context = PRS_ALLOC_MEM(ps, RPC_CONTEXT, rpc->num_contexts))) {
+			return False;
+		}
+	}
+
+	for (i = 0; i < rpc->num_contexts; i++ ) {
+		if (!smb_io_rpc_context("", &rpc->rpc_context[i], ps, depth))
+			return False;
+	}
+
 	return True;
 }
 
@@ -568,73 +649,20 @@ BOOL smb_io_rpc_hdr_fault(const char *desc, RPC_HDR_FAULT *rpc, prs_struct *ps,
     return True;
 }
 
-/*******************************************************************
- Init an RPC_HDR_AUTHA structure.
-********************************************************************/
-
-void init_rpc_hdr_autha(RPC_HDR_AUTHA *rai,
-				uint16 max_tsize, uint16 max_rsize,
-				uint8 auth_type, uint8 auth_level,
-				uint8 stub_type_len)
-{
-	rai->max_tsize = max_tsize; /* maximum transmission fragment size (0x1630) */
-	rai->max_rsize = max_rsize; /* max receive fragment size (0x1630) */   
-
-	rai->auth_type     = auth_type; /* nt lm ssp 0x0a */
-	rai->auth_level    = auth_level; /* 0x06 */
-	rai->stub_type_len = stub_type_len; /* 0x00 */
-	rai->padding       = 0; /* padding 0x00 */
-
-	rai->unknown       = 0x0014a0c0; /* non-zero pointer to something */
-}
-
-/*******************************************************************
- Reads or writes an RPC_HDR_AUTHA structure.
-********************************************************************/
-
-BOOL smb_io_rpc_hdr_autha(const char *desc, RPC_HDR_AUTHA *rai, prs_struct *ps, int depth)
-{
-	if (rai == NULL)
-		return False;
-
-	prs_debug(ps, depth, desc, "smb_io_rpc_hdr_autha");
-	depth++;
-
-	if(!prs_uint16("max_tsize    ", ps, depth, &rai->max_tsize))
-		return False;
-	if(!prs_uint16("max_rsize    ", ps, depth, &rai->max_rsize))
-		return False;
-
-	if(!prs_uint8 ("auth_type    ", ps, depth, &rai->auth_type)) /* 0x0a nt lm ssp */
-		return False;
-	if(!prs_uint8 ("auth_level   ", ps, depth, &rai->auth_level)) /* 0x06 */
-		return False;
-	if(!prs_uint8 ("stub_type_len", ps, depth, &rai->stub_type_len))
-		return False;
-	if(!prs_uint8 ("padding      ", ps, depth, &rai->padding))
-		return False;
-
-	if(!prs_uint32("unknown      ", ps, depth, &rai->unknown)) /* 0x0014a0c0 */
-		return False;
-
-	return True;
-}
-
 /*******************************************************************
  Inits an RPC_HDR_AUTH structure.
 ********************************************************************/
 
 void init_rpc_hdr_auth(RPC_HDR_AUTH *rai,
 				uint8 auth_type, uint8 auth_level,
-				uint8 padding,
-				uint32 ptr)
+				uint8 auth_pad_len,
+				uint32 auth_context_id)
 {
 	rai->auth_type     = auth_type; /* nt lm ssp 0x0a */
 	rai->auth_level    = auth_level; /* 0x06 */
-	rai->padding       = padding;
-	rai->reserved      = 0;
-
-	rai->auth_context  = ptr; /* non-zero pointer to something */
+	rai->auth_pad_len  = auth_pad_len;
+	rai->auth_reserved = 0;
+	rai->auth_context_id = auth_context_id;
 }
 
 /*******************************************************************
@@ -656,11 +684,48 @@ BOOL smb_io_rpc_hdr_auth(const char *desc, RPC_HDR_AUTH *rai, prs_struct *ps, in
 		return False;
 	if(!prs_uint8 ("auth_level   ", ps, depth, &rai->auth_level)) /* 0x06 */
 		return False;
-	if(!prs_uint8 ("padding      ", ps, depth, &rai->padding))
+	if(!prs_uint8 ("auth_pad_len ", ps, depth, &rai->auth_pad_len))
 		return False;
-	if(!prs_uint8 ("reserved     ", ps, depth, &rai->reserved))
+	if(!prs_uint8 ("auth_reserved", ps, depth, &rai->auth_reserved))
 		return False;
-	if(!prs_uint32("auth_context ", ps, depth, &rai->auth_context))
+	if(!prs_uint32("auth_context_id", ps, depth, &rai->auth_context_id))
+		return False;
+
+	return True;
+}
+
+
+/*******************************************************************
+ Init an RPC_HDR_AUTHA structure.
+********************************************************************/
+
+void init_rpc_hdr_autha(RPC_HDR_AUTHA *rai,
+				uint16 max_tsize, uint16 max_rsize,
+				RPC_HDR_AUTH *auth)
+{
+	rai->max_tsize = max_tsize; /* maximum transmission fragment size (0x1630) */
+	rai->max_rsize = max_rsize; /* max receive fragment size (0x1630) */   
+	rai->auth = *auth;
+}
+
+/*******************************************************************
+ Reads or writes an RPC_HDR_AUTHA structure.
+********************************************************************/
+
+BOOL smb_io_rpc_hdr_autha(const char *desc, RPC_HDR_AUTHA *rai, prs_struct *ps, int depth)
+{
+	if (rai == NULL)
+		return False;
+
+	prs_debug(ps, depth, desc, "smb_io_rpc_hdr_autha");
+	depth++;
+
+	if(!prs_uint16("max_tsize    ", ps, depth, &rai->max_tsize))
+		return False;
+	if(!prs_uint16("max_rsize    ", ps, depth, &rai->max_rsize))
+		return False;
+
+	if(!smb_io_rpc_hdr_auth("auth", &rai->auth, ps, depth))
 		return False;
 
 	return True;
@@ -1189,7 +1254,8 @@ BOOL smb_io_rpc_auth_netsec_neg(const char *desc, RPC_AUTH_NETSEC_NEG *neg,
 /*******************************************************************
 reads or writes an RPC_AUTH_NETSEC_CHK structure.
 ********************************************************************/
-BOOL smb_io_rpc_auth_netsec_chk(const char *desc, RPC_AUTH_NETSEC_CHK * chk,
+BOOL smb_io_rpc_auth_netsec_chk(const char *desc, int auth_len, 
+                                RPC_AUTH_NETSEC_CHK * chk,
 				prs_struct *ps, int depth)
 {
 	if (chk == NULL)
@@ -1198,10 +1264,19 @@ BOOL smb_io_rpc_auth_netsec_chk(const char *desc, RPC_AUTH_NETSEC_CHK * chk,
 	prs_debug(ps, depth, desc, "smb_io_rpc_auth_netsec_chk");
 	depth++;
 
-	prs_uint8s(False, "sig  ", ps, depth, chk->sig, sizeof(chk->sig));
-	prs_uint8s(False, "seq_num", ps, depth, chk->seq_num, sizeof(chk->seq_num));
-	prs_uint8s(False, "packet_digest", ps, depth, chk->packet_digest, sizeof(chk->packet_digest));
-	prs_uint8s(False, "confounder", ps, depth, chk->confounder, sizeof(chk->confounder));
+	if ( !prs_uint8s(False, "sig  ", ps, depth, chk->sig, sizeof(chk->sig)) )
+		return False;
+		
+	if ( !prs_uint8s(False, "seq_num", ps, depth, chk->seq_num, sizeof(chk->seq_num)) )
+		return False;
+		
+	if ( !prs_uint8s(False, "packet_digest", ps, depth, chk->packet_digest, sizeof(chk->packet_digest)) )
+		return False;
+	
+	if ( auth_len == RPC_AUTH_NETSEC_SIGN_OR_SEAL_CHK_LEN ) {
+		if ( !prs_uint8s(False, "confounder", ps, depth, chk->confounder, sizeof(chk->confounder)) )
+			return False;
+	}
 
 	return True;
 }
