@@ -26,6 +26,10 @@
 
 #include "includes.h"
 
+extern struct dcinfo last_dcinfo;
+extern BOOL server_auth2_negotiated;
+extern userdom_struct current_user_info;
+
 #undef DBGC_CLASS
 #define DBGC_CLASS DBGC_RPC_SRV
 
@@ -53,12 +57,6 @@ static void init_net_r_req_chal(NET_R_REQ_CHAL *r_c,
  net_reply_logon_ctrl:
  *************************************************************************/
 
-/* Some flag values reverse engineered from NLTEST.EXE */
-
-#define LOGON_CTRL_IN_SYNC          0x00
-#define LOGON_CTRL_REPL_NEEDED      0x01
-#define LOGON_CTRL_REPL_IN_PROGRESS 0x02
-
 NTSTATUS _net_logon_ctrl(pipes_struct *p, NET_Q_LOGON_CTRL *q_u, 
 		       NET_R_LOGON_CTRL *r_u)
 {
@@ -424,7 +422,7 @@ NTSTATUS _net_auth_2(pipes_struct *p, NET_Q_AUTH_2 *q_u, NET_R_AUTH_2 *r_u)
 	init_net_r_auth_2(r_u, &srv_cred, &srv_flgs, status);
 
 	if (NT_STATUS_IS_OK(status)) {
-		extern struct dcinfo last_dcinfo;
+		server_auth2_negotiated = True;
 		last_dcinfo = p->dc;
 	}
 
@@ -451,6 +449,7 @@ NTSTATUS _net_srv_pwset(pipes_struct *p, NET_Q_SRV_PWSET *q_u, NET_R_SRV_PWSET *
 	if (!(p->dc.authenticated && deal_with_creds(p->dc.sess_key, &p->dc.clnt_cred, &q_u->clnt_id.cred, &srv_cred)))
 		return NT_STATUS_INVALID_HANDLE;
 
+	reseed_client_creds(&p->dc.clnt_cred, &q_u->clnt_id.cred);
 	memcpy(&p->dc.srv_cred, &p->dc.clnt_cred, sizeof(p->dc.clnt_cred));
 
 	DEBUG(5,("_net_srv_pwset: %d\n", __LINE__));
@@ -547,6 +546,8 @@ NTSTATUS _net_sam_logoff(pipes_struct *p, NET_Q_SAM_LOGOFF *q_u, NET_R_SAM_LOGOF
 						     &q_u->sam_id.client.cred, &srv_cred)))
 		return NT_STATUS_INVALID_HANDLE;
 
+	/* what happens if we get a logoff for an unknown user? */
+	reseed_client_creds(&p->dc.clnt_cred, &q_u->sam_id.client.cred);
 	memcpy(&p->dc.srv_cred, &p->dc.clnt_cred, sizeof(p->dc.clnt_cred));
 
 	/* XXXX maybe we want to say 'no', reject the client's credentials */
@@ -575,7 +576,6 @@ NTSTATUS _net_sam_logon(pipes_struct *p, NET_Q_SAM_LOGON *q_u, NET_R_SAM_LOGON *
 	fstring nt_username, nt_domain, nt_workstation;
 	auth_usersupplied_info *user_info = NULL;
 	auth_serversupplied_info *server_info = NULL;
-	extern userdom_struct current_user_info;
 	SAM_ACCOUNT *sampw;
 	struct auth_context *auth_context = NULL;
 	        
@@ -606,8 +606,6 @@ NTSTATUS _net_sam_logon(pipes_struct *p, NET_Q_SAM_LOGON *q_u, NET_R_SAM_LOGON *
 	if (!(p->dc.authenticated && deal_with_creds(p->dc.sess_key, &p->dc.clnt_cred, &q_u->sam_id.client.cred, &srv_cred)))
 		return NT_STATUS_INVALID_HANDLE;
 
-	memcpy(&p->dc.srv_cred, &p->dc.clnt_cred, sizeof(p->dc.clnt_cred));
-    
 	r_u->buffer_creds = 1; /* yes, we have valid server credentials */
 	memcpy(&r_u->srv_creds, &srv_cred, sizeof(r_u->srv_creds));
 
@@ -718,10 +716,25 @@ NTSTATUS _net_sam_logon(pipes_struct *p, NET_Q_SAM_LOGON *q_u, NET_R_SAM_LOGON *
 	/* Check account and password */
     
 	if (!NT_STATUS_IS_OK(status)) {
+		/* If we don't know what this domain is, we need to 
+		   indicate that we are not authoritative.  This 
+		   allows the client to decide if it needs to try 
+		   a local user.  Fix by jpjanosi@us.ibm.com, #2976 */
+                if ( NT_STATUS_EQUAL(status, NT_STATUS_NO_SUCH_USER) 
+		     && !strequal(nt_domain, get_global_sam_name())
+		     && !is_trusted_domain(nt_domain) )
+			r_u->auth_resp = 0; /* We are not authoritative */
+
 		free_server_info(&server_info);
 		return status;
 	}
 
+	/* moved from right after deal_with_creds above, since we weren't
+	   supposed to update unless logon was successful */
+
+	reseed_client_creds(&p->dc.clnt_cred, &q_u->sam_id.client.cred);
+	memcpy(&p->dc.srv_cred, &p->dc.clnt_cred, sizeof(p->dc.clnt_cred));
+    
 	if (server_info->guest) {
 		/* We don't like guest domain logons... */
 		DEBUG(5,("_net_sam_logon: Attempted domain logon as GUEST denied.\n"));
@@ -767,8 +780,13 @@ NTSTATUS _net_sam_logon(pipes_struct *p, NET_Q_SAM_LOGON *q_u, NET_R_SAM_LOGON *
 			return NT_STATUS_UNSUCCESSFUL;
 		}
 		
-		pstrcpy(my_name, global_myname());
-
+		
+		if(server_info->login_server) {
+		        pstrcpy(my_name, server_info->login_server);
+		} else {
+		        pstrcpy(my_name, global_myname());
+		}
+		
 		if (!NT_STATUS_IS_OK(status 
 				     = nt_token_to_group_list(p->mem_ctx, 
 							      &domain_sid, 
