@@ -40,6 +40,9 @@
 
 #include "includes.h"
 
+extern struct pipe_id_info pipe_names[];
+extern struct current_user current_user;
+
 #undef DBGC_CLASS
 #define DBGC_CLASS DBGC_RPC_SRV
 
@@ -49,6 +52,7 @@
  next. This is the way the netlogon schannel works.
 **************************************************************/
 struct dcinfo last_dcinfo;
+BOOL server_auth2_negotiated = False;
 
 static void NTLMSSPcalc_p( pipes_struct *p, unsigned char *data, int len)
 {
@@ -158,7 +162,7 @@ BOOL create_next_pdu(pipes_struct *p)
 
 	if(p->out_data.data_sent_length + data_len >= prs_offset(&p->out_data.rdata)) {
 		p->hdr.flags |= RPC_FLG_LAST;
-		if ((auth_seal || auth_verify) && (data_len_left % 8)) {
+		if ((auth_seal || auth_verify || p->netsec_auth_validated) && (data_len_left % 8)) {
 			ss_padding_len = 8 - (data_len_left % 8);
 			DEBUG(10,("create_next_pdu: adding sign/seal padding of %u\n",
 				ss_padding_len ));
@@ -578,9 +582,9 @@ BOOL api_pipe_bind_auth_resp(pipes_struct *p, prs_struct *rpc_in_p)
 		return False;
 	}
 
-	if (autha_info.auth_type != NTLMSSP_AUTH_TYPE || autha_info.auth_level != RPC_PIPE_AUTH_SEAL_LEVEL) {
+	if (autha_info.auth.auth_type != NTLMSSP_AUTH_TYPE || autha_info.auth.auth_level != RPC_PIPE_AUTH_SEAL_LEVEL) {
 		DEBUG(0,("api_pipe_bind_auth_resp: incorrect auth type (%d) or level (%d).\n",
-			(int)autha_info.auth_type, (int)autha_info.auth_level ));
+			(int)autha_info.auth.auth_type, (int)autha_info.auth.auth_level ));
 		return False;
 	}
 
@@ -662,7 +666,7 @@ static BOOL setup_bind_nak(pipes_struct *p)
 
 	if(!prs_uint16("reject code", &outgoing_rpc, 0, &zero)) {
 		prs_mem_free(&outgoing_rpc);
-        return False;
+		return False;
 	}
 
 	p->out_data.data_sent_length = 0;
@@ -751,7 +755,6 @@ BOOL setup_fault_pdu(pipes_struct *p, NTSTATUS status)
 BOOL check_bind_req(struct pipes_struct *p, RPC_IFACE* abstract,
                     RPC_IFACE* transfer, uint32 context_id)
 {
-	extern struct pipe_id_info pipe_names[];
 	char *pipe_name = p->name;
 	int i=0;
 	fstring pname;
@@ -765,6 +768,7 @@ BOOL check_bind_req(struct pipes_struct *p, RPC_IFACE* abstract,
 		
 	for ( i=0; pipe_names[i].client_pipe; i++ ) 
 	{
+		DEBUG(10,("checking %s\n", pipe_names[i].client_pipe));
 		if ( strequal(pipe_names[i].client_pipe, pname)
 			&& (abstract->version == pipe_names[i].abstr_syntax.version) 
 			&& (memcmp(&abstract->uuid, &pipe_names[i].abstr_syntax.uuid, sizeof(struct uuid)) == 0)
@@ -892,11 +896,9 @@ BOOL api_pipe_bind_req(pipes_struct *p, prs_struct *rpc_in_p)
 
 	if (i == rpc_lookup_size) {
 		if (NT_STATUS_IS_ERR(smb_probe_module("rpc", p->name))) {
-                       DEBUG(3,("api_pipe_bind_req: Unknown pipe name %s in bind request.\n",
-                                p->name ));
-                       if(!setup_bind_nak(p))
-                               return False;
-                       return True;
+			DEBUG(3,("api_pipe_bind_req: Unknown pipe name %s in bind request.\n",
+				p->name ));
+			return setup_bind_nak(p);
                 }
 
                 for (i = 0; i < rpc_lookup_size; i++) {
@@ -917,7 +919,7 @@ BOOL api_pipe_bind_req(pipes_struct *p, prs_struct *rpc_in_p)
 	/* decode the bind request */
 	if(!smb_io_rpc_hdr_rb("", &hdr_rb, rpc_in_p, 0))  {
 		DEBUG(0,("api_pipe_bind_req: unable to unmarshall RPC_HDR_RB struct.\n"));
-		return False;
+		return setup_bind_nak(p);
 	}
 
 	/*
@@ -934,64 +936,75 @@ BOOL api_pipe_bind_req(pipes_struct *p, prs_struct *rpc_in_p)
 
 		if(!smb_io_rpc_hdr_auth("", &auth_info, rpc_in_p, 0)) {
 			DEBUG(0,("api_pipe_bind_req: unable to unmarshall RPC_HDR_AUTH struct.\n"));
-			return False;
+			return setup_bind_nak(p);
 		}
 
-		if(auth_info.auth_type == NTLMSSP_AUTH_TYPE) {
+		switch(auth_info.auth_type) {
+			case NTLMSSP_AUTH_TYPE:
 
-			if(!smb_io_rpc_auth_verifier("", &auth_verifier, rpc_in_p, 0)) {
-				DEBUG(0,("api_pipe_bind_req: unable to "
-					 "unmarshall RPC_HDR_AUTH struct.\n"));
-				return False;
+				if(!smb_io_rpc_auth_verifier("", &auth_verifier, rpc_in_p, 0)) {
+					DEBUG(0,("api_pipe_bind_req: unable to "
+						 "unmarshall RPC_HDR_AUTH struct.\n"));
+					return setup_bind_nak(p);
+				}
+
+				if(!strequal(auth_verifier.signature, "NTLMSSP")) {
+					DEBUG(0,("api_pipe_bind_req: "
+						 "auth_verifier.signature != NTLMSSP\n"));
+					return setup_bind_nak(p);
+				}
+
+				if(auth_verifier.msg_type != NTLMSSP_NEGOTIATE) {
+					DEBUG(0,("api_pipe_bind_req: "
+						 "auth_verifier.msg_type (%d) != NTLMSSP_NEGOTIATE\n",
+						 auth_verifier.msg_type));
+					return setup_bind_nak(p);
+				}
+
+				if(!smb_io_rpc_auth_ntlmssp_neg("", &ntlmssp_neg, rpc_in_p, 0)) {
+					DEBUG(0,("api_pipe_bind_req: "
+						 "Failed to unmarshall RPC_AUTH_NTLMSSP_NEG.\n"));
+					return setup_bind_nak(p);
+				}
+
+				p->ntlmssp_chal_flags = SMBD_NTLMSSP_NEG_FLAGS;
+				p->ntlmssp_auth_requested = True;
+				break;
+
+			case NETSEC_AUTH_TYPE:
+			{
+				RPC_AUTH_NETSEC_NEG neg;
+				struct netsec_auth_struct *a = &(p->netsec_auth);
+
+				if (!server_auth2_negotiated) {
+					DEBUG(0, ("Attempt to bind using schannel "
+						  "without successful serverauth2\n"));
+					return setup_bind_nak(p);
+				}
+
+				if (!smb_io_rpc_auth_netsec_neg("", &neg, rpc_in_p, 0)) {
+					DEBUG(0,("api_pipe_bind_req: "
+						 "Could not unmarshal SCHANNEL auth neg\n"));
+					return setup_bind_nak(p);
+				}
+
+				p->netsec_auth_validated = True;
+
+				memset(a->sess_key, 0, sizeof(a->sess_key));
+				memcpy(a->sess_key, last_dcinfo.sess_key, sizeof(last_dcinfo.sess_key));
+
+				a->seq_num = 0;
+
+				DEBUG(10,("schannel auth: domain [%s] myname [%s]\n",
+					  neg.domain, neg.myname));
+				break;
 			}
 
-			if(!strequal(auth_verifier.signature, "NTLMSSP")) {
-				DEBUG(0,("api_pipe_bind_req: "
-					 "auth_verifier.signature != NTLMSSP\n"));
-				return False;
-			}
-
-			if(auth_verifier.msg_type != NTLMSSP_NEGOTIATE) {
-				DEBUG(0,("api_pipe_bind_req: "
-					 "auth_verifier.msg_type (%d) != NTLMSSP_NEGOTIATE\n",
-					 auth_verifier.msg_type));
-				return False;
-			}
-
-			if(!smb_io_rpc_auth_ntlmssp_neg("", &ntlmssp_neg, rpc_in_p, 0)) {
-				DEBUG(0,("api_pipe_bind_req: "
-					 "Failed to unmarshall RPC_AUTH_NTLMSSP_NEG.\n"));
-				return False;
-			}
-
-			p->ntlmssp_chal_flags = SMBD_NTLMSSP_NEG_FLAGS;
-			p->ntlmssp_auth_requested = True;
-
-		} else if (auth_info.auth_type == NETSEC_AUTH_TYPE) {
-
-			RPC_AUTH_NETSEC_NEG neg;
-			struct netsec_auth_struct *a = &(p->netsec_auth);
-
-			if (!smb_io_rpc_auth_netsec_neg("", &neg, rpc_in_p, 0)) {
-				DEBUG(0,("api_pipe_bind_req: "
-					 "Could not unmarshal SCHANNEL auth neg\n"));
-				return False;
-			}
-
-			p->netsec_auth_validated = True;
-
-			memset(a->sess_key, 0, sizeof(a->sess_key));
-			memcpy(a->sess_key, last_dcinfo.sess_key, sizeof(last_dcinfo.sess_key));
-
-			a->seq_num = 0;
-
-			DEBUG(10,("schannel auth: domain [%s] myname [%s]\n",
-				  neg.domain, neg.myname));
-
-		} else {
-			DEBUG(0,("api_pipe_bind_req: unknown auth type %x requested.\n",
-				 auth_info.auth_type ));
-			return False;
+			case SPNEGO_AUTH_TYPE:
+			default:
+				DEBUG(0,("api_pipe_bind_req: unknown auth type %x requested.\n",
+					 auth_info.auth_type ));
+				return setup_bind_nak(p);
 		}
 	}
 
@@ -1058,15 +1071,15 @@ BOOL api_pipe_bind_req(pipes_struct *p, prs_struct *rpc_in_p)
 		unknown to NT4)
 		Needed when adding entries to a DACL from NT5 - SK */
 
-	if(check_bind_req(p, &hdr_rb.abstract, &hdr_rb.transfer, hdr_rb.context_id )) 
-	{
+	if(check_bind_req(p, &hdr_rb.rpc_context[0].abstract, &hdr_rb.rpc_context[0].transfer[0],
+				hdr_rb.rpc_context[0].context_id )) {
 		init_rpc_hdr_ba(&hdr_ba,
 	                MAX_PDU_FRAG_LEN,
 	                MAX_PDU_FRAG_LEN,
 	                assoc_gid,
 	                ack_pipe_name,
 	                0x1, 0x0, 0x0,
-	                &hdr_rb.transfer);
+	                &hdr_rb.rpc_context[0].transfer[0]);
 	} else {
 		RPC_IFACE null_interface;
 		ZERO_STRUCT(null_interface);
@@ -1426,7 +1439,6 @@ struct current_user *get_current_user(struct current_user *user, pipes_struct *p
 	if (p->ntlmssp_auth_validated) {
 		memcpy(user, &p->pipe_user, sizeof(struct current_user));
 	} else {
-		extern struct current_user current_user;
 		memcpy(user, &current_user, sizeof(struct current_user));
 	}
 
@@ -1631,6 +1643,12 @@ void get_pipe_fns( int idx, struct api_struct **fns, int *n_fns )
 		case PI_NETDFS:
 			netdfs_get_pipe_fns( &cmds, &n_cmds );
 			break;
+		case PI_SVCCTL:
+			svcctl_get_pipe_fns( &cmds, &n_cmds );
+			break;
+	        case PI_EVENTLOG:
+			eventlog_get_pipe_fns( &cmds, &n_cmds );
+			break;
 #ifdef DEVELOPER
 		case PI_ECHO:
 			echo_get_pipe_fns( &cmds, &n_cmds );
