@@ -27,6 +27,17 @@
 
 extern DOM_SID domain_sid;
 
+/****************************************************************************
+ display sam_user_info_7 structure
+ ****************************************************************************/
+static void display_sam_user_info_7(SAM_USER_INFO_7 *usr)
+{
+	fstring temp;
+
+	unistr2_to_ascii(temp, &usr->uni_name, sizeof(temp)-1);
+	printf("\tUser Name   :\t%s\n", temp);
+}
+
 /****************************************************************************
  display sam_user_info_21 structure
  ****************************************************************************/
@@ -130,6 +141,27 @@ static const char *display_time(NTTIME nttime)
 	return (string);
 }
 
+static const char* server_role_str(uint32 server_role)
+{
+	switch(server_role) {
+		case ROLE_STANDALONE:
+			return SMB_STRDUP("ROLE_STANDALONE");
+			break;
+		case ROLE_DOMAIN_MEMBER:
+			return SMB_STRDUP("ROLE_DOMAIN_MEMBER");
+			break;
+		case ROLE_DOMAIN_BDC:
+			return SMB_STRDUP("ROLE_DOMAIN_BDC");
+			break;
+		case ROLE_DOMAIN_PDC:
+			return SMB_STRDUP("ROLE_DOMAIN_PDC");
+			break;
+		default:
+			return SMB_STRDUP("Unknown -- internal error?");
+			break;
+	}
+}
+
 static void display_sam_unk_info_1(SAM_UNK_INFO_1 *info1)
 {
 	
@@ -148,26 +180,47 @@ static void display_sam_unk_info_2(SAM_UNK_INFO_2 *info2)
 	fstring name;
 
 	unistr2_to_ascii(name, &info2->uni_domain, sizeof(name) - 1); 
-	printf("Domain:\t%s\n", name);
+	printf("Domain:\t\t%s\n", name);
 
 	unistr2_to_ascii(name, &info2->uni_server, sizeof(name) - 1); 
-	printf("Server:\t%s\n", name);
+	printf("Server:\t\t%s\n", name);
+
+	unistr2_to_ascii(name, &info2->uni_comment, sizeof(name) - 1); 
+	printf("Comment:\t%s\n", name);
 
 	printf("Total Users:\t%d\n", info2->num_domain_usrs);
 	printf("Total Groups:\t%d\n", info2->num_domain_grps);
 	printf("Total Aliases:\t%d\n", info2->num_local_grps);
 	
-	printf("Sequence No:\t%d\n", info2->seq_num);
-	
-	printf("Unknown 0:\t0x%x\n", info2->unknown_0);
-	printf("Unknown 1:\t0x%x\n", info2->unknown_1);
-	printf("Unknown 2:\t0x%x\n", info2->unknown_2);
-	printf("Unknown 3:\t0x%x\n", info2->unknown_3);
+	printf("Sequence No:\t%d\n", info2->seq_num.low);
+
+	printf("Force Logoff:\t%d\n", (int)nt_time_to_unix_abs(&info2->logout));
+
 	printf("Unknown 4:\t0x%x\n", info2->unknown_4);
-	printf("Unknown 5:\t0x%x\n", info2->unknown_5);
+	printf("Server Role:\t%s\n", server_role_str(info2->server_role));
 	printf("Unknown 6:\t0x%x\n", info2->unknown_6);
 }
 
+static void display_sam_unk_info_7(SAM_UNK_INFO_7 *info7)
+{
+	printf("Server Role:\t%s\n", server_role_str(info7->server_role));
+}
+
+static void display_sam_unk_info_8(SAM_UNK_INFO_8 *info8)
+{
+	printf("Sequence No:\t%d\n", info8->seq_num.low);
+	printf("Domain Create Time:\t%s\n", 
+		http_timestring(nt_time_to_unix(&info8->domain_create_time)));
+
+}
+
+static void display_sam_unk_info_12(SAM_UNK_INFO_12 *info12)
+{
+	printf("Bad password lockout duration:               %s\n", display_time(info12->duration));
+	printf("Reset Lockout after:                         %s\n", display_time(info12->reset_count));
+	printf("Lockout after bad attempts:                  %d\n", info12->bad_attempt_lockout);
+}
+
 static void display_sam_info_1(SAM_ENTRY1 *e1, SAM_STR1 *s1)
 {
 	fstring tmp;
@@ -320,7 +373,17 @@ static NTSTATUS cmd_samr_query_user(struct cli_state *cli,
 	if (!NT_STATUS_IS_OK(result))
 		goto done;
 
-	display_sam_user_info_21(user_ctr->info.id21);
+	switch (user_ctr->switch_value) {
+	case 21:
+		display_sam_user_info_21(user_ctr->info.id21);
+		break;
+	case 7:
+		display_sam_user_info_7(user_ctr->info.id7);
+		break;
+	default:
+		printf("Unsupported infolevel: %d\n", info_level);
+		break;
+	}
 
 done:
 	return result;
@@ -505,25 +568,39 @@ static NTSTATUS cmd_samr_query_useraliases(struct cli_state *cli,
 {
 	POLICY_HND 		connect_pol, domain_pol;
 	NTSTATUS		result = NT_STATUS_UNSUCCESSFUL;
-	uint32 			user_rid, num_aliases, *alias_rids;
+	DOM_SID                *sids;
+	int                     num_sids;
+	uint32 			num_aliases, *alias_rids;
 	uint32			access_mask = MAXIMUM_ALLOWED_ACCESS;
 	int 			i;
 	fstring			server;
-	DOM_SID			tmp_sid;
-	DOM_SID2		sid;
-	DOM_SID global_sid_Builtin;
+	DOM_SID2	       *sid2;
 
-	string_to_sid(&global_sid_Builtin, "S-1-5-32");
-
-	if ((argc < 3) || (argc > 4)) {
-		printf("Usage: %s builtin|domain rid [access mask]\n", argv[0]);
-		return NT_STATUS_OK;
+	if (argc < 3) {
+		printf("Usage: %s builtin|domain sid1 sid2 ...\n", argv[0]);
+		return NT_STATUS_INVALID_PARAMETER;
 	}
 
-	sscanf(argv[2], "%i", &user_rid);
-	
-	if (argc > 3)
-		sscanf(argv[3], "%x", &access_mask);
+	sids = NULL;
+	num_sids = 0;
+
+	for (i=2; i<argc; i++) {
+		DOM_SID tmp_sid;
+		if (!string_to_sid(&tmp_sid, argv[i])) {
+			printf("%s is not a legal SID\n", argv[i]);
+			return NT_STATUS_INVALID_PARAMETER;
+		}
+		add_sid_to_array(mem_ctx, &tmp_sid, &sids, &num_sids);
+	}
+
+	sid2 = TALLOC_ARRAY(mem_ctx, DOM_SID2, num_sids);
+	if (sid2 == NULL)
+		return NT_STATUS_NO_MEMORY;
+
+	for (i=0; i<num_sids; i++) {
+		sid_copy(&sid2[i].sid, &sids[i]);
+		sid2[i].num_auths = sid2[i].sid.num_auths;
+	}
 
 	slprintf(server, sizeof(fstring)-1, "\\\\%s", cli->desthost);
 	strupper_m(server);
@@ -541,18 +618,19 @@ static NTSTATUS cmd_samr_query_useraliases(struct cli_state *cli,
 	else if (StrCaseCmp(argv[1], "builtin")==0)
 		result = cli_samr_open_domain(cli, mem_ctx, &connect_pol,
 					      access_mask,
-					      &global_sid_Builtin, &domain_pol);
-	else
-		return NT_STATUS_OK;
+					      &global_sid_Builtin,
+					      &domain_pol);
+	else {
+		printf("Usage: %s builtin|domain sid1 sid2 ...\n", argv[0]);
+		return NT_STATUS_INVALID_PARAMETER;
+	}
 
 	if (!NT_STATUS_IS_OK(result))
 		goto done;
 
-	sid_copy(&tmp_sid, &domain_sid);
-	sid_append_rid(&tmp_sid, user_rid);
-	init_dom_sid2(&sid, &tmp_sid);
-
-	result = cli_samr_query_useraliases(cli, mem_ctx, &domain_pol, 1, &sid, &num_aliases, &alias_rids);
+	result = cli_samr_query_useraliases(cli, mem_ctx, &domain_pol,
+					    num_sids, sid2,
+					    &num_aliases, &alias_rids);
 
 	if (!NT_STATUS_IS_OK(result))
 		goto done;
@@ -642,14 +720,17 @@ static NTSTATUS cmd_samr_enum_dom_users(struct cli_state *cli,
 	uint16 acb_mask = ACB_NORMAL;
 	BOOL got_connect_pol = False, got_domain_pol = False;
 
-	if ((argc < 1) || (argc > 2)) {
-		printf("Usage: %s [access_mask]\n", argv[0]);
+	if ((argc < 1) || (argc > 3)) {
+		printf("Usage: %s [access_mask] [acb_mask]\n", argv[0]);
 		return NT_STATUS_OK;
 	}
 	
 	if (argc > 1)
 		sscanf(argv[1], "%x", &access_mask);
 
+	if (argc > 2)
+		sscanf(argv[2], "%hx", &acb_mask);
+
 	/* Get sam policy handle */
 
 	result = try_samr_connects(cli, mem_ctx, MAXIMUM_ALLOWED_ACCESS, 
@@ -785,11 +866,8 @@ static NTSTATUS cmd_samr_enum_als_groups(struct cli_state *cli,
 	uint32 start_idx, size, num_als_groups, i;
 	uint32 access_mask = MAXIMUM_ALLOWED_ACCESS;
 	struct acct_info *als_groups;
-	DOM_SID global_sid_Builtin;
 	BOOL got_connect_pol = False, got_domain_pol = False;
 
-	string_to_sid(&global_sid_Builtin, "S-1-5-32");
-
 	if ((argc < 2) || (argc > 3)) {
 		printf("Usage: %s builtin|domain [access mask]\n", argv[0]);
 		return NT_STATUS_OK;
@@ -867,9 +945,6 @@ static NTSTATUS cmd_samr_query_aliasmem(struct cli_state *cli,
 	uint32 alias_rid, num_members, i;
 	uint32 access_mask = MAXIMUM_ALLOWED_ACCESS;
 	DOM_SID *alias_sids;
-	DOM_SID global_sid_Builtin;
-	
-	string_to_sid(&global_sid_Builtin, "S-1-5-32");
 
 	if ((argc < 3) || (argc > 4)) {
 		printf("Usage: %s builtin|domain rid [access mask]\n", argv[0]);
@@ -1020,7 +1095,7 @@ static NTSTATUS cmd_samr_query_dispinfo(struct cli_state *cli,
 	}
 
 
-	while(1) {
+	do {
 
 		if (!got_params)
 			get_query_dispinfo_params(
@@ -1033,7 +1108,7 @@ static NTSTATUS cmd_samr_query_dispinfo(struct cli_state *cli,
 
 		loop_count++;
 
-		if (!NT_STATUS_IS_OK(result) && !NT_STATUS_EQUAL(result, STATUS_MORE_ENTRIES)) 
+		if (NT_STATUS_IS_ERR(result))
 			break;
 
 		if (num_entries == 0) 
@@ -1058,7 +1133,7 @@ static NTSTATUS cmd_samr_query_dispinfo(struct cli_state *cli,
 				break;
 			}
 		}
-	}
+	} while ( NT_STATUS_EQUAL(result, STATUS_MORE_ENTRIES));
 
  done:
 	return result;
@@ -1121,6 +1196,15 @@ static NTSTATUS cmd_samr_query_dominfo(struct cli_state *cli,
 	case 2:
 		display_sam_unk_info_2(&ctr.info.inf2);
 		break;
+	case 7:
+		display_sam_unk_info_7(&ctr.info.inf7);
+		break;
+	case 8:
+		display_sam_unk_info_8(&ctr.info.inf8);
+		break;
+	case 12:
+		display_sam_unk_info_12(&ctr.info.inf12);
+		break;
 	default:
 		printf("cannot display domain info for switch value %d\n",
 		       switch_level);
@@ -1186,6 +1270,75 @@ static NTSTATUS cmd_samr_create_dom_user(struct cli_state *cli,
 	if (!NT_STATUS_IS_OK(result))
 		goto done;
 
+	result = cli_samr_close(cli, mem_ctx, &user_pol);
+	if (!NT_STATUS_IS_OK(result)) goto done;
+
+	result = cli_samr_close(cli, mem_ctx, &domain_pol);
+	if (!NT_STATUS_IS_OK(result)) goto done;
+
+	result = cli_samr_close(cli, mem_ctx, &connect_pol);
+	if (!NT_STATUS_IS_OK(result)) goto done;
+
+ done:
+	return result;
+}
+
+/* Create domain group */
+
+static NTSTATUS cmd_samr_create_dom_group(struct cli_state *cli, 
+                                          TALLOC_CTX *mem_ctx,
+                                          int argc, const char **argv) 
+{
+	POLICY_HND connect_pol, domain_pol, group_pol;
+	NTSTATUS result = NT_STATUS_UNSUCCESSFUL;
+	const char *grp_name;
+	uint32 access_mask = MAXIMUM_ALLOWED_ACCESS;
+
+	if ((argc < 2) || (argc > 3)) {
+		printf("Usage: %s groupname [access mask]\n", argv[0]);
+		return NT_STATUS_OK;
+	}
+
+	grp_name = argv[1];
+	
+	if (argc > 2)
+                sscanf(argv[2], "%x", &access_mask);
+
+	/* Get sam policy handle */
+
+	result = try_samr_connects(cli, mem_ctx, MAXIMUM_ALLOWED_ACCESS, 
+				   &connect_pol);
+
+	if (!NT_STATUS_IS_OK(result))
+		goto done;
+
+	/* Get domain policy handle */
+
+	result = cli_samr_open_domain(cli, mem_ctx, &connect_pol,
+				      access_mask,
+				      &domain_sid, &domain_pol);
+
+	if (!NT_STATUS_IS_OK(result))
+		goto done;
+
+	/* Create domain user */
+
+	result = cli_samr_create_dom_group(cli, mem_ctx, &domain_pol,
+					   grp_name, MAXIMUM_ALLOWED_ACCESS,
+					   &group_pol);
+
+	if (!NT_STATUS_IS_OK(result))
+		goto done;
+
+	result = cli_samr_close(cli, mem_ctx, &group_pol);
+	if (!NT_STATUS_IS_OK(result)) goto done;
+
+	result = cli_samr_close(cli, mem_ctx, &domain_pol);
+	if (!NT_STATUS_IS_OK(result)) goto done;
+
+	result = cli_samr_close(cli, mem_ctx, &connect_pol);
+	if (!NT_STATUS_IS_OK(result)) goto done;
+
  done:
 	return result;
 }
@@ -1202,9 +1355,6 @@ static NTSTATUS cmd_samr_lookup_names(struct cli_state *cli,
 	uint32 num_rids, num_names, *name_types, *rids;
 	const char **names;
 	int i;
-	DOM_SID global_sid_Builtin;
-
-	string_to_sid(&global_sid_Builtin, "S-1-5-32");
 
 	if (argc < 3) {
 		printf("Usage: %s  domain|builtin name1 [name2 [name3] [...]]\n", argv[0]);
@@ -1268,7 +1418,6 @@ static NTSTATUS cmd_samr_lookup_rids(struct cli_state *cli,
 {
 	NTSTATUS result = NT_STATUS_UNSUCCESSFUL;
 	POLICY_HND connect_pol, domain_pol;
-	uint32 flags = 0x000003e8; /* Unknown */
 	uint32 num_rids, num_names, *rids, *name_types;
 	char **names;
 	int i;
@@ -1301,11 +1450,11 @@ static NTSTATUS cmd_samr_lookup_rids(struct cli_state *cli,
 	for (i = 0; i < argc - 1; i++)
                 sscanf(argv[i + 1], "%i", &rids[i]);
 
-	result = cli_samr_lookup_rids(cli, mem_ctx, &domain_pol,
-				      flags, num_rids, rids,
+	result = cli_samr_lookup_rids(cli, mem_ctx, &domain_pol, num_rids, rids,
 				      &num_names, &names, &name_types);
 
-	if (!NT_STATUS_IS_OK(result))
+	if (!NT_STATUS_IS_OK(result) &&
+	    !NT_STATUS_EQUAL(result, STATUS_SOME_UNMAPPED))
 		goto done;
 
 	/* Display results */
@@ -1550,6 +1699,7 @@ struct cmd_set samr_commands[] = {
 	{ "enumalsgroups",      RPC_RTYPE_NTSTATUS, cmd_samr_enum_als_groups,       NULL, PI_SAMR,	"Enumerate alias groups",  "" },
 
 	{ "createdomuser",      RPC_RTYPE_NTSTATUS, cmd_samr_create_dom_user,       NULL, PI_SAMR,	"Create domain user",      "" },
+	{ "createdomgroup",     RPC_RTYPE_NTSTATUS, cmd_samr_create_dom_group,      NULL, PI_SAMR,	"Create domain group",     "" },
 	{ "samlookupnames",     RPC_RTYPE_NTSTATUS, cmd_samr_lookup_names,          NULL, PI_SAMR,	"Look up names",           "" },
 	{ "samlookuprids",      RPC_RTYPE_NTSTATUS, cmd_samr_lookup_rids,           NULL, PI_SAMR,	"Look up names",           "" },
 	{ "deletedomuser",      RPC_RTYPE_NTSTATUS, cmd_samr_delete_dom_user,       NULL, PI_SAMR,	"Delete domain user",      "" },
