@@ -20,11 +20,16 @@
 
 #include "includes.h"
 
+extern uint16 global_smbpid;
+extern int keepalive;
+extern struct auth_context *negprot_global_auth_context;
+extern int smb_echo_count;
+
 struct timeval smb_last_time;
 
 static char *InBuffer = NULL;
-char *OutBuffer = NULL;
-char *last_inbuf = NULL;
+static char *OutBuffer = NULL;
+static char *current_inbuf = NULL;
 
 /* 
  * Size of data we can send to client. Set
@@ -61,55 +66,216 @@ uint16 get_current_mid(void)
  for processing.
 ****************************************************************************/
 
-typedef struct {
-	ubi_slNode msg_next;
-	char *msg_buf;
-	int msg_len;
-} pending_message_list;
+static struct pending_message_list *smb_oplock_queue;
+static struct pending_message_list *smb_sharing_violation_queue;
 
-static ubi_slList smb_oplock_queue = { NULL, (ubi_slNodePtr)&smb_oplock_queue, 0};
+enum q_type { OPLOCK_QUEUE, SHARE_VIOLATION_QUEUE };
+
+/****************************************************************************
+ Free up a message.
+****************************************************************************/
+
+static void free_queued_message(struct pending_message_list *msg)
+{
+	data_blob_free(&msg->buf);
+	data_blob_free(&msg->private_data);
+	SAFE_FREE(msg);
+}
 
 /****************************************************************************
  Function to push a message onto the tail of a linked list of smb messages ready
  for processing.
 ****************************************************************************/
 
-static BOOL push_message(ubi_slList *list_head, char *buf, int msg_len)
+static BOOL push_queued_message(enum q_type qt, char *buf, int msg_len, struct timeval *ptv, char *private_data, size_t private_len)
 {
-	pending_message_list *msg = (pending_message_list *)
-                               malloc(sizeof(pending_message_list));
+	struct pending_message_list *tmp_msg;
+	struct pending_message_list *msg = SMB_MALLOC_P(struct pending_message_list);
 
 	if(msg == NULL) {
 		DEBUG(0,("push_message: malloc fail (1)\n"));
 		return False;
 	}
 
-	msg->msg_buf = (char *)malloc(msg_len);
-	if(msg->msg_buf == NULL) {
+	memset(msg,'\0',sizeof(*msg));
+
+	msg->buf = data_blob(buf, msg_len);
+	if(msg->buf.data == NULL) {
 		DEBUG(0,("push_message: malloc fail (2)\n"));
 		SAFE_FREE(msg);
 		return False;
 	}
 
-	memcpy(msg->msg_buf, buf, msg_len);
-	msg->msg_len = msg_len;
+	if (ptv) {
+		msg->msg_time = *ptv;
+	}
 
-	ubi_slAddTail( list_head, msg);
+	if (private_data) {
+		msg->private_data = data_blob(private_data, private_len);
+		if (msg->private_data.data == NULL) {
+			DEBUG(0,("push_message: malloc fail (3)\n"));
+			data_blob_free(&msg->buf);
+			SAFE_FREE(msg);
+			return False;
+		}
+	}
 
-	/* Push the MID of this packet on the signing queue. */
-	srv_defer_sign_response(SVAL(buf,smb_mid));
+	if (qt == OPLOCK_QUEUE) {
+		DLIST_ADD_END(smb_oplock_queue, msg, tmp_msg);
+	} else {
+		DLIST_ADD_END(smb_sharing_violation_queue, msg, tmp_msg);
+	}
+
+	DEBUG(10,("push_message: pushed message length %u on queue %s\n",
+		(unsigned int)msg_len,
+		qt == OPLOCK_QUEUE ? "smb_oplock_queue" : "smb_sharing_violation_queue" ));
 
 	return True;
 }
 
 /****************************************************************************
- Function to push a smb message onto a linked list of local smb messages ready
+ Function to push an oplock smb message onto a linked list of local smb messages ready
  for processing.
 ****************************************************************************/
 
 BOOL push_oplock_pending_smb_message(char *buf, int msg_len)
 {
-	return push_message(&smb_oplock_queue, buf, msg_len);
+	BOOL ret = push_queued_message(OPLOCK_QUEUE, buf, msg_len, NULL, NULL, 0);
+	if (ret) {
+		/* Push the MID of this packet on the signing queue. */
+		srv_defer_sign_response(SVAL(buf,smb_mid));
+	}
+	return ret;
+}
+
+/****************************************************************************
+ Function to delete a sharing violation open message by mid.
+****************************************************************************/
+
+void remove_sharing_violation_open_smb_message(uint16 mid)
+{
+	struct pending_message_list *pml;
+
+	if (!lp_defer_sharing_violations()) {
+		return;
+	}
+
+	for (pml = smb_sharing_violation_queue; pml; pml = pml->next) {
+		if (mid == SVAL(pml->buf.data,smb_mid)) {
+			DEBUG(10,("remove_sharing_violation_open_smb_message: deleting mid %u len %u\n",
+				(unsigned int)mid, (unsigned int)pml->buf.length ));
+			DLIST_REMOVE(smb_sharing_violation_queue, pml);
+			free_queued_message(pml);
+			return;
+		}
+	}
+}
+
+/****************************************************************************
+ Move a sharing violation open retry message to the front of the list and
+ schedule it for immediate processing.
+****************************************************************************/
+
+void schedule_sharing_violation_open_smb_message(uint16 mid)
+{
+	struct pending_message_list *pml;
+	int i = 0;
+
+	if (!lp_defer_sharing_violations()) {
+		return;
+	}
+
+	for (pml = smb_sharing_violation_queue; pml; pml = pml->next) {
+		uint16 msg_mid = SVAL(pml->buf.data,smb_mid);
+		DEBUG(10,("schedule_sharing_violation_open_smb_message: [%d] msg_mid = %u\n", i++,
+			(unsigned int)msg_mid ));
+		if (mid == msg_mid) {
+			DEBUG(10,("schedule_sharing_violation_open_smb_message: scheduling mid %u\n",
+				mid ));
+			pml->msg_time.tv_sec = 0;
+			pml->msg_time.tv_usec = 0;
+			DLIST_PROMOTE(smb_sharing_violation_queue, pml);
+			return;
+		}
+	}
+
+	DEBUG(10,("schedule_sharing_violation_open_smb_message: failed to find message mid %u\n",
+		mid ));
+}
+
+/****************************************************************************
+ Return true if this mid is on the deferred queue.
+****************************************************************************/
+
+BOOL open_was_deferred(uint16 mid)
+{
+	struct pending_message_list *pml;
+
+	if (!lp_defer_sharing_violations()) {
+		return False;
+	}
+
+	for (pml = smb_sharing_violation_queue; pml; pml = pml->next) {
+		if (SVAL(pml->buf.data,smb_mid) == mid) {
+			set_saved_error_triple(SMB_SUCCESS, 0, NT_STATUS_OK);
+			return True;
+		}
+	}
+	return False;
+}
+
+/****************************************************************************
+ Return the message queued by this mid.
+****************************************************************************/
+
+struct pending_message_list *get_open_deferred_message(uint16 mid)
+{
+	struct pending_message_list *pml;
+
+	if (!lp_defer_sharing_violations()) {
+		return NULL;
+	}
+
+	for (pml = smb_sharing_violation_queue; pml; pml = pml->next) {
+		if (SVAL(pml->buf.data,smb_mid) == mid) {
+			return pml;
+		}
+	}
+	return NULL;
+}
+
+/****************************************************************************
+ Function to push a sharing violation open smb message onto a linked list of local smb messages ready
+ for processing. We must use current_inbuf here not Inbuf in case we're in a chained message set.
+****************************************************************************/
+
+BOOL push_sharing_violation_open_smb_message(struct timeval *ptv, char *private_data, size_t priv_len)
+{
+	uint16 mid = SVAL(current_inbuf,smb_mid);
+	struct timeval tv;
+	SMB_BIG_INT tdif;
+
+	if (!lp_defer_sharing_violations()) {
+		return True;
+	}
+
+	tv = *ptv;
+	tdif = tv.tv_sec;
+	tdif *= 1000000;
+	tdif += tv.tv_usec;
+
+	/* Add on the timeout. */
+	tdif += SHARING_VIOLATION_USEC_WAIT;
+	
+	tv.tv_sec = tdif / 1000000;
+	tv.tv_usec = tdif % 1000000;
+	
+	DEBUG(10,("push_sharing_violation_open_smb_message: pushing message len %u mid %u\
+ timeout time [%u.%06u]\n", (unsigned int) smb_len(current_inbuf)+4, (unsigned int)mid,
+		(unsigned int)tv.tv_sec, (unsigned int)tv.tv_usec));
+
+	return push_queued_message(SHARE_VIOLATION_QUEUE, current_inbuf,
+			smb_len(current_inbuf)+4, &tv, private_data, priv_len);
 }
 
 /****************************************************************************
@@ -121,11 +287,17 @@ static void async_processing(char *buffer, int buffer_len)
 {
 	DEBUG(10,("async_processing: Doing async processing.\n"));
 
+	process_aio_queue();
+
 	/* check for oplock messages (both UDP and kernel) */
 	if (receive_local_message(buffer, buffer_len, 1)) {
 		process_local_message(buffer, buffer_len);
 	}
 
+	/* Do the aio check again after receive_local_message as it does a select
+	   and may have eaten our signal. */
+	process_aio_queue();
+
 	if (got_sig_term) {
 		exit_server("Caught TERM signal");
 	}
@@ -168,12 +340,17 @@ static BOOL receive_message_or_smb(char *buffer, int buffer_len, int timeout)
 	fd_set fds;
 	int selrtn;
 	struct timeval to;
+	struct timeval *pto;
 	int maxfd;
 
 	smb_read_error = 0;
 
  again:
 
+	to.tv_sec = timeout / 1000;
+	to.tv_usec = (timeout % 1000) * 1000;
+	pto = timeout > 0 ? &to : NULL;
+
 	/*
 	 * Note that this call must be before processing any SMB
 	 * messages as we need to synchronously process any messages
@@ -185,18 +362,57 @@ static BOOL receive_message_or_smb(char *buffer, int buffer_len, int timeout)
 	 * Check to see if we already have a message on the smb queue.
 	 * If so - copy and return it.
 	 */
-  	if(ubi_slCount(&smb_oplock_queue) != 0) {
-		pending_message_list *msg = (pending_message_list *)ubi_slRemHead(&smb_oplock_queue);
-		memcpy(buffer, msg->msg_buf, MIN(buffer_len, msg->msg_len));
+  	if(smb_oplock_queue != NULL) {
+		struct pending_message_list *msg = smb_oplock_queue;
+		memcpy(buffer, msg->buf.data, MIN(buffer_len, msg->buf.length));
   
 		/* Free the message we just copied. */
-		SAFE_FREE(msg->msg_buf);
-		SAFE_FREE(msg);
+		DLIST_REMOVE(smb_oplock_queue, msg);
+		free_queued_message(msg);
 		
 		DEBUG(5,("receive_message_or_smb: returning queued smb message.\n"));
 		return True;
 	}
 
+	/*
+	 * Check to see if we already have a message on the deferred open queue
+	 * and it's time to schedule.
+	 */
+  	if(smb_sharing_violation_queue != NULL) {
+		BOOL pop_message = False;
+		struct pending_message_list *msg = smb_sharing_violation_queue;
+
+		if (msg->msg_time.tv_sec == 0 && msg->msg_time.tv_usec == 0) {
+			pop_message = True;
+		} else {
+			struct timeval tv;
+			SMB_BIG_INT tdif;
+
+			GetTimeOfDay(&tv);
+			tdif = usec_time_diff(&msg->msg_time, &tv);
+			if (tdif <= 0) {
+				/* Timed out. Schedule...*/
+				pop_message = True;
+				DEBUG(10,("receive_message_or_smb: queued message timed out.\n"));
+			} else {
+				/* Make a more accurate select timeout. */
+				to.tv_sec = tdif / 1000000;
+				to.tv_usec = tdif % 1000000;
+				pto = &to;
+				DEBUG(10,("receive_message_or_smb: select with timeout of [%u.%06u]\n",
+					(unsigned int)pto->tv_sec, (unsigned int)pto->tv_usec ));
+			}
+		}
+
+		if (pop_message) {
+			memcpy(buffer, msg->buf.data, MIN(buffer_len, msg->buf.length));
+  
+			/* We leave this message on the queue so the open code can
+			   know this is a retry. */
+			DEBUG(5,("receive_message_or_smb: returning deferred open smb message.\n"));
+			return True;
+		}
+	}
 
 	/*
 	 * Setup the select read fd set.
@@ -227,10 +443,7 @@ static BOOL receive_message_or_smb(char *buffer, int buffer_len, int timeout)
 	FD_SET(smbd_server_fd(),&fds);
 	maxfd = setup_oplock_select_set(&fds);
 
-	to.tv_sec = timeout / 1000;
-	to.tv_usec = (timeout % 1000) * 1000;
-
-	selrtn = sys_select(MAX(maxfd,smbd_server_fd())+1,&fds,NULL,NULL,timeout>0?&to:NULL);
+	selrtn = sys_select(MAX(maxfd,smbd_server_fd())+1,&fds,NULL,NULL,pto);
 
 	/* if we get EINTR then maybe we have received an oplock
 	   signal - treat this as select returning 1. This is ugly, but
@@ -344,6 +557,7 @@ force write permissions on print services.
 #define CAN_IPC (1<<3)
 #define AS_GUEST (1<<5)
 #define QUEUE_IN_OPLOCK (1<<6)
+#define DO_CHDIR (1<<7)
 
 /* 
    define a list of possible SMB messages and their corresponding
@@ -373,7 +587,7 @@ static const struct smb_message_struct {
 /* 0x0e */ { "SMBctemp",reply_ctemp,AS_USER | QUEUE_IN_OPLOCK },
 /* 0x0f */ { "SMBmknew",reply_mknew,AS_USER}, 
 /* 0x10 */ { "SMBchkpth",reply_chkpth,AS_USER},
-/* 0x11 */ { "SMBexit",reply_exit,0},
+/* 0x11 */ { "SMBexit",reply_exit,DO_CHDIR},
 /* 0x12 */ { "SMBlseek",reply_lseek,AS_USER},
 /* 0x13 */ { "SMBlockread",reply_lockread,AS_USER},
 /* 0x14 */ { "SMBwriteunlock",reply_writeunlock,AS_USER},
@@ -469,7 +683,7 @@ static const struct smb_message_struct {
 /* 0x6e */ { NULL, NULL, 0 },
 /* 0x6f */ { NULL, NULL, 0 },
 /* 0x70 */ { "SMBtcon",reply_tcon,0},
-/* 0x71 */ { "SMBtdis",reply_tdis,0},
+/* 0x71 */ { "SMBtdis",reply_tdis,DO_CHDIR},
 /* 0x72 */ { "SMBnegprot",reply_negprot,0},
 /* 0x73 */ { "SMBsesssetupX",reply_sesssetup_and_X,0},
 /* 0x74 */ { "SMBulogoffX", reply_ulogoffX, 0}, /* ulogoff doesn't give a valid TID */
@@ -650,7 +864,6 @@ static int switch_message(int type,char *inbuf,char *outbuf,int size,int bufsize
 {
 	static pid_t pid= (pid_t)-1;
 	int outsize = 0;
-	extern uint16 global_smbpid;
 
 	type &= 0xff;
 
@@ -658,6 +871,8 @@ static int switch_message(int type,char *inbuf,char *outbuf,int size,int bufsize
 		pid = sys_getpid();
 
 	errno = 0;
+	set_saved_error_triple(0, 0, NT_STATUS_OK);
+
 	last_message = type;
 
 	/* Make sure this is an SMB packet. smb_size contains NetBIOS header so subtract 4 from it. */
@@ -682,7 +897,7 @@ static int switch_message(int type,char *inbuf,char *outbuf,int size,int bufsize
 		uint16 session_tag = (lp_security() == SEC_SHARE) ? UID_FIELD_INVALID : SVAL(inbuf,smb_uid);
 		connection_struct *conn = conn_find(SVAL(inbuf,smb_tid));
 
-		DEBUG(3,("switch message %s (pid %d)\n",smb_fn_name(type),(int)pid));
+		DEBUG(3,("switch message %s (pid %d) conn 0x%x\n",smb_fn_name(type),(int)pid,(unsigned int)conn));
 
 		smb_dump(smb_fn_name(type), 1, inbuf, size);
 		if(global_oplock_break) {
@@ -727,8 +942,14 @@ static int switch_message(int type,char *inbuf,char *outbuf,int size,int bufsize
 			change_to_root_user();
 
 		/* does this protocol need a valid tree connection? */
-		if ((flags & AS_USER) && !conn)
-			return ERROR_DOS(ERRSRV, ERRinvnid);
+		if ((flags & AS_USER) && !conn) {
+			/* Amazingly, the error code depends on the command (from Samba4). */
+			if (type == SMBntcreateX) {
+				return ERROR_NT(NT_STATUS_INVALID_HANDLE);
+			} else {
+				return ERROR_DOS(ERRSRV, ERRinvnid);
+			}
+		}
 
 
 		/* does this protocol need to be run as the connected user? */
@@ -754,16 +975,19 @@ static int switch_message(int type,char *inbuf,char *outbuf,int size,int bufsize
 			return(ERROR_DOS(ERRSRV,ERRaccess));	    
 
 		/* load service specific parameters */
-		if (conn && !set_current_service(conn,(flags & AS_USER)?True:False))
-			return(ERROR_DOS(ERRSRV,ERRaccess));
+		if (conn) {
+			if (!set_current_service(conn,SVAL(inbuf,smb_flg),(flags & (AS_USER|DO_CHDIR)?True:False))) {
+				return(ERROR_DOS(ERRSRV,ERRaccess));
+			}
+			conn->num_smb_operations++;
+		}
 
 		/* does this protocol need to be run as guest? */
 		if ((flags & AS_GUEST) && (!change_to_guest() || 
 				!check_access(smbd_server_fd(), lp_hostsallow(-1), lp_hostsdeny(-1))))
 			return(ERROR_DOS(ERRSRV,ERRaccess));
 
-		last_inbuf = inbuf;
-
+		current_inbuf = inbuf; /* In case we need to defer this message in open... */
 		outsize = smb_messages[type].fn(conn, inbuf,outbuf,size,bufsize);
 	}
 
@@ -928,7 +1152,12 @@ const char *smb_fn_name(int type)
  Helper functions for contruct_reply.
 ****************************************************************************/
 
-static uint32 common_flags2 = FLAGS2_LONG_PATH_COMPONENTS|FLAGS2_EXTENDED_SECURITY|FLAGS2_32_BIT_ERROR_CODES;
+static uint32 common_flags2 = FLAGS2_LONG_PATH_COMPONENTS|FLAGS2_32_BIT_ERROR_CODES;
+
+void add_to_common_flags2(uint32 v)
+{
+	common_flags2 |= v;
+}
 
 void remove_from_common_flags2(uint32 v)
 {
@@ -971,8 +1200,7 @@ int chain_reply(char *inbuf,char *outbuf,int size,int bufsize)
 	int outsize2;
 	char inbuf_saved[smb_wct];
 	char outbuf_saved[smb_wct];
-	int wct = CVAL(outbuf,smb_wct);
-	int outsize = smb_size + 2*wct + SVAL(outbuf,smb_vwv0+2*wct);
+	int outsize = smb_len(outbuf) + 4;
 
 	/* maybe its not chained */
 	if (smb_com2 == 0xFF) {
@@ -1076,16 +1304,51 @@ static int setup_select_timeout(void)
 
 void check_reload(int t)
 {
+	static pid_t mypid = 0;
 	static time_t last_smb_conf_reload_time = 0;
+	static time_t last_printer_reload_time = 0;
+	time_t printcap_cache_time = (time_t)lp_printcap_cache_time();
 
-	if(last_smb_conf_reload_time == 0)
+	if(last_smb_conf_reload_time == 0) {
 		last_smb_conf_reload_time = t;
+		/* Our printing subsystem might not be ready at smbd start up.
+		   Then no printer is available till the first printers check
+		   is performed.  A lower initial interval circumvents this. */
+		if ( printcap_cache_time > 60 )
+			last_printer_reload_time = t - printcap_cache_time + 60;
+		else
+			last_printer_reload_time = t;
+	}
+
+	if (mypid != getpid()) { /* First time or fork happened meanwhile */
+		/* randomize over 60 second the printcap reload to avoid all
+		 * process hitting cupsd at the same time */
+		int time_range = 60;
+
+		last_printer_reload_time += random() % time_range;
+		mypid = getpid();
+	}
 
 	if (reload_after_sighup || (t >= last_smb_conf_reload_time+SMBD_RELOAD_CHECK)) {
 		reload_services(True);
 		reload_after_sighup = False;
 		last_smb_conf_reload_time = t;
 	}
+
+	/* 'printcap cache time = 0' disable the feature */
+	
+	if ( printcap_cache_time != 0 )
+	{ 
+		/* see if it's time to reload or if the clock has been set back */
+		
+		if ( (t >= last_printer_reload_time+printcap_cache_time) 
+			|| (t-last_printer_reload_time  < 0) ) 
+		{
+			DEBUG( 3,( "Printcap cache time expired.\n"));
+			reload_printers();
+			last_printer_reload_time = t;
+		}
+	}
 }
 
 /****************************************************************************
@@ -1098,7 +1361,6 @@ static BOOL timeout_processing(int deadtime, int *select_timeout, time_t *last_t
 	static time_t last_idle_closed_check = 0;
 	time_t t;
 	BOOL allidle = True;
-	extern int keepalive;
 
 	if (smb_read_error == READ_EOF) {
 		DEBUG(3,("timeout_processing: End of file from client (client has disconnected).\n"));
@@ -1142,7 +1404,6 @@ static BOOL timeout_processing(int deadtime, int *select_timeout, time_t *last_t
 	}
 
 	if (keepalive && (t - last_keepalive_sent_time)>keepalive) {
-		extern struct auth_context *negprot_global_auth_context;
 		if (!send_keepalive(smbd_server_fd())) {
 			DEBUG( 2, ( "Keepalive failed - exiting.\n" ) );
 			return False;
@@ -1255,26 +1516,111 @@ machine %s in domain %s.\n", global_myname(), lp_workgroup()));
 }
 
 /****************************************************************************
-  process commands from the client
+ Accessor functions for InBuffer, OutBuffer.
+****************************************************************************/
+
+char *get_InBuffer(void)
+{
+	return InBuffer;
+}
+
+void set_InBuffer(char *new_inbuf)
+{
+	InBuffer = new_inbuf;
+	current_inbuf = InBuffer;
+}
+
+char *get_OutBuffer(void)
+{
+	return OutBuffer;
+}
+
+void set_OutBuffer(char *new_outbuf)
+{
+	OutBuffer = new_outbuf;
+}
+
+/****************************************************************************
+ Free an InBuffer. Checks if not in use by aio system.
+ Must have been allocated by NewInBuffer.
+****************************************************************************/
+
+void free_InBuffer(char *inbuf)
+{
+	if (!aio_inbuffer_in_use(inbuf)) {
+		if (current_inbuf == inbuf) {
+			current_inbuf = NULL;
+		}
+		SAFE_FREE(inbuf);
+	}
+}
+
+/****************************************************************************
+ Free an OutBuffer. No outbuffers currently stolen by aio system.
+ Must have been allocated by NewInBuffer.
+****************************************************************************/
+
+void free_OutBuffer(char *outbuf)
+{
+	SAFE_FREE(outbuf);
+}
+
+const int total_buffer_size = (BUFFER_SIZE + LARGE_WRITEX_HDR_SIZE + SAFETY_MARGIN);
+
+/****************************************************************************
+ Allocate a new InBuffer. Returns the new and old ones.
+****************************************************************************/
+
+char *NewInBuffer(char **old_inbuf)
+{
+	char *new_inbuf = (char *)SMB_MALLOC(total_buffer_size);
+	if (!new_inbuf) {
+		return NULL;
+	}
+	if (old_inbuf) {
+		*old_inbuf = InBuffer;
+	}
+	InBuffer = new_inbuf;
+#if defined(DEVELOPER)
+	clobber_region(SAFE_STRING_FUNCTION_NAME, SAFE_STRING_LINE, InBuffer, total_buffer_size);
+#endif
+	return InBuffer;
+}
+
+/****************************************************************************
+ Allocate a new OutBuffer. Returns the new and old ones.
+****************************************************************************/
+
+char *NewOutBuffer(char **old_outbuf)
+{
+	char *new_outbuf = (char *)SMB_MALLOC(total_buffer_size);
+	if (!new_outbuf) {
+		return NULL;
+	}
+	if (old_outbuf) {
+		*old_outbuf = OutBuffer;
+	}
+	OutBuffer = new_outbuf;
+#if defined(DEVELOPER)
+	clobber_region(SAFE_STRING_FUNCTION_NAME, SAFE_STRING_LINE, OutBuffer, total_buffer_size);
+#endif
+	return OutBuffer;
+}
+
+/****************************************************************************
+ Process commands from the client
 ****************************************************************************/
 
 void smbd_process(void)
 {
-	extern int smb_echo_count;
 	time_t last_timeout_processing_time = time(NULL);
 	unsigned int num_smbs = 0;
-	const size_t total_buffer_size = BUFFER_SIZE + LARGE_WRITEX_HDR_SIZE + SAFETY_MARGIN;
 
-	InBuffer = (char *)malloc(total_buffer_size);
-	OutBuffer = (char *)malloc(total_buffer_size);
-	if ((InBuffer == NULL) || (OutBuffer == NULL)) 
+	/* Allocate the primary Inbut/Output buffers. */
+
+	if ((NewInBuffer(NULL) == NULL) || (NewOutBuffer(NULL) == NULL)) 
 		return;
 
-#if defined(DEVELOPER)
-	clobber_region(SAFE_STRING_FUNCTION_NAME, SAFE_STRING_LINE, InBuffer, total_buffer_size);
-	clobber_region(SAFE_STRING_FUNCTION_NAME, SAFE_STRING_LINE, OutBuffer, total_buffer_size);
-#endif
-
 	max_recv = MIN(lp_maxxmit(),BUFFER_SIZE);
 
 	while (True) {
@@ -1291,10 +1637,6 @@ void smbd_process(void)
 		lp_talloc_free();
 		main_loop_talloc_free();
 
-		/* run all registered idle events */
-		smb_run_idle_events(time(NULL));
-
-
 		/* Did someone ask for immediate checks on things like blocking locks ? */
 		if (select_timeout == 0) {
 			if(!timeout_processing( deadtime, &select_timeout, &last_timeout_processing_time))
