@@ -30,6 +30,7 @@ int last_message = -1;
 /* a useful macro to debug the last message processed */
 #define LAST_MESSAGE() smb_fn_name(last_message)
 
+extern struct auth_context *negprot_global_auth_context;
 extern pstring user_socket_options;
 extern SIG_ATOMIC_T got_sig_term;
 extern SIG_ATOMIC_T reload_after_sighup;
@@ -64,7 +65,7 @@ static void smbd_set_server_fd(int fd)
 static void sig_term(void)
 {
 	got_sig_term = 1;
-	sys_select_signal();
+	sys_select_signal(SIGTERM);
 }
 
 /****************************************************************************
@@ -74,7 +75,7 @@ static void sig_term(void)
 static void sig_hup(int sig)
 {
 	reload_after_sighup = 1;
-	sys_select_signal();
+	sys_select_signal(SIGHUP);
 }
 
 /****************************************************************************
@@ -186,6 +187,7 @@ static BOOL open_sockets_smbd(BOOL is_daemon, BOOL interactive, const char *smb_
 	int fd_listenset[FD_SETSIZE];
 	fd_set listen_set;
 	int s;
+	int maxfd = 0;
 	int i;
 	char *ports;
 
@@ -241,7 +243,9 @@ static BOOL open_sockets_smbd(BOOL is_daemon, BOOL interactive, const char *smb_
 
 			for (ptr=ports; next_token(&ptr, tok, NULL, sizeof(tok)); ) {
 				unsigned port = atoi(tok);
-				if (port == 0) continue;
+				if (port == 0) {
+					continue;
+				}
 				s = fd_listenset[num_sockets] = open_socket_in(SOCK_STREAM, port, 0, ifip->s_addr, True);
 				if(s == -1)
 					return False;
@@ -259,6 +263,7 @@ static BOOL open_sockets_smbd(BOOL is_daemon, BOOL interactive, const char *smb_
 					return False;
 				}
 				FD_SET(s,&listen_set);
+				maxfd = MAX( maxfd, s);
 
 				num_sockets++;
 				if (num_sockets >= FD_SETSIZE) {
@@ -301,6 +306,7 @@ static BOOL open_sockets_smbd(BOOL is_daemon, BOOL interactive, const char *smb_
 
 			fd_listenset[num_sockets] = s;
 			FD_SET(s,&listen_set);
+			maxfd = MAX( maxfd, s);
 
 			num_sockets++;
 
@@ -335,7 +341,7 @@ static BOOL open_sockets_smbd(BOOL is_daemon, BOOL interactive, const char *smb_
 		memcpy((char *)&lfds, (char *)&listen_set, 
 		       sizeof(listen_set));
 		
-		num = sys_select(FD_SETSIZE,&lfds,NULL,NULL,NULL);
+		num = sys_select(maxfd+1,&lfds,NULL,NULL,NULL);
 		
 		if (num == -1 && errno == EINTR) {
 			if (got_sig_term) {
@@ -409,11 +415,12 @@ static BOOL open_sockets_smbd(BOOL is_daemon, BOOL interactive, const char *smb_
 				   in smbstatus for port 445 connects */
 				set_remote_machine_name(get_peer_addr(smbd_server_fd()), False);
 				
-				/* Reset global variables in util.c so
-				   that client substitutions will be
-				   done correctly in the process.  */
-				reset_globals_after_fork();
+				/* Reset the state of the random
+				 * number generation system, so
+				 * children do not get the same random
+				 * numbers as each other */
 
+				set_need_random_reseed();
 				/* tdb needs special fork handling - remove CLEAR_IF_FIRST flags */
 				if (tdb_reopen_all() == -1) {
 					DEBUG(0,("tdb_reopen_all failed.\n"));
@@ -457,6 +464,39 @@ static BOOL open_sockets_smbd(BOOL is_daemon, BOOL interactive, const char *smb_
 /* NOTREACHED	return True; */
 }
 
+/****************************************************************************
+ Reload printers
+**************************************************************************/
+void reload_printers(void)
+{
+	int snum;
+	int n_services = lp_numservices();
+	int pnum = lp_servicenumber(PRINTERS_NAME);
+	const char *pname;
+
+	pcap_cache_reload();
+
+	/* remove stale printers */
+	for (snum = 0; snum < n_services; snum++) {
+		/* avoid removing PRINTERS_NAME or non-autoloaded printers */
+		if (snum == pnum || !(lp_snum_ok(snum) && lp_print_ok(snum) &&
+		                      lp_autoloaded(snum)))
+			continue;
+
+		pname = lp_printername(snum);
+		if (!pcap_printername_ok(pname)) {
+			DEBUG(3, ("removing stale printer %s\n", pname));
+
+			if (is_printer_published(NULL, snum, NULL))
+				nt_printer_publish(NULL, snum, SPOOL_DS_UNPUBLISH);
+			del_a_printer(pname);
+			lp_killservice(snum);
+		}
+	}
+
+	load_printers();
+}
+
 /****************************************************************************
  Reload the services file.
 **************************************************************************/
@@ -481,10 +521,10 @@ BOOL reload_services(BOOL test)
 		return(True);
 
 	lp_killunused(conn_snum_used);
-	
+
 	ret = lp_load(dyn_CONFIGFILE, False, False, True);
 
-	load_printers();
+	reload_printers();
 
 	/* perhaps the config filename is now set */
 	if (!test)
@@ -494,18 +534,16 @@ BOOL reload_services(BOOL test)
 
 	load_interfaces();
 
-	{
-		if (smbd_server_fd() != -1) {      
-			set_socket_options(smbd_server_fd(),"SO_KEEPALIVE");
-			set_socket_options(smbd_server_fd(), user_socket_options);
-		}
+	if (smbd_server_fd() != -1) {      
+		set_socket_options(smbd_server_fd(),"SO_KEEPALIVE");
+		set_socket_options(smbd_server_fd(), user_socket_options);
 	}
 
 	mangle_reset_cache();
 	reset_stat_cache();
 
 	/* this forces service parameters to be flushed */
-	set_current_service(NULL,True);
+	set_current_service(NULL,0,True);
 
 	return(ret);
 }
@@ -545,6 +583,10 @@ static BOOL dump_core(void)
 
 
 	DEBUG(0,("Dumping core in %s\n", dname));
+	/* Ensure we don't have a signal handler for abort. */
+#ifdef SIGABRT
+	CatchSignal(SIGABRT,SIGNAL_CAST SIG_DFL);
+#endif
 	abort();
 	return(True);
 }
@@ -557,8 +599,6 @@ static BOOL dump_core(void)
 void exit_server(const char *reason)
 {
 	static int firsttime=1;
-	extern char *last_inbuf;
-	extern struct auth_context *negprot_global_auth_context;
 
 	if (!firsttime)
 		exit(0);
@@ -594,6 +634,7 @@ void exit_server(const char *reason)
 
 	if (!reason) {   
 		int oldlevel = DEBUGLEVEL;
+		char *last_inbuf = get_InBuffer();
 		DEBUGLEVEL = 10;
 		DEBUG(0,("Last message was %s\n",smb_fn_name(last_message)));
 		if (last_inbuf)
@@ -706,6 +747,10 @@ void build_options(BOOL screen);
 		log_stdout = True;
 	}
 
+	if (interactive && (DEBUGLEVEL >= 9)) {
+		talloc_enable_leak_report();
+	}
+
 	if (log_stdout && Fork) {
 		DEBUG(0,("ERROR: Can't log to stdout (-S) unless daemon is in foreground (-F) or interactive (-i)\n"));
 		exit(1);
@@ -715,7 +760,7 @@ void build_options(BOOL screen);
 
 	/* we want to re-seed early to prevent time delays causing
            client problems at a later date. (tridge) */
-	generate_random_buffer(NULL, 0, False);
+	generate_random_buffer(NULL, 0);
 
 	/* make absolutely sure we run as root - to handle cases where people
 	   are crazy enough to have it setuid */
@@ -777,6 +822,9 @@ void build_options(BOOL screen);
 
 	init_structs();
 
+	if (!init_guest_info())
+		return -1;
+
 #ifdef WITH_PROFILE
 	if (!profile_setup(False)) {
 		DEBUG(0,("ERROR: failed to setup profiling\n"));
@@ -839,18 +887,26 @@ void build_options(BOOL screen);
 	if (!init_registry())
 		exit(1);
 
+#if 0
+	if (!init_svcctl_db())
+                exit(1);
+#endif
+
 	if (!print_backend_init())
 		exit(1);
 
 	/* Setup the main smbd so that we can get messages. */
+	/* don't worry about general printing messages here */
+
 	claim_connection(NULL,"",0,True,FLAG_MSG_GENERAL|FLAG_MSG_SMBD);
 
-	/* 
-	   DO NOT ENABLE THIS TILL YOU COPE WITH KILLING THESE TASKS AND INETD
-	   THIS *killed* LOTS OF BUILD FARM MACHINES. IT CREATED HUNDREDS OF 
-	   smbd PROCESSES THAT NEVER DIE
-	   start_background_queue(); 
-	*/
+	/* only start the background queue daemon if we are 
+	   running as a daemon -- bad things will happen if
+	   smbd is launched via inetd and we fork a copy of 
+	   ourselves here */
+
+	if ( is_daemon && !interactive )
+		start_background_queue(); 
 
 	if (!open_sockets_smbd(is_daemon, interactive, ports))
 		exit(1);
@@ -895,6 +951,9 @@ void build_options(BOOL screen);
 	if (!init_change_notify())
 		exit(1);
 
+	/* Setup aio signal handler. */
+	initialize_async_io_handler();
+
 	/* re-initialise the timezone */
 	TimeInit();
 
@@ -904,6 +963,7 @@ void build_options(BOOL screen);
 	smbd_process();
 	
 	namecache_shutdown();
+
 	exit_server("normal exit");
 	return(0);
 }
