@@ -47,10 +47,15 @@ void stat_cache_add( const char *full_orig_name, const char *orig_translated_pat
 	TDB_DATA data_val;
 	char *original_path;
 	size_t original_path_length;
+	size_t sc_size = lp_max_stat_cache_size();
 
 	if (!lp_stat_cache())
 		return;
 
+	if (sc_size && (tdb_stat_cache->map_size > sc_size*1024)) {
+		reset_stat_cache();
+	}
+
 	ZERO_STRUCT(data_val);
 
 	/*
@@ -59,7 +64,7 @@ void stat_cache_add( const char *full_orig_name, const char *orig_translated_pat
 
 	if((*full_orig_name == '\0') || (full_orig_name[0] == '.' && 
 				((full_orig_name[1] == '\0') ||
-				 (full_orig_name[1] == '.' && full_orig_name[1] == '\0'))))
+				 (full_orig_name[1] == '.' && full_orig_name[2] == '\0'))))
 		return;
 
 	/*
