@@ -62,6 +62,9 @@ static struct vfs_ops default_vfs = {
 	
 		vfswrap_opendir,
 		vfswrap_readdir,
+		vfswrap_seekdir,
+		vfswrap_telldir,
+		vfswrap_rewinddir,
 		vfswrap_mkdir,
 		vfswrap_rmdir,
 		vfswrap_closedir,
@@ -142,7 +145,16 @@ static struct vfs_ops default_vfs = {
 		vfswrap_fremovexattr,
 		vfswrap_setxattr,
 		vfswrap_lsetxattr,
-		vfswrap_fsetxattr
+		vfswrap_fsetxattr,
+
+		/* AIO operations. */
+		vfswrap_aio_read,
+		vfswrap_aio_write,
+		vfswrap_aio_return,
+		vfswrap_aio_cancel,
+		vfswrap_aio_error,
+		vfswrap_aio_fsync,
+		vfswrap_aio_suspend
 	}
 };
 
@@ -224,13 +236,15 @@ BOOL vfs_init_custom(connection_struct *conn, const char *vfs_object)
 		return False;
 	}
 
-	if(!backends) static_init_vfs;
+	if(!backends) {
+		static_init_vfs;
+	}
 
 	DEBUG(3, ("Initialising custom vfs hooks from [%s]\n", vfs_object));
 
 	module_name = smb_xstrdup(vfs_object);
 
-	p = strchr(module_name, ':');
+	p = strchr_m(module_name, ':');
 
 	if (p) {
 		*p = 0;
@@ -272,20 +286,20 @@ BOOL vfs_init_custom(connection_struct *conn, const char *vfs_object)
 	DLIST_ADD(conn->vfs_handles, handle);
 
  	for(i=0; ops[i].op != NULL; i++) {
- 	  DEBUG(5, ("Checking operation #%d (type %d, layer %d)\n", i, ops[i].type, ops[i].layer));
- 	  if(ops[i].layer == SMB_VFS_LAYER_OPAQUE) {
- 	    /* Check whether this operation was already made opaque by different module */
- 	    if(((void**)&conn->vfs_opaque.ops)[ops[i].type] == ((void**)&default_vfs.ops)[ops[i].type]) {
- 	      /* No, it isn't overloaded yet. Overload. */
- 	      DEBUGADD(5, ("Making operation type %d opaque [module %s]\n", ops[i].type, vfs_object));
- 	      ((void**)&conn->vfs_opaque.ops)[ops[i].type] = ops[i].op;
- 	      ((vfs_handle_struct **)&conn->vfs_opaque.handles)[ops[i].type] = handle;
- 	    }
- 	  }
- 	  /* Change current VFS disposition*/
- 	  DEBUGADD(5, ("Accepting operation type %d from module %s\n", ops[i].type, vfs_object));
- 	  ((void**)&conn->vfs.ops)[ops[i].type] = ops[i].op;
- 	  ((vfs_handle_struct **)&conn->vfs.handles)[ops[i].type] = handle;
+		DEBUG(5, ("Checking operation #%d (type %d, layer %d)\n", i, ops[i].type, ops[i].layer));
+		if(ops[i].layer == SMB_VFS_LAYER_OPAQUE) {
+			/* Check whether this operation was already made opaque by different module */
+			if(((void**)&conn->vfs_opaque.ops)[ops[i].type] == ((void**)&default_vfs.ops)[ops[i].type]) {
+				/* No, it isn't overloaded yet. Overload. */
+				DEBUGADD(5, ("Making operation type %d opaque [module %s]\n", ops[i].type, vfs_object));
+				((void**)&conn->vfs_opaque.ops)[ops[i].type] = ops[i].op;
+				((vfs_handle_struct **)&conn->vfs_opaque.handles)[ops[i].type] = handle;
+			}
+		}
+		/* Change current VFS disposition*/
+		DEBUGADD(5, ("Accepting operation type %d from module %s\n", ops[i].type, vfs_object));
+		((void**)&conn->vfs.ops)[ops[i].type] = ops[i].op;
+		((vfs_handle_struct **)&conn->vfs.handles)[ops[i].type] = handle;
 	}
 
 	SAFE_FREE(module_name);
@@ -416,7 +430,7 @@ ssize_t vfs_read_data(files_struct *fsp, char *buf, size_t byte_count)
 
 	while (total < byte_count)
 	{
-		ssize_t ret = SMB_VFS_READ(fsp, fsp->fd, buf + total,
+		ssize_t ret = SMB_VFS_READ(fsp, fsp->fh->fd, buf + total,
 					byte_count - total);
 
 		if (ret == 0) return total;
@@ -438,7 +452,7 @@ ssize_t vfs_pread_data(files_struct *fsp, char *buf,
 
 	while (total < byte_count)
 	{
-		ssize_t ret = SMB_VFS_PREAD(fsp, fsp->fd, buf + total,
+		ssize_t ret = SMB_VFS_PREAD(fsp, fsp->fh->fd, buf + total,
 					byte_count - total, offset + total);
 
 		if (ret == 0) return total;
@@ -463,7 +477,7 @@ ssize_t vfs_write_data(files_struct *fsp,const char *buffer,size_t N)
 	ssize_t ret;
 
 	while (total < N) {
-		ret = SMB_VFS_WRITE(fsp,fsp->fd,buffer + total,N - total);
+		ret = SMB_VFS_WRITE(fsp,fsp->fh->fd,buffer + total,N - total);
 
 		if (ret == -1)
 			return -1;
@@ -482,7 +496,7 @@ ssize_t vfs_pwrite_data(files_struct *fsp,const char *buffer,
 	ssize_t ret;
 
 	while (total < N) {
-		ret = SMB_VFS_PWRITE(fsp, fsp->fd, buffer + total,
+		ret = SMB_VFS_PWRITE(fsp, fsp->fh->fd, buffer + total,
                                 N - total, offset + total);
 
 		if (ret == -1)
@@ -521,7 +535,7 @@ int vfs_allocate_file_space(files_struct *fsp, SMB_BIG_UINT len)
 		return -1;
 	}
 
-	ret = SMB_VFS_FSTAT(fsp,fsp->fd,&st);
+	ret = SMB_VFS_FSTAT(fsp,fsp->fh->fd,&st);
 	if (ret == -1)
 		return ret;
 
@@ -535,7 +549,7 @@ int vfs_allocate_file_space(files_struct *fsp, SMB_BIG_UINT len)
 				fsp->fsp_name, (double)st.st_size ));
 
 		flush_write_cache(fsp, SIZECHANGE_FLUSH);
-		if ((ret = SMB_VFS_FTRUNCATE(fsp, fsp->fd, (SMB_OFF_T)len)) != -1) {
+		if ((ret = SMB_VFS_FTRUNCATE(fsp, fsp->fh->fd, (SMB_OFF_T)len)) != -1) {
 			set_filelen_write_cache(fsp, len);
 		}
 		return ret;
@@ -549,6 +563,9 @@ int vfs_allocate_file_space(files_struct *fsp, SMB_BIG_UINT len)
 	len -= st.st_size;
 	len /= 1024; /* Len is now number of 1k blocks needed. */
 	space_avail = SMB_VFS_DISK_FREE(conn,fsp->fsp_name,False,&bsize,&dfree,&dsize);
+	if (space_avail == (SMB_BIG_UINT)-1) {
+		return -1;
+	}
 
 	DEBUG(10,("vfs_allocate_file_space: file %s, grow. Current size %.0f, needed blocks = %.0f, space avail = %.0f\n",
 			fsp->fsp_name, (double)st.st_size, (double)len, (double)space_avail ));
@@ -574,12 +591,78 @@ int vfs_set_filelen(files_struct *fsp, SMB_OFF_T len)
 	release_level_2_oplocks_on_change(fsp);
 	DEBUG(10,("vfs_set_filelen: ftruncate %s to len %.0f\n", fsp->fsp_name, (double)len));
 	flush_write_cache(fsp, SIZECHANGE_FLUSH);
-	if ((ret = SMB_VFS_FTRUNCATE(fsp, fsp->fd, len)) != -1)
+	if ((ret = SMB_VFS_FTRUNCATE(fsp, fsp->fh->fd, len)) != -1)
 		set_filelen_write_cache(fsp, len);
 
 	return ret;
 }
 
+/****************************************************************************
+ A vfs fill sparse call.
+ Writes zeros from the end of file to len, if len is greater than EOF.
+ Used only by strict_sync.
+ Returns 0 on success, -1 on failure.
+****************************************************************************/
+
+static char *sparse_buf;
+#define SPARSE_BUF_WRITE_SIZE (32*1024)
+
+int vfs_fill_sparse(files_struct *fsp, SMB_OFF_T len)
+{
+	int ret;
+	SMB_STRUCT_STAT st;
+	SMB_OFF_T offset;
+	size_t total;
+	size_t num_to_write;
+	ssize_t pwrite_ret;
+
+	release_level_2_oplocks_on_change(fsp);
+	ret = SMB_VFS_FSTAT(fsp,fsp->fh->fd,&st);
+	if (ret == -1) {
+		return ret;
+	}
+
+	if (len <= st.st_size) {
+		return 0;
+	}
+
+	DEBUG(10,("vfs_fill_sparse: write zeros in file %s from len %.0f to len %.0f (%.0f bytes)\n",
+		fsp->fsp_name, (double)st.st_size, (double)len, (double)(len - st.st_size)));
+
+	flush_write_cache(fsp, SIZECHANGE_FLUSH);
+
+	if (!sparse_buf) {
+		sparse_buf = SMB_CALLOC_ARRAY(char, SPARSE_BUF_WRITE_SIZE);
+		if (!sparse_buf) {
+			errno = ENOMEM;
+			return -1;
+		}
+	}
+
+	offset = st.st_size;
+	num_to_write = len - st.st_size;
+	total = 0;
+
+	while (total < num_to_write) {
+		size_t curr_write_size = MIN(SPARSE_BUF_WRITE_SIZE, (num_to_write - total));
+
+		pwrite_ret = SMB_VFS_PWRITE(fsp, fsp->fh->fd, sparse_buf, curr_write_size, offset + total);
+		if (pwrite_ret == -1) {
+			DEBUG(10,("vfs_fill_sparse: SMB_VFS_PWRITE for file %s failed with error %s\n",
+				fsp->fsp_name, strerror(errno) ));
+			return -1;
+		}
+		if (pwrite_ret == 0) {
+			return 0;
+		}
+
+		total += pwrite_ret;
+	}
+
+	set_filelen_write_cache(fsp, len);
+	return 0;
+}
+
 /****************************************************************************
  Transfer some data (n bytes) between two file_struct's.
 ****************************************************************************/
@@ -602,7 +685,7 @@ SMB_OFF_T vfs_transfer_file(files_struct *in, files_struct *out, SMB_OFF_T n)
 	in_fsp = in;
 	out_fsp = out;
 
-	return transfer_file_internal(in_fsp->fd, out_fsp->fd, n, read_fn, write_fn);
+	return transfer_file_internal(in_fsp->fh->fd, out_fsp->fh->fd, n, read_fn, write_fn);
 }
 
 /*******************************************************************
@@ -611,13 +694,13 @@ SMB_OFF_T vfs_transfer_file(files_struct *in, files_struct *out, SMB_OFF_T n)
 
 char *vfs_readdirname(connection_struct *conn, void *p)
 {
-	struct dirent *ptr= NULL;
+	SMB_STRUCT_DIRENT *ptr= NULL;
 	char *dname;
 
 	if (!p)
 		return(NULL);
 
-	ptr = (struct dirent *)SMB_VFS_READDIR(conn,p);
+	ptr = SMB_VFS_READDIR(conn,p);
 	if (!ptr)
 		return(NULL);
 
@@ -725,7 +808,9 @@ char *vfs_GetWd(connection_struct *conn, char *path)
 		in trouble :-) */
 
 	if (SMB_VFS_STAT(conn, ".",&st) == -1) {
-		DEBUG(0,("Very strange, couldn't stat \".\" path=%s\n", path));
+		/* Known to fail for root: the directory may be
+		 * NFS-mounted and exported with root_squash (so has no root access). */
+		DEBUG(1,("vfs_GetWd: couldn't stat \".\" path=%s error %s (NFS problem ?)\n", path, strerror(errno) ));
 		return(SMB_VFS_GETWD(conn,path));
 	}
 
@@ -886,7 +971,7 @@ BOOL reduce_name(connection_struct *conn, const pstring fname)
 #ifdef PATH_MAX
 				safe_strcpy(resolved_name_buf, tmp_fname, PATH_MAX);
 #else
-				pstrcpy(pstring resolved_name_buf, tmp_fname);
+				pstrcpy(resolved_name_buf, tmp_fname);
 #endif
 				resolved_name = resolved_name_buf;
 #endif
@@ -894,7 +979,8 @@ BOOL reduce_name(connection_struct *conn, const pstring fname)
 			}
 			default:
 				DEBUG(1,("reduce_name: couldn't get realpath for %s\n", fname));
-				errno = saved_errno;
+				/* Don't restore the saved errno. We need to return the error that
+				   realpath caused here as it was not one of the cases we handle. JRA. */
 				return False;
 		}
 	}
@@ -936,7 +1022,7 @@ BOOL reduce_name(connection_struct *conn, const pstring fname)
         }
 #endif
 
-	DEBUG(3,("reduce_name: %s reduced to %s\n", fname, p));
+	DEBUG(3,("reduce_name: %s reduced to %s\n", fname, resolved_name));
 	if (free_resolved_name)
 		SAFE_FREE(resolved_name);
 	errno = saved_errno;
