@@ -87,6 +87,7 @@ const char *opt_destination = NULL;
 BOOL opt_have_ip = False;
 struct in_addr opt_dest_ip;
 
+extern struct in_addr loopback_ip;
 extern BOOL AllowDebugChange;
 
 uint32 get_sec_channel_type(const char *param) 
@@ -218,7 +219,7 @@ NTSTATUS connect_to_ipc_anonymous(struct cli_state **c,
  *
  * @return Normal NTSTATUS return.
  **/
-NTSTATUS connect_pipe(struct cli_state **cli_dst, int pipe_num, BOOL *got_pipe)
+NTSTATUS connect_dst_pipe(struct cli_state **cli_dst, int pipe_num, BOOL *got_pipe)
 {
 	NTSTATUS nt_status;
 	char *server_name = SMB_STRDUP("127.0.0.1");
@@ -321,7 +322,6 @@ BOOL net_find_server(unsigned flags, struct in_addr *server_ip, char **server_na
 		}
 		*server_name = SMB_STRDUP(inet_ntoa(opt_dest_ip));
 	} else if (!(flags & NET_FLAGS_LOCALHOST_DEFAULT_INSANE)) {
-		extern struct in_addr loopback_ip;
 		*server_ip = loopback_ip;
 		*server_name = SMB_STRDUP("127.0.0.1");
 	}
@@ -552,20 +552,22 @@ static int net_getdomainsid(int argc, const char **argv)
 
 #ifdef WITH_FAKE_KASERVER
 
-int net_afskey_usage(int argc, const char **argv)
+int net_help_afs(int argc, const char **argv)
 {
-	d_printf("  net afskey filename\n"
+	d_printf("  net afs key filename\n"
 		 "\tImports a OpenAFS KeyFile into our secrets.tdb\n\n");
+	d_printf("  net afs impersonate <user> <cell>\n"
+		 "\tCreates a token for user@cell\n\n");
 	return -1;
 }
 
-static int net_afskey(int argc, const char **argv)
+static int net_afs_key(int argc, const char **argv)
 {
 	int fd;
 	struct afs_keyfile keyfile;
 
 	if (argc != 2) {
-		d_printf("usage: 'net afskey <keyfile> cell'\n");
+		d_printf("usage: 'net afs key <keyfile> cell'\n");
 		return -1;
 	}
 
@@ -592,64 +594,76 @@ static int net_afskey(int argc, const char **argv)
 	return 0;
 }
 
+static int net_afs_impersonate(int argc, const char **argv)
+{
+	char *token;
+
+	if (argc != 2) {
+		fprintf(stderr, "Usage: net afs impersonate <user> <cell>\n");
+	        exit(1);
+	}
+
+	token = afs_createtoken_str(argv[0], argv[1]);
+
+	if (token == NULL) {
+		fprintf(stderr, "Could not create token\n");
+	        exit(1);
+	}
+
+	if (!afs_settoken_str(token)) {
+		fprintf(stderr, "Could not set token into kernel\n");
+	        exit(1);
+	}
+
+	printf("Success: %s@%s\n", argv[0], argv[1]);
+	return 0;
+}
+
+static int net_afs(int argc, const char **argv)
+{
+	struct functable func[] = {
+		{"key", net_afs_key},
+		{"impersonate", net_afs_impersonate},
+		{"help", net_help_afs},
+		{NULL, NULL}
+	};
+	return net_run_function(argc, argv, func, net_help_afs);
+}
+
 #endif /* WITH_FAKE_KASERVER */
 
+static BOOL search_maxrid(struct pdb_search *search, const char *type,
+			  uint32 *max_rid)
+{
+	struct samr_displayentry *entries;
+	uint32 i, num_entries;
+
+	if (search == NULL) {
+		d_printf("get_maxrid: Could not search %s\n", type);
+		return False;
+	}
+
+	num_entries = pdb_search_entries(search, 0, 0xffffffff, &entries);
+	for (i=0; i<num_entries; i++)
+		*max_rid = MAX(*max_rid, entries[i].rid);
+	pdb_search_destroy(search);
+	return True;
+}
+
 static uint32 get_maxrid(void)
 {
-	SAM_ACCOUNT *pwd = NULL;
 	uint32 max_rid = 0;
-	GROUP_MAP *map = NULL;
-	int num_entries = 0;
-	int i;
 
-	if (!pdb_setsampwent(False)) {
-		DEBUG(0, ("load_sampwd_entries: Unable to open passdb.\n"));
+	if (!search_maxrid(pdb_search_users(0), "users", &max_rid))
 		return 0;
-	}
 
-	for (; (NT_STATUS_IS_OK(pdb_init_sam(&pwd))) 
-		     && pdb_getsampwent(pwd) == True; pwd=NULL) {
-		uint32 rid;
-
-		if (!sid_peek_rid(pdb_get_user_sid(pwd), &rid)) {
-			DEBUG(0, ("can't get RID for user '%s'\n",
-				  pdb_get_username(pwd)));
-			pdb_free_sam(&pwd);
-			continue;
-		}
-
-		if (rid > max_rid)
-			max_rid = rid;
-
-		DEBUG(1,("%d is user '%s'\n", rid, pdb_get_username(pwd)));
-		pdb_free_sam(&pwd);
-	}
-
-	pdb_endsampwent();
-	pdb_free_sam(&pwd);
-
-	if (!pdb_enum_group_mapping(SID_NAME_UNKNOWN, &map, &num_entries,
-				    ENUM_ONLY_MAPPED))
-		return max_rid;
-
-	for (i = 0; i < num_entries; i++) {
-		uint32 rid;
-
-		if (!sid_peek_check_rid(get_global_sam_sid(), &map[i].sid,
-					&rid)) {
-			DEBUG(3, ("skipping map for group '%s', SID %s\n",
-				  map[i].nt_name,
-				  sid_string_static(&map[i].sid)));
-			continue;
-		}
-		DEBUG(1,("%d is group '%s'\n", rid, map[i].nt_name));
-
-		if (rid > max_rid)
-			max_rid = rid;
-	}
-
-	SAFE_FREE(map);
+	if (!search_maxrid(pdb_search_groups(), "groups", &max_rid))
+		return 0;
 
+	if (!search_maxrid(pdb_search_aliases(get_global_sam_sid()),
+			   "aliases", &max_rid))
+		return 0;
+	
 	return max_rid;
 }
 
@@ -707,7 +721,7 @@ static struct functable net_func[] = {
 	{"STATUS", net_status},
 	{"USERSIDLIST", net_usersidlist},
 #ifdef WITH_FAKE_KASERVER
-	{"AFSKEY", net_afskey},
+	{"AFS", net_afs},
 #endif
 
 	{"HELP", net_help},
