@@ -234,7 +234,7 @@ static void display_print_driver_3(DRIVER_INFO_3 *i1)
 }
 
 
-static void display_reg_value(pstring subkey, REGISTRY_VALUE value)
+static void display_reg_value(const char *subkey, REGISTRY_VALUE value)
 {
 	pstring text;
 
@@ -292,11 +292,10 @@ static void display_reg_value(pstring subkey, REGISTRY_VALUE value)
  *
  * @return Normal NTSTATUS return.
  **/ 
-static NTSTATUS 
-net_copy_fileattr(TALLOC_CTX *mem_ctx,
+NTSTATUS net_copy_fileattr(TALLOC_CTX *mem_ctx,
 		  struct cli_state *cli_share_src,
 		  struct cli_state *cli_share_dst, 
-		  char *src_name, char *dst_name,
+		  const char *src_name, const char *dst_name,
 		  BOOL copy_acls, BOOL copy_attrs,
 		  BOOL copy_timestamps, BOOL is_file)
 {
@@ -305,7 +304,7 @@ net_copy_fileattr(TALLOC_CTX *mem_ctx,
 	int fnum_dst = 0;
 	SEC_DESC *sd = NULL;
 	uint16 attr;
-	time_t atime, ctime, mtime;
+	time_t f_atime, f_ctime, f_mtime;
 
 
 	if (!copy_timestamps && !copy_acls && !copy_attrs)
@@ -346,7 +345,7 @@ net_copy_fileattr(TALLOC_CTX *mem_ctx,
 
 		/* get file attributes */
 		if (!cli_getattrE(cli_share_src, fnum_src, &attr, NULL, 
-				 &ctime, &atime, &mtime)) {
+				 &f_ctime, &f_atime, &f_mtime)) {
 			DEBUG(0,("failed to get file-attrs: %s\n", 
 				cli_errstr(cli_share_src)));
 			nt_status = cli_nt_error(cli_share_src);
@@ -368,7 +367,7 @@ net_copy_fileattr(TALLOC_CTX *mem_ctx,
 	if (copy_timestamps) {
 
 		/* set timestamps */
-		if (!cli_setattrE(cli_share_dst, fnum_dst, ctime, atime, mtime)) {
+		if (!cli_setattrE(cli_share_dst, fnum_dst, f_ctime, f_atime, f_mtime)) {
 			DEBUG(0,("failed to set file-attrs (timestamps): %s\n",
 				cli_errstr(cli_share_dst)));
 			nt_status = cli_nt_error(cli_share_dst);
@@ -449,7 +448,7 @@ out:
 NTSTATUS net_copy_file(TALLOC_CTX *mem_ctx,
 		       struct cli_state *cli_share_src,
 		       struct cli_state *cli_share_dst, 
-		       char *src_name, char *dst_name,
+		       const char *src_name, const char *dst_name,
 		       BOOL copy_acls, BOOL copy_attrs,
 		       BOOL copy_timestamps, BOOL is_file)
 {
@@ -466,7 +465,7 @@ NTSTATUS net_copy_file(TALLOC_CTX *mem_ctx,
 	if (!src_name || !dst_name)
 		goto out;
 
-	if (cli_share_dst == NULL || cli_share_dst == NULL)
+	if (cli_share_src == NULL || cli_share_dst == NULL)
 		goto out; 
 		
 
@@ -479,7 +478,8 @@ NTSTATUS net_copy_file(TALLOC_CTX *mem_ctx,
 		fnum_src = cli_nt_create(cli_share_src, src_name, READ_CONTROL_ACCESS);
 
 	if (fnum_src == -1) {
-		DEBUGADD(0,("cannot open file %s on originating server %s\n", 
+		DEBUGADD(0,("cannot open %s %s on originating server %s\n",
+			is_file ? "file":"dir",
 			src_name, cli_errstr(cli_share_src)));
 		nt_status = cli_nt_error(cli_share_src);
 		goto out;
@@ -780,8 +780,10 @@ copy_print_driver_3(TALLOC_CTX *mem_ctx,
 		return nt_status;
 
 	while (valid) {
+		
 		rpcstr_pull(dependentfiles, i1->dependentfiles+length, sizeof(dependentfiles), -1, STR_TERMINATE);
-		length+=strlen(dependentfiles)+1;
+		length += strlen(dependentfiles)+1;
+		
 		if (strlen(dependentfiles) > 0) {
 
 			nt_status = net_copy_driverfile(mem_ctx, 
@@ -817,19 +819,11 @@ net_spoolss_enum_printers(struct cli_state *cli, TALLOC_CTX *mem_ctx,
 {
 
 	WERROR result;
-	uint32 needed;
 
 	/* enum printers */
-	result = cli_spoolss_enum_printers(
-		cli, mem_ctx, 0, &needed, name, flags,
+	result = cli_spoolss_enum_printers(cli, mem_ctx, name, flags,
 		level, num_printers, ctr);
 
-	if (W_ERROR_V(result) == W_ERROR_V(WERR_INSUFFICIENT_BUFFER)) 
-		result = cli_spoolss_enum_printers(
-			cli, mem_ctx, needed, NULL, name, flags,
-			level, num_printers, ctr);
-	
-
 	if (!W_ERROR_IS_OK(result)) {
 		printf("cannot enum printers: %s\n", dos_errstr(result));
 		return False;
@@ -887,16 +881,10 @@ net_spoolss_getprinter(struct cli_state *cli, TALLOC_CTX *mem_ctx,
 		       PRINTER_INFO_CTR *ctr)
 {
 	WERROR result;
-	uint32 needed;
 
 	/* getprinter call */
-	result = cli_spoolss_getprinter(cli, 
-		mem_ctx, 0, &needed, hnd, level, ctr);
+	result = cli_spoolss_getprinter(cli, mem_ctx, hnd, level, ctr);
 
-	if (W_ERROR_V(result) == W_ERROR_V(WERR_INSUFFICIENT_BUFFER)) 
-		result = cli_spoolss_getprinter(cli, 
-			mem_ctx, needed, NULL, hnd, level, ctr);
-			
 	if (!W_ERROR_IS_OK(result)) {
 		printf("cannot get printer-info: %s\n", dos_errstr(result));
 		return False;
@@ -949,17 +937,10 @@ net_spoolss_enumprinterkey(struct cli_state *cli, TALLOC_CTX *mem_ctx,
 			   uint16 **keylist)
 {
 	WERROR result;
-	uint32 needed, len;
 
 	/* enumprinterkey call */
-	result = cli_spoolss_enumprinterkey(
-		cli, mem_ctx, 0, &needed, hnd, keyname, NULL, NULL);
+	result = cli_spoolss_enumprinterkey(cli, mem_ctx, hnd, keyname, keylist, NULL);
 		
-	if (W_ERROR_V(result) == W_ERROR_V(WERR_MORE_DATA)) 
-		result = cli_spoolss_enumprinterkey(
-			cli, mem_ctx, needed, NULL, hnd, keyname, keylist,
-			&len);
-
 	if (!W_ERROR_IS_OK(result)) {
 		printf("enumprinterkey failed: %s\n", dos_errstr(result));
 		return False;
@@ -976,16 +957,10 @@ net_spoolss_enumprinterdataex(struct cli_state *cli, TALLOC_CTX *mem_ctx,
 			      REGVAL_CTR *ctr) 
 {
 	WERROR result;
-	uint32 needed;
 
 	/* enumprinterdataex call */
-	result = cli_spoolss_enumprinterdataex(
-		cli, mem_ctx, 0, &needed, hnd, keyname, NULL);
+	result = cli_spoolss_enumprinterdataex(cli, mem_ctx, hnd, keyname, ctr);
 			
-	if (W_ERROR_V(result) == W_ERROR_V(WERR_MORE_DATA)) 
-		result = cli_spoolss_enumprinterdataex(
-			cli, mem_ctx, needed, NULL, hnd, keyname, ctr);
-					
 	if (!W_ERROR_IS_OK(result)) {
 		printf("enumprinterdataex failed: %s\n", dos_errstr(result));
 		return False;
@@ -1022,16 +997,9 @@ net_spoolss_enumforms(struct cli_state *cli, TALLOC_CTX *mem_ctx,
 										       
 {
 	WERROR result;
-	uint32 needed;
 
 	/* enumforms call */
-	result = cli_spoolss_enumforms(
-			cli, mem_ctx, 0, &needed, hnd, level, num_forms, forms);
-
-	if (W_ERROR_V(result) == W_ERROR_V(WERR_INSUFFICIENT_BUFFER)) 
-		result = cli_spoolss_enumforms(
-				cli, mem_ctx, needed, NULL, hnd, level,
-				num_forms, forms);
+	result = cli_spoolss_enumforms(cli, mem_ctx, hnd, level, num_forms, forms);
 
 	if (!W_ERROR_IS_OK(result)) {
 		printf("could not enum forms: %s\n", dos_errstr(result));
@@ -1049,18 +1017,12 @@ net_spoolss_enumprinterdrivers (struct cli_state *cli, TALLOC_CTX *mem_ctx,
 				PRINTER_DRIVER_CTR *ctr)
 {
 	WERROR result;
-	uint32 needed;
 
 	/* enumprinterdrivers call */
 	result = cli_spoolss_enumprinterdrivers(
-			cli, mem_ctx, 0, &needed, level,
+			cli, mem_ctx, level,
 			env, num_drivers, ctr);
 
-	if (W_ERROR_V(result) == W_ERROR_V(WERR_INSUFFICIENT_BUFFER))
-		result = cli_spoolss_enumprinterdrivers(
-				cli, mem_ctx, needed, NULL, level,
-				env, num_drivers, ctr);
-	
 	if (!W_ERROR_IS_OK(result)) {
 		printf("cannot enum drivers: %s\n", dos_errstr(result));
 		return False;
@@ -1078,18 +1040,12 @@ net_spoolss_getprinterdriver(struct cli_state *cli,
 			     PRINTER_DRIVER_CTR *ctr)
 {
 	WERROR result;
-	uint32 needed;
 	
 	/* getprinterdriver call */
 	result = cli_spoolss_getprinterdriver(
-			cli, mem_ctx, 0, &needed, hnd, level,
+			cli, mem_ctx, hnd, level,
 			env, version, ctr);
 
-	if (W_ERROR_V(result) == W_ERROR_V(WERR_INSUFFICIENT_BUFFER))
-		result = cli_spoolss_getprinterdriver(
-				cli, mem_ctx, needed, NULL, hnd, level,
-				env, version, ctr);
-
 	if (!W_ERROR_IS_OK(result)) {
 		DEBUG(1,("cannot get driver (for architecture: %s): %s\n", 
 			env, dos_errstr(result)));
@@ -1137,7 +1093,6 @@ get_printer_info(struct cli_state *cli, TALLOC_CTX *mem_ctx,
 		 uint32 *num_printers, PRINTER_INFO_CTR *ctr)
 {
 
-	char *sharename;
 	POLICY_HND hnd;
 
 	/* no arguments given, enumerate all printers */
@@ -1153,9 +1108,7 @@ get_printer_info(struct cli_state *cli, TALLOC_CTX *mem_ctx,
 
 
 	/* argument given, get a single printer by name */
-	sharename = SMB_STRDUP(argv[0]);
-
-	if (!net_spoolss_open_printer_ex(cli, mem_ctx, sharename,
+	if (!net_spoolss_open_printer_ex(cli, mem_ctx, argv[0],
 			MAXIMUM_ALLOWED_ACCESS,	cli->user_name, &hnd)) 
 		return False;
 
@@ -1254,7 +1207,7 @@ NTSTATUS rpc_printer_driver_list_internals(const DOM_SID *domain_sid, const char
 
         for (i=0; archi_table[i].long_archi!=NULL; i++) {
 
-		int num_drivers;
+		uint32 num_drivers;
 
 		/* enum remote drivers */
 		if (!net_spoolss_enumprinterdrivers(cli, mem_ctx, level,
@@ -1312,7 +1265,7 @@ static NTSTATUS rpc_printer_publish_internals_args(struct cli_state *cli, TALLOC
 	POLICY_HND hnd;
 	BOOL got_hnd = False;
 	WERROR result;
-	char *action_str;
+	const char *action_str;
 
 	if (!get_printer_info(cli, mem_ctx, 2, argc, argv, &num_printers, &ctr))
 		return nt_status;
@@ -1348,6 +1301,7 @@ static NTSTATUS rpc_printer_publish_internals_args(struct cli_state *cli, TALLOC
 			action_str = "unpublished";
 			break;
 		default:
+			action_str = "unknown action";
 			printf("unkown action: %d\n", action);
 			break;
 		}
@@ -1355,7 +1309,7 @@ static NTSTATUS rpc_printer_publish_internals_args(struct cli_state *cli, TALLOC
 		ctr_pub.printers_7->action = action;
 
 		result = cli_spoolss_setprinter(cli, mem_ctx, &hnd, level, &ctr_pub, 0);
-		if (!W_ERROR_IS_OK(result) && (W_ERROR_V(result) =! W_ERROR_V(WERR_IO_PENDING))) {
+		if (!W_ERROR_IS_OK(result) && (W_ERROR_V(result) != W_ERROR_V(WERR_IO_PENDING))) {
 			printf("cannot set printer-info: %s\n", dos_errstr(result));
 			goto done;
 		}
@@ -1394,7 +1348,7 @@ NTSTATUS rpc_printer_publish_update_internals(const DOM_SID *domain_sid, const c
 }
 
 /** 
- * List print-queues w.r.t. thei publishing
+ * List print-queues w.r.t. their publishing state
  *
  * All parameters are provided by the run_rpc_command function, except for
  * argc, argv which are passed through. 
@@ -1516,7 +1470,7 @@ NTSTATUS rpc_printer_migrate_security_internals(const DOM_SID *domain_sid, const
 	DEBUG(3,("copying printer ACLs\n"));
 
 	/* connect destination PI_SPOOLSS */
-	nt_status = connect_pipe(&cli_dst, PI_SPOOLSS, &got_dst_spoolss_pipe);
+	nt_status = connect_dst_pipe(&cli_dst, PI_SPOOLSS, &got_dst_spoolss_pipe);
 	if (!NT_STATUS_IS_OK(nt_status))
 		return nt_status;
 
@@ -1665,7 +1619,7 @@ NTSTATUS rpc_printer_migrate_forms_internals(const DOM_SID *domain_sid, const ch
 	DEBUG(3,("copying forms\n"));
 	
 	/* connect destination PI_SPOOLSS */
-	nt_status = connect_pipe(&cli_dst, PI_SPOOLSS, &got_dst_spoolss_pipe);
+	nt_status = connect_dst_pipe(&cli_dst, PI_SPOOLSS, &got_dst_spoolss_pipe);
 	if (!NT_STATUS_IS_OK(nt_status))
 		return nt_status;
 	
@@ -1845,7 +1799,7 @@ NTSTATUS rpc_printer_migrate_drivers_internals(const DOM_SID *domain_sid, const
 
 	DEBUG(3,("copying printer-drivers\n"));
 
-	nt_status = connect_pipe(&cli_dst, PI_SPOOLSS, &got_dst_spoolss_pipe);
+	nt_status = connect_dst_pipe(&cli_dst, PI_SPOOLSS, &got_dst_spoolss_pipe);
 	if (!NT_STATUS_IS_OK(nt_status))
 		return nt_status;
 	
@@ -1854,7 +1808,7 @@ NTSTATUS rpc_printer_migrate_drivers_internals(const DOM_SID *domain_sid, const
 	nt_status = connect_to_service(&cli_share_src, &cli->dest_ip, 
 			cli->desthost, "print$", "A:");
 	if (!NT_STATUS_IS_OK(nt_status)) 
-		return nt_status;
+		goto done;
 
 	got_src_driver_share = True;
 
@@ -1874,7 +1828,7 @@ NTSTATUS rpc_printer_migrate_drivers_internals(const DOM_SID *domain_sid, const
 		goto done;
 	}
 
-	if (!num_printers) {
+	if (num_printers == 0) {
 		printf ("no printers found on server.\n");
 		nt_status = NT_STATUS_OK;
 		goto done;
@@ -1959,6 +1913,12 @@ NTSTATUS rpc_printer_migrate_drivers_internals(const DOM_SID *domain_sid, const
 
 		}
 
+		if (strlen(drivername) == 0) {
+			DEBUGADD(1,("Did not get driver for printer %s\n",
+				    printername));
+			goto done;
+		}
+
 		/* setdriver dst */
 		init_unistr(&info_ctr_dst.printers_2->drivername, drivername);
 		
@@ -2044,13 +2004,13 @@ NTSTATUS rpc_printer_migrate_printers_internals(const DOM_SID *domain_sid, const
 	DEBUG(3,("copying printers\n"));
 
 	/* connect destination PI_SPOOLSS */
-	nt_status = connect_pipe(&cli_dst, PI_SPOOLSS, &got_dst_spoolss_pipe);
+	nt_status = connect_dst_pipe(&cli_dst, PI_SPOOLSS, &got_dst_spoolss_pipe);
 	if (!NT_STATUS_IS_OK(nt_status))
 		return nt_status;
 
 
 	/* enum printers */
-	if (!get_printer_info(cli, mem_ctx, 2, argc, argv, &num_printers, &ctr_enum)) {
+	if (!get_printer_info(cli, mem_ctx, level, argc, argv, &num_printers, &ctr_enum)) {
 		nt_status = NT_STATUS_UNSUCCESSFUL;
 		goto done;
 	}
@@ -2204,7 +2164,6 @@ NTSTATUS rpc_printer_migrate_settings_internals(const DOM_SID *domain_sid, const
 	char *devicename = NULL, *unc_name = NULL, *url = NULL;
 	fstring longname;
 
-	const char *keyname = NULL;
 	uint16 *keylist = NULL, *curkey;
 
 	ZERO_STRUCT(ctr_enum);
@@ -2212,7 +2171,7 @@ NTSTATUS rpc_printer_migrate_settings_internals(const DOM_SID *domain_sid, const
 	DEBUG(3,("copying printer settings\n"));
 
 	/* connect destination PI_SPOOLSS */
-	nt_status = connect_pipe(&cli_dst, PI_SPOOLSS, &got_dst_spoolss_pipe);
+	nt_status = connect_dst_pipe(&cli_dst, PI_SPOOLSS, &got_dst_spoolss_pipe);
 	if (!NT_STATUS_IS_OK(nt_status))
 		return nt_status;
 
@@ -2241,7 +2200,7 @@ NTSTATUS rpc_printer_migrate_settings_internals(const DOM_SID *domain_sid, const
 			sizeof(printername), -1, STR_TERMINATE);
 		rpcstr_pull(sharename, ctr_enum.printers_2[i].sharename.buffer, 
 			sizeof(sharename), -1, STR_TERMINATE);
-		keyname = "";
+		
 		/* we can reset NT_STATUS here because we do not 
 		   get any real NT_STATUS-codes anymore from now on */
 		nt_status = NT_STATUS_UNSUCCESSFUL;
@@ -2297,42 +2256,40 @@ NTSTATUS rpc_printer_migrate_settings_internals(const DOM_SID *domain_sid, const
 			DEBUG(3,("republished printer\n"));
 		}
 
-		/* copy devmode (info level 2) */
-		ctr_dst.printers_2->devmode = TALLOC_MEMDUP(mem_ctx, 
-			ctr_enum.printers_2[i].devmode, sizeof(DEVICEMODE));
+		if (ctr_enum.printers_2[i].devmode != NULL) {
 
-		/* do not copy security descriptor (we have another command for that) */
-		ctr_dst.printers_2->secdesc = NULL;
+			/* copy devmode (info level 2) */
+			ctr_dst.printers_2->devmode =
+				TALLOC_MEMDUP(mem_ctx,
+					      ctr_enum.printers_2[i].devmode,
+					      sizeof(DEVICEMODE));
 
-
-		/* devmode->devicename is possibly broken at the moment for all 
-		   strlen(longprinternames) > MAXDEVICENAME (that is 32 chars) 
-		   this fires up thousands of safe_strncpy-debug0-messages 
-		   on my test-servers
-		   TODO: tell jerry, jra, etc. again.
-		*/
+			/* do not copy security descriptor (we have another
+			 * command for that) */
+			ctr_dst.printers_2->secdesc = NULL;
 
 #if 0
-		if (asprintf(&devicename, "\\\\%s\\%s", longname, printername) < 0) {
-			nt_status = NT_STATUS_NO_MEMORY;
-			goto done;
-		}
+			if (asprintf(&devicename, "\\\\%s\\%s", longname,
+				     printername) < 0) {
+				nt_status = NT_STATUS_NO_MEMORY;
+				goto done;
+			}
 
-		init_unistr(&ctr_dst.printers_2->devmode->devicename, devicename); 
+			init_unistr(&ctr_dst.printers_2->devmode->devicename,
+				    devicename); 
 #endif
-		if (!net_spoolss_setprinter(cli_dst, mem_ctx, &hnd_dst, 
-						level, &ctr_dst)) 
-			goto done;
+			if (!net_spoolss_setprinter(cli_dst, mem_ctx, &hnd_dst,
+						    level, &ctr_dst)) 
+				goto done;
 		
-		DEBUGADD(1,("\tSetPrinter of DEVICEMODE succeeded\n"));
-
-
+			DEBUGADD(1,("\tSetPrinter of DEVICEMODE succeeded\n"));
+		}
 
 		/* STEP 2: COPY REGISTRY VALUES */
 	
 		/* please keep in mind that samba parse_spools gives horribly 
 		   crippled results when used to cli_spoolss_enumprinterdataex 
-		   a win2k3-server.  
+		   a win2k3-server.  (Bugzilla #1851)
 		   FIXME: IIRC I've seen it too on a win2k-server 
 		*/
 
@@ -2340,7 +2297,7 @@ NTSTATUS rpc_printer_migrate_settings_internals(const DOM_SID *domain_sid, const
 		result = cli_spoolss_enumprinterdata(cli, mem_ctx, &hnd_src, p, 0, 0,
 			&val_needed, &data_needed, NULL);
 
-		/* loop for all printerdata */
+		/* loop for all printerdata of "PrinterDriverData" */
 		while (W_ERROR_IS_OK(result)) {
 			
 			REGISTRY_VALUE value;
@@ -2354,7 +2311,7 @@ NTSTATUS rpc_printer_migrate_settings_internals(const DOM_SID *domain_sid, const
 
 				/* display_value */
 				if (opt_verbose) 
-					display_reg_value(NULL, value);
+					display_reg_value(SPOOL_PRINTERDATA_KEY, value);
 
 				/* set_value */
 				if (!net_spoolss_setprinterdata(cli_dst, mem_ctx, 
@@ -2373,7 +2330,7 @@ NTSTATUS rpc_printer_migrate_settings_internals(const DOM_SID *domain_sid, const
 		   respond to enumprinterkey, win2k does, so continue 
 		   in case of an error */
 
-		if (!net_spoolss_enumprinterkey(cli, mem_ctx, &hnd_src, keyname, &keylist)) {
+		if (!net_spoolss_enumprinterkey(cli, mem_ctx, &hnd_src, "", &keylist)) {
 			printf("got no key-data\n");
 			continue;
 		}
