@@ -31,12 +31,10 @@
  * This program reports current SMB connections
  */
 
-#define NO_SYSLOG
-
 #include "includes.h"
 
 #define SMB_MAXPIDS		2048
-static pstring 		Ucrit_username = "";               /* added by OH */
+static uid_t 		Ucrit_uid = 0;               /* added by OH */
 static pid_t		Ucrit_pid[SMB_MAXPIDS];  /* Ugly !!! */   /* added by OH */
 static int		Ucrit_MaxPid=0;                    /* added by OH */
 static unsigned int	Ucrit_IsActive = 0;                /* added by OH */
@@ -46,24 +44,23 @@ static int            shares_only = 0;            /* Added by RJS */
 static int            locks_only  = 0;            /* Added by RJS */
 static BOOL processes_only=False;
 static int show_brl;
+static BOOL numeric_only = False;
 
 const char *username = NULL;
 
 /* added by OH */
-static void Ucrit_addUsername(const char *username)
+static void Ucrit_addUid(uid_t uid)
 {
-	pstrcpy(Ucrit_username, username);
-	
-	if ( strlen(Ucrit_username) > 0 )
-		Ucrit_IsActive = 1;
+	Ucrit_uid = uid;
+	Ucrit_IsActive = 1;
 }
 
-static unsigned int Ucrit_checkUsername(const char *username)
+static unsigned int Ucrit_checkUid(uid_t uid)
 {
 	if ( !Ucrit_IsActive ) 
 		return 1;
 	
-	if ( strcmp(Ucrit_username,username) == 0 ) 
+	if ( uid == Ucrit_uid ) 
 		return 1;
 	
 	return 0;
@@ -91,7 +88,7 @@ static BOOL Ucrit_addPid( pid_t pid )
 
 	if ( Ucrit_MaxPid >= SMB_MAXPIDS ) {
 		d_printf("ERROR: More than %d pids for user %s!\n",
-			SMB_MAXPIDS, Ucrit_username);
+			 SMB_MAXPIDS, uidtoname(Ucrit_uid));
 
 		return False;
 	}
@@ -112,37 +109,47 @@ static void print_share_mode(share_mode_entry *e, char *fname)
 	count++;
 
 	if (Ucrit_checkPid(e->pid)) {
-          d_printf("%-5d  ",(int)e->pid);
-	  switch (GET_DENY_MODE(e->share_mode)) {
-	  case DENY_NONE: d_printf("DENY_NONE  "); break;
-	  case DENY_ALL:  d_printf("DENY_ALL   "); break;
-	  case DENY_DOS:  d_printf("DENY_DOS   "); break;
-	  case DENY_READ: d_printf("DENY_READ  "); break;
-	  case DENY_WRITE:printf("DENY_WRITE "); break;
-	  case DENY_FCB:  d_printf("DENY_FCB "); break;
-	  }
-	  d_printf("0x%-8x  ",(unsigned int)e->desired_access);
-	  switch (e->share_mode&0xF) {
-	  case 0: d_printf("RDONLY     "); break;
-	  case 1: d_printf("WRONLY     "); break;
-	  case 2: d_printf("RDWR       "); break;
-	  }
+		d_printf("%-5d  ",(int)e->pid);
+		switch (map_share_mode_to_deny_mode(e->share_access,
+						    e->private_options)) {
+			case DENY_NONE: d_printf("DENY_NONE  "); break;
+			case DENY_ALL:  d_printf("DENY_ALL   "); break;
+			case DENY_DOS:  d_printf("DENY_DOS   "); break;
+			case DENY_READ: d_printf("DENY_READ  "); break;
+			case DENY_WRITE:printf("DENY_WRITE "); break;
+			case DENY_FCB:  d_printf("DENY_FCB "); break;
+			default: {
+				d_printf("unknown-please report ! "
+					 "e->share_access = 0x%x, "
+					 "e->private_options = 0x%x\n",
+					 (unsigned int)e->share_access,
+					 (unsigned int)e->private_options );
+				break;
+			}
+		}
+		d_printf("0x%-8x  ",(unsigned int)e->access_mask);
+		if (e->access_mask & (FILE_READ_DATA|FILE_WRITE_DATA)) {
+			d_printf("RDWR       ");
+		} else if (e->access_mask & FILE_WRITE_DATA) {
+			d_printf("WRONLY     ");
+		} else {
+			d_printf("RDONLY     ");
+		}
 
-	  if((e->op_type & 
-	     (EXCLUSIVE_OPLOCK|BATCH_OPLOCK)) == 
-	      (EXCLUSIVE_OPLOCK|BATCH_OPLOCK))
-		d_printf("EXCLUSIVE+BATCH ");
-	  else if (e->op_type & EXCLUSIVE_OPLOCK)
-		d_printf("EXCLUSIVE       ");
-	  else if (e->op_type & BATCH_OPLOCK)
-		d_printf("BATCH           ");
-	  else if (e->op_type & LEVEL_II_OPLOCK)
-		d_printf("LEVEL_II        ");
-	  else
-		d_printf("NONE            ");
+		if((e->op_type & (EXCLUSIVE_OPLOCK|BATCH_OPLOCK)) == 
+					(EXCLUSIVE_OPLOCK|BATCH_OPLOCK)) {
+			d_printf("EXCLUSIVE+BATCH ");
+		} else if (e->op_type & EXCLUSIVE_OPLOCK) {
+			d_printf("EXCLUSIVE       ");
+		} else if (e->op_type & BATCH_OPLOCK) {
+			d_printf("BATCH           ");
+		} else if (e->op_type & LEVEL_II_OPLOCK) {
+			d_printf("LEVEL_II        ");
+		} else {
+			d_printf("NONE            ");
+		}
 
-	  d_printf(" %s   %s",fname,
-             asctime(LocalTime((time_t *)&e->time.tv_sec)));
+		d_printf(" %s   %s",fname, asctime(LocalTime((time_t *)&e->time.tv_sec)));
 	}
 }
 
@@ -538,11 +545,11 @@ static int traverse_fn1(TDB_CONTEXT *tdb, TDB_DATA kbuf, TDB_DATA dbuf, void *st
 	if (crec.cnum == -1)
 		return 0;
 
-	if (!process_exists(crec.pid) || !Ucrit_checkUsername(uidtoname(crec.uid))) {
+	if (!process_exists(crec.pid) || !Ucrit_checkUid(crec.uid)) {
 		return 0;
 	}
 
-	d_printf("%-10.10s   %5d   %-12s  %s",
+	d_printf("%-10s   %5d   %-12s  %s",
 	       crec.name,(int)crec.pid,
 	       crec.machine,
 	       asctime(LocalTime(&crec.start)));
@@ -553,21 +560,27 @@ static int traverse_fn1(TDB_CONTEXT *tdb, TDB_DATA kbuf, TDB_DATA dbuf, void *st
 static int traverse_sessionid(TDB_CONTEXT *tdb, TDB_DATA kbuf, TDB_DATA dbuf, void *state)
 {
 	struct sessionid sessionid;
+	fstring uid_str, gid_str;
 
 	if (dbuf.dsize != sizeof(sessionid))
 		return 0;
 
 	memcpy(&sessionid, dbuf.dptr, sizeof(sessionid));
 
-	if (!process_exists(sessionid.pid) || !Ucrit_checkUsername(uidtoname(sessionid.uid))) {
+	if (!process_exists(sessionid.pid) || !Ucrit_checkUid(sessionid.uid)) {
 		return 0;
 	}
 
 	Ucrit_addPid( sessionid.pid );
 
+	fstr_sprintf(uid_str, "%d", sessionid.uid);
+	fstr_sprintf(gid_str, "%d", sessionid.gid);
+
 	d_printf("%5d   %-12s  %-12s  %-12s (%s)\n",
-	       (int)sessionid.pid, uidtoname(sessionid.uid), gidtoname(sessionid.gid), 
-	       sessionid.remote_machine, sessionid.hostname);
+		 (int)sessionid.pid,
+		 numeric_only ? uid_str : uidtoname(sessionid.uid),
+		 numeric_only ? gid_str : gidtoname(sessionid.gid), 
+		 sessionid.remote_machine, sessionid.hostname);
 	
 	return 0;
 }
@@ -594,6 +607,7 @@ static int traverse_sessionid(TDB_CONTEXT *tdb, TDB_DATA kbuf, TDB_DATA dbuf, vo
 		{"profile",	'P', POPT_ARG_NONE,	&profile_only, 'P', "Do profiling" },
 #endif /* WITH_PROFILE */
 		{"byterange",	'B', POPT_ARG_NONE,	&show_brl, 'B', "Include byte range locks"},
+		{"numeric",	'n', POPT_ARG_NONE,	&numeric_only, 'n', "Numeric uid/gid"},
 		POPT_COMMON_SAMBA
 		POPT_TABLEEND
 	};
@@ -613,7 +627,7 @@ static int traverse_sessionid(TDB_CONTEXT *tdb, TDB_DATA kbuf, TDB_DATA dbuf, vo
 	while ((c = poptGetNextOpt(pc)) != -1) {
 		switch (c) {
 		case 'u':                                      
-			Ucrit_addUsername(poptGetOptArg(pc));             
+			Ucrit_addUid(nametouid(poptGetOptArg(pc)));
 			break;
 		}
 	}
@@ -625,7 +639,7 @@ static int traverse_sessionid(TDB_CONTEXT *tdb, TDB_DATA kbuf, TDB_DATA dbuf, vo
 	show_shares    = !(processes_only || locks_only || profile_only) || shares_only;
 
 	if ( username )
-		Ucrit_addUsername( username );
+		Ucrit_addUid( nametouid(username) );
 
 	if (verbose) {
 		d_printf("using configfile = %s\n", dyn_CONFIGFILE);
